/*
 * Copyright (C) 1998 Stephen Chenney and Jeffrey Ichnowski
 *
 * This program is free software.  You can redistribute it and/or
 * modify it under the terms of the GNU General Public License version
 * 2 as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
import vrml.*;
import vrml.field.*;
import vrml.node.*;
import berkeley.cs.dmc.bspline.BSpline;

public class TrackControl extends Script {
    private MFNode  newGeometry = null;
    private MFNode  controlPts = null;
    private MFNode  controlWidgets = null;
    private SFNode  self = null;
    private Browser browser = null;

    private BaseNode[]  transNodes = null;
    private BaseNode[]  orientNodes = null;
    private BaseNode[]  posnNodes = null;
    private BaseNode[]  addTrans = null;
    private BaseNode[]  liftTrans = null;

    private boolean[] liftStatus = null;

    private int last_changed = 0;

    private BSpline pt_spline = new BSpline(3, 0, null);
    private BSpline norm_spline = new BSpline(3, 0, null);

    private String eoln = System.getProperty("line.separator");

    private String sphere_1_text =
	"#VRML V2.0 utf8" + eoln
      + "Transform { translation ";
    private String sphere_2_text = eoln
      + " children [" + eoln
      + "  SphereSensor { }" + eoln
      + "  Transform { children [" + eoln
      + "  Shape {" + eoln
      + "    geometry IndexedLineSet {" + eoln
      + "      coord Coordinate { point [ 0.0 0.0 0.0 0.0 2.0 0.0 ] }" + eoln
      + "      coordIndex [ 0 1 ]" + eoln
      + "    }" + eoln
      + "    appearance Appearance { material Material {" + eoln
      + "      emissiveColor 0.4 0.0 0.0" + eoln
      + "      diffuseColor 0.0 0.0 0.0" + eoln
      + "      ambientIntensity 0.0" + eoln
      + "    } }" + eoln
      + "  }" + eoln
      + "  Shape {" + eoln
      + "    geometry Sphere { radius 0.5 }" + eoln
      + "    appearance Appearance { material Material {" + eoln
      + "      diffuseColor 0.8 0.0 0.0" + eoln
      + "    } }" + eoln
      + "  }" + eoln
      + " ] }" + eoln
      + "] }";


    private String posn_1_text =
	"#VRML V2.0 utf8" + eoln
      + "Group { children [" + eoln
      + " DEF scr Script {" + eoln
      + "  url [ \"TranslateControl.class\" ]" + eoln
      + "  field SFInt32 num ";

    private String posn_2_text = eoln
      + "  eventIn SFInt32 set_num" + eoln
      + "  eventIn SFBool sensorActive" + eoln
      + "  eventIn SFVec3f xy_changed" + eoln
      + "  eventIn SFVec3f xz_changed" + eoln
      + "  eventIn SFVec3f yz_changed" + eoln
      + "  eventIn SFRotation orient_changed" + eoln
      + "  eventIn SFTime  addTouch" + eoln
      + "  eventIn SFTime  liftTouch" + eoln
      + "  eventOut SFVec3f trans_set" + eoln
      + "  eventOut SFVec3f lag_set" + eoln
      + "  eventOut SFInt32 controlChanged" + eoln
      + "  eventOut SFInt32 addEvent" + eoln
      + "  eventOut SFInt32 liftEvent" + eoln
      + " }" + eoln
      + " Transform { children [" + eoln
      + "  DEF addSense TouchSensor { }" + eoln
      + "  Shape {" + eoln
      + "   geometry Sphere { radius 0.2 }" + eoln
      + "   appearance Appearance { material Material {" + eoln
      + "     diffuseColor 0.0 0.8 0.0" + eoln
      + "   } } }" + eoln
      + " ] }" + eoln
      + " Transform { children [" + eoln
      + "  DEF liftSense TouchSensor { }" + eoln
      + "  Shape {" + eoln
      + "   geometry Sphere { radius 0.2 }" + eoln
      + "   appearance Appearance { material Material {" + eoln
      + "     diffuseColor 0.0 0.0 0.2" + eoln
      + "   } } }" + eoln
      + " ] }" + eoln
      + " DEF xy Transform { children [" + eoln
      + "  DEF xyp PlaneSensor { autoOffset FALSE }" + eoln
      + "  Shape {" + eoln
      + "    geometry IndexedFaceSet {" + eoln
      + "      solid FALSE" + eoln
      + "      coord Coordinate { point [ 1.0  1.0 0.0" + eoln
      + "                                -1.0  1.0 0.0" + eoln
      + "                                -1.0 -1.0 0.0" + eoln
      + "                                 1.0 -1.0 0.0 ] }" + eoln
      + "      coordIndex [ 0 1 2 3 ] }" + eoln
      + "    appearance Appearance { material Material {" + eoln
      + "      emissiveColor 0.7 0.7 0.7" + eoln
      + "      diffuseColor 0.0 0.0 0.0" + eoln
      + "      ambientIntensity 0.0" + eoln
      + "    } }" + eoln
      + "  } ] }" + eoln
      + " DEF xz Transform { rotation 1.0 0.0 0.0 1.57 children [" + eoln
      + "  DEF xzp PlaneSensor { autoOffset FALSE }" + eoln
      + "  Shape {" + eoln
      + "    geometry IndexedFaceSet {" + eoln
      + "      solid FALSE" + eoln
      + "      coord Coordinate { point [ 1.0  1.0 0.0" + eoln
      + "                                -1.0  1.0 0.0" + eoln
      + "                                -1.0 -1.0 0.0" + eoln
      + "                                 1.0 -1.0 0.0 ] }" + eoln
      + "      coordIndex [ 0 1 2 3 ] }" + eoln
      + "    appearance Appearance { material Material {" + eoln
      + "      emissiveColor 1.0 1.0 1.0" + eoln
      + "      diffuseColor 0.0 0.0 0.0" + eoln
      + "      ambientIntensity 0.0" + eoln
      + "    } }" + eoln
      + "  } ] }" + eoln
      + " DEF yz Transform { rotation 0.0 -1.0 0.0 1.57 children [" + eoln
      + "  DEF yzp PlaneSensor { autoOffset FALSE }" + eoln
      + "  Shape {" + eoln
      + "    geometry IndexedFaceSet {" + eoln
      + "      solid FALSE" + eoln
      + "      coord Coordinate { point [ 1.0  1.0 0.0" + eoln
      + "                                -1.0  1.0 0.0" + eoln
      + "                                -1.0 -1.0 0.0" + eoln
      + "                                 1.0 -1.0 0.0 ] }" + eoln
      + "      coordIndex [ 0 1 2 3 ] }" + eoln
      + "    appearance Appearance { material Material {" + eoln
      + "      emissiveColor 0.5 0.5 0.5" + eoln
      + "      diffuseColor 0.0 0.0 0.0" + eoln
      + "      ambientIntensity 0.0" + eoln
      + "    } }" + eoln
      + "  } ] } ] }" + eoln
      + "ROUTE addSense.touchTime TO scr.addTouch" + eoln
      + "ROUTE liftSense.touchTime TO scr.liftTouch" + eoln
      + "ROUTE scr.lag_set TO xy.set_translation" + eoln
      + "ROUTE scr.lag_set TO yz.set_translation" + eoln
      + "ROUTE scr.lag_set TO xz.set_translation" + eoln
      + "ROUTE xyp.isActive TO scr.sensorActive" + eoln
      + "ROUTE xzp.isActive TO scr.sensorActive" + eoln
      + "ROUTE yzp.isActive TO scr.sensorActive" + eoln
      + "ROUTE xyp.translation_changed TO scr.xy_changed" + eoln
      + "ROUTE yzp.translation_changed TO scr.yz_changed" + eoln
      + "ROUTE xzp.translation_changed TO scr.xz_changed" + eoln;


    public void initialize() {
      try {
	browser = this.getBrowser();
	newGeometry = (MFNode)((Node)((SFNode)getField("newGeometry"))
				.getValue()).getExposedField("children");
	controlPts = (MFNode)((Node)((SFNode)getField("controlPts"))
				.getValue()).getExposedField("children");
	controlWidgets = (MFNode)((Node)((SFNode)getField("controlWdgts"))
				.getValue()).getExposedField("children");
	self = (SFNode)getField("self");

	float[] posn = new float[3];

	posn[0] = 21.518963f; posn[1] = 1.0f; posn[2] = 4.427049f;
	createControlPt(0, posn);
	posn[0] = 14.042409f; posn[1] = 1.0413438f; posn[2] = 5.579808f;
	createControlPt(1, posn);
	posn[0] = 14.0492325f; posn[1] = 1.0198106f; posn[2] = -1.4436634f;
	createControlPt(2, posn);
	posn[0] = 7.7339516f; posn[1] = 9.219274f; posn[2] = -0.8862512f;
	createControlPt(3, posn);
	posn[0] = 0.02485007f; posn[1] = 6.30905f; posn[2] = -1.0354645f;
	createControlPt(4, posn);
	posn[0] = -6.8167505f; posn[1] = 4.522277f; posn[2] = -0.6704053f;
	createControlPt(5, posn);
	posn[0] = -14.109677f; posn[1] = 2.068463f; posn[2] = -0.5064248f;
	createControlPt(6, posn);
	posn[0] = -20.632145f; posn[1] = 1.3742733f; posn[2] = 1.5510848f;
	createControlPt(7, posn);
	posn[0] = -19.381033f; posn[1] = 1.0f; posn[2] = 9.507294f;
	createControlPt(8, posn);
	posn[0] = -14.168044f; posn[1] = 1.2612991f; posn[2] = 12.104755f;
	createControlPt(9, posn);
	posn[0] = -5.849577f; posn[1] = 12.569652f; posn[2] = 11.43606f;
	createControlPt(10, posn);
	posn[0] = -1.3165593f; posn[1] = 0.99716485f; posn[2] = 11.393228f;
	createControlPt(11, posn);
	posn[0] = 2.6194885f; posn[1] = 1.1348124f; posn[2] = 11.329943f;
	createControlPt(12, posn);
	posn[0] = 7.791444f; posn[1] = 6.366364f; posn[2] = 11.002439f;
	createControlPt(13, posn);
	posn[0] = 14.794787f; posn[1] = 1.3413737f; posn[2] = 10.4128685f;
	createControlPt(14, posn);
	posn[0] = 21.290281f; posn[1] = 2.9845026f; posn[2] = 10.452518f;
	createControlPt(15, posn);
/*
	posn[0] = 5.0f; posn[1] = 1.0f; posn[2] = 5.0f;
	posn[0] = -5.0f; posn[1] = 1.0f; posn[2] = 5.0f;
	createControlPt(1, posn);
	posn[0] = -5.0f; posn[1] = 1.0f; posn[2] = -5.0f;
	createControlPt(2, posn);
	posn[0] = 5.0f; posn[1] = 1.0f; posn[2] = -5.0f;
	createControlPt(3, posn);
*/
      } catch (Exception e) {
	System.err.println(e);
	e.printStackTrace();
      }
    }


    public void processEvent(Event e) {
	String e_name = e.getName();

	if ( e_name.equals("controlChanged") )
	{
	    last_changed = ((ConstSFInt32)e.getValue()).getValue();
	    setGeometry();
	}
	else if ( e_name.equals("addNode") )
	{
	    int new_loc = ((ConstSFInt32)e.getValue()).getValue();
	    float[] posn = new float[3];
	    ((SFVec3f)((Node)addTrans[new_loc]).getExposedField("translation"))
	                               .getValue(posn);
	    createControlPt(new_loc, posn);
	}
	else if ( e_name.equals("liftToggle") )
	{
	    int new_loc = ((ConstSFInt32)e.getValue()).getValue();
	    liftStatus[new_loc] = ! liftStatus[new_loc];

	    Node node = (Node)((MFNode)((Node)liftTrans[new_loc])
	                                           .getExposedField("children"))
						   .get1Value(1);
	    node =(Node)((SFNode)node.getExposedField("appearance")).getValue();
	    node =(Node)((SFNode)node.getExposedField("material")).getValue();
	    float[] color = new float[3];
	    if ( liftStatus[new_loc] )
		color[2] = 0.8f;
	    else
		color[2] = 0.2f;
	    ((SFColor)node.getExposedField("diffuseColor")).setValue(color);
	}
	else if ( e_name.equals("save") )
	    saveTrack();

	Runtime rt = Runtime.getRuntime();
	long isFree = rt.freeMemory();
	long wasFree;
	do {
	    wasFree = isFree;
	    rt.gc();
	    isFree = rt.freeMemory();
	} while ( isFree > wasFree );
	rt.runFinalization();
    }


    private void rotationToNormal(float[] rotation, float[] norm) {
	float sin = (float)Math.sin((double)rotation[3]);
	float cos = (float)Math.cos((double)rotation[3]);
	float vers = 1.0f - cos;
	float length = (float)Math.sqrt((double)(rotation[0] * rotation[0]
	                                       + rotation[1] * rotation[1]
	                                       + rotation[2] * rotation[2]));
	rotation[0] /= length;
	rotation[1] /= length;
	rotation[2] /= length;

	norm[0] = rotation[1] * rotation[0] * vers - rotation[2] * sin;
	norm[1] = rotation[1] * rotation[1] * vers + cos;
	norm[2] = rotation[1] * rotation[2] * vers + rotation[0] * sin;
    }


    private void setControlPts(int num_controls) {
	float[][]   pt_control_pts = new float[num_controls + 3][];
	float[][]   norm_control_pts = new float[num_controls + 3][];
	float[]     rotation = new float[4];

	try {

	for ( int i = 0 ; i < num_controls ; i++ )
	{
	    pt_control_pts[i] = new float[3];
	    norm_control_pts[i] = new float[3];
	    ((SFVec3f)((Node)transNodes[i]).getExposedField("translation"))
	                                   .getValue(pt_control_pts[i]);
	    ((SFRotation)((Node)orientNodes[i]).getExposedField("rotation"))
	                                       .getValue(rotation);
	    rotationToNormal(rotation, norm_control_pts[i]);
	}
	for ( int i = 0 ; i < 3 ; i++ )
	{
	    pt_control_pts[num_controls + i] = new float[3];
	    norm_control_pts[num_controls + i] = new float[3];
	    ((SFVec3f)((Node)transNodes[i]).getExposedField("translation"))
		                   .getValue(pt_control_pts[num_controls + i]);
	    ((SFRotation)((Node)orientNodes[i]).getExposedField("rotation"))
	                                       .getValue(rotation);
	    rotationToNormal(rotation, norm_control_pts[num_controls + i]);
	}

	pt_spline.newControls(num_controls + 3, pt_control_pts);
	norm_spline.newControls(num_controls + 3, norm_control_pts);

	} catch (Exception e) {
	    System.err.println(e);
	    e.printStackTrace();
	}
    }


    private void saveTrack() {
	int num_controls = transNodes.length;
	float[] pt = new float[3];
	float[] norm = new float[3];
	float[] rotation = new float[4];
	
	System.out.println(num_controls);

	for ( int i = 0 ; i < num_controls ; i++ )
	{
	    ((SFVec3f)((Node)transNodes[i]).getExposedField("translation"))
					   .getValue(pt);
	    ((SFRotation)((Node)orientNodes[i]).getExposedField("rotation"))
					       .getValue(rotation);
	    rotationToNormal(rotation, norm);

	    System.out.println(pt[0] + " " + pt[1] + " " + pt[2] + " " + norm[0] + " " + norm[1] + " " + norm[2]);
	}

	int count = 0;
	for ( int i = 0 ; i < num_controls ; i++ )
	{
	    if ( liftStatus[i] )
		count++;
	}
	System.out.println(count);
	for ( int i = 0 ; i < num_controls ; i++ )
	{
	    if ( liftStatus[i] )
		System.out.println(i + " " + ( i + 1 ) + " 1.0");
	}
    }


    private void setGeometry() {
	int num_controls = transNodes.length;

	if ( num_controls < 4 )
	    return;

	setControlPts(num_controls);

	String  track_text = "#VRML V2.0 utf8" + eoln
                           + "Group { children [" + eoln
			   + "  Shape { geometry IndexedLineSet {" + eoln
			   + "    coord Coordinate { point [";


	float[] pt = new float[3];
	float[] norm = new float[3];
	int     count = 0;
	for ( float v = 0.0f ; v < num_controls - 0.01f ; v += 0.1f )
	{
	  try {
	    pt_spline.evaluate(v, pt);
	    track_text = track_text + pt[0]+" "+pt[1]+" "+pt[2]+eoln;
	    count++;
	  } catch (ArithmeticException e) {
	    System.err.println(e);
	  }
	}
	track_text = track_text + "] } coordIndex [";

	for ( int i = 0 ; i < count ; i++ )
	    track_text = track_text + " " + i;
	track_text = track_text + " 0 ] }" + eoln
	           +" appearance Appearance { material Material {" + eoln
	           +"   emissiveColor 0.0 0.0 0.8 } } }" + eoln;

	track_text = track_text + "  Shape { geometry IndexedLineSet {" + eoln
	                        + "    coord Coordinate { point [";
	for ( float v = 0.0f ; v < num_controls - 0.01f ; v += 0.1f )
	{
	  try {
	    pt_spline.evaluate(v, pt);
	    track_text = track_text + pt[0] + " " + pt[1] + " -99.0" + eoln;
	  } catch (ArithmeticException e) {
	    System.err.println(e);
	  }
	}
	track_text = track_text + "] } coordIndex [";

	for ( int i = 0 ; i < count ; i++ )
	    track_text = track_text + " " + i;
	track_text = track_text + " 0 ] }" + eoln
	           +" appearance Appearance { material Material {" + eoln
	           +"   emissiveColor 0.0 0.0 0.8 } } }" + eoln;

	track_text = track_text + "  Shape { geometry IndexedLineSet {" + eoln
	                        + "    coord Coordinate { point [";
	for ( float v = 0.0f ; v < num_controls - 0.01f ; v += 0.1f )
	{
	  try {
	    pt_spline.evaluate(v, pt);
	    track_text = track_text + " -99.0 " + pt[1] + " " + pt[2] + eoln;
	  } catch (ArithmeticException e) {
	    System.err.println(e);
	  }
	}
	track_text = track_text + "] } coordIndex [";

	for ( int i = 0 ; i < count ; i++ )
	    track_text = track_text + " " + i;
	track_text = track_text + " 0 ] }" + eoln
	           +" appearance Appearance { material Material {" + eoln
	           +"   emissiveColor 0.0 0.0 0.8 } } }" + eoln;

	track_text = track_text + "  Shape { geometry IndexedLineSet {" + eoln
	                        + "    coord Coordinate { point [";
	for ( float v = 0.0f ; v < num_controls - 0.01f ; v += 0.1f )
	{
	  try {
	    pt_spline.evaluate(v, pt);
	    track_text = track_text + " " + pt[0] + " 0.1 " + pt[2] + eoln;
	  } catch (ArithmeticException e) {
	    System.err.println(e);
	  }
	}
	track_text = track_text + "] } coordIndex [";

	for ( int i = 0 ; i < count ; i++ )
	    track_text = track_text + " " + i;
	track_text = track_text + " 0 ] }" + eoln
	           +" appearance Appearance { material Material {" + eoln
	           +"   emissiveColor 0.0 0.0 0.8 } } }" + eoln;


	for ( float v = 0.0f ; v < num_controls - 0.01f ; v += 0.2f )
	{
	  try {
	    pt_spline.evaluate(v, pt);
	    norm_spline.evaluate(v, norm);
	    norm[0] += pt[0];
	    norm[1] += pt[1];
	    norm[2] += pt[2];

	    track_text += "  Shape { geometry IndexedLineSet {" + eoln
	                + "    coord Coordinate { point [" + eoln
			+ "      " + pt[0] + " " + pt[1] + " " + pt[2] + eoln
			+ "      " + norm[0] +" "+ norm[1] +" "+ norm[2] + eoln
			+ "    ] } coordIndex [ 0 1 ] }" + eoln
		        +" appearance Appearance { material Material {" + eoln
		        +"   emissiveColor 0.0 0.0 0.5 } } }" + eoln;

	  } catch (Exception e) {
	    System.err.println("Line 399");
	    System.err.println(e);
	  }
	}

	track_text = track_text + "] }";

	BaseNode[] geom_node = null;

	try {
	    geom_node = browser.createVrmlFromString(track_text);
	    newGeometry.setValue(geom_node);
	} catch (Exception ex) {
	    System.err.println(ex);
	    ex.printStackTrace();
	    System.err.println(track_text);
	}

	for ( float v = 0.5f ; v < num_controls ; v += 1.0f )
	{
	  try {
	    pt_spline.evaluate(v, pt);
	    int i = (int)Math.floor(v / 1.0) + 2;
	    if ( i >= transNodes.length )
		i -= transNodes.length;
	    ((SFVec3f)((Node)addTrans[i]).getExposedField("translation"))
					 .setValue(pt);
	  } catch (Exception e) {
	    System.err.println(e);
	    e.printStackTrace();
	  }
	}

	for ( float v = 0.25f ; v < num_controls ; v += 1.0f )
	{
	  try {
	    pt_spline.evaluate(v, pt);
	    int i = (int)Math.floor(v / 1.0);
	    ((SFVec3f)((Node)liftTrans[i]).getExposedField("translation"))
					  .setValue(pt);
	  } catch (Exception e) {
	    System.err.println(e);
	    e.printStackTrace();
	  }
	}
    }


    private void createControlPt(int num, float[] posn) {
	Node   posn_node = null;
	Node   sphere_node = null;
	Node   sensor_node = null;
	Node   normal_node = null;
	Node   script_node = null;
	Node   trans_node = null;
	MFNode children = null;
	try {
	    int num_controls = ( transNodes==null ? 0 : transNodes.length);

	    posn_node = (Node)browser.createVrmlFromString(posn_1_text
		    + num + eoln + "field SFVec3f location " + posn[0]
		    + " " + posn[1] + " " + posn[2] + posn_2_text)[0];
	    script_node = (Node)((MFNode)posn_node.getExposedField("children"))
						  .get1Value(0);

	    if ( posnNodes != null )
	    {
		BaseNode[] new_nodes = new BaseNode[posnNodes.length + 1];
		for ( int i = 0 ; i < num ; i++ )
		    new_nodes[i] = posnNodes[i];
		new_nodes[num] = (BaseNode)posn_node;
		for ( int i = num ; i < posnNodes.length ; i++ )
		{
		    new_nodes[i+1] = posnNodes[i];
		    ((SFInt32)((Node)((MFNode)((Node)posnNodes[i])
				    .getExposedField("children")).get1Value(0))
				    .getEventIn("set_num")).setValue(i+1);
		}
		posnNodes = new_nodes;
	    }
	    else
	    {
	        posnNodes = new BaseNode[1];
		posnNodes[0] = (BaseNode)posn_node;
	    }

	    controlWidgets.setValue(posnNodes);

	    sphere_node = (Node)browser.createVrmlFromString(sphere_1_text
                            + posn[0] + " " + posn[1] + " " + posn[2]
			    + sphere_2_text)[0];
	    MFNode chil = (MFNode)sphere_node.getExposedField("children");
	    sensor_node = (Node)chil.get1Value(0);
	    normal_node = (Node)chil.get1Value(1);

	    if ( transNodes != null )
	    {
		BaseNode[] new_nodes = new Node[transNodes.length + 1];
		for ( int i = 0 ; i < num ; i++ )
		    new_nodes[i] = transNodes[i];
		new_nodes[num] = (BaseNode)sphere_node;
		for ( int i = num ; i < transNodes.length ; i++ )
		    new_nodes[i+1] = transNodes[i];
		transNodes = new_nodes;
	    }
	    else
	    {
	        transNodes = new BaseNode[1];
		transNodes[0] = (BaseNode)sphere_node;
	    }
	    if ( orientNodes != null )
	    {
		BaseNode[] new_nodes = new Node[orientNodes.length + 1];
		for ( int i = 0 ; i < num ; i++ )
		    new_nodes[i] = orientNodes[i];
		new_nodes[num] = (BaseNode)normal_node;
		for ( int i = num ; i < orientNodes.length ; i++ )
		    new_nodes[i+1] = orientNodes[i];
		orientNodes = new_nodes;
	    }
	    else
	    {
	        orientNodes = new BaseNode[1];
		orientNodes[0] = (BaseNode)normal_node;
	    }

	    controlPts.setValue(transNodes);

	    browser.addRoute(sensor_node, "rotation_changed",
	                     script_node, "orient_changed");
	    browser.addRoute(sensor_node, "rotation_changed",
	                     normal_node, "set_rotation");
	    browser.addRoute(script_node, "trans_set",
	                     sphere_node, "set_translation");
	    browser.addRoute(script_node, "controlChanged",
	                     self.getValue(), "controlChanged");
	    browser.addRoute(script_node, "addEvent",
	                     self.getValue(), "addNode");
	    browser.addRoute(script_node, "liftEvent",
	                     self.getValue(), "liftToggle");

	    trans_node = (Node)((MFNode)posn_node.getExposedField("children"))
						 .get1Value(2);

	    if ( liftTrans != null )
	    {
		BaseNode[] new_nodes = new BaseNode[liftTrans.length + 1];
		for ( int i = 0 ; i < num ; i++ )
		    new_nodes[i] = liftTrans[i];
		new_nodes[num] = trans_node;
		for ( int i = num ; i < liftTrans.length ; i++ )
		    new_nodes[i+1] = liftTrans[i];
		liftTrans = new_nodes;
	    }
	    else
	    {
	        liftTrans = new BaseNode[1];
		liftTrans[0] = trans_node;
	    }

	    if ( liftStatus != null )
	    {
		boolean[] new_vals = new boolean[liftStatus.length + 1];
		for ( int i = 0 ; i < num ; i++ )
		    new_vals[i] = liftStatus[i];
		new_vals[num] = false;
		for ( int i = num ; i < liftStatus.length ; i++ )
		    new_vals[i+1] = liftStatus[i];
		liftStatus = new_vals;
	    }
	    else
	    {
		liftStatus = new boolean[1];
		liftStatus[0] = false;
	    }

	    trans_node = (Node)((MFNode)posn_node.getExposedField("children"))
						 .get1Value(1);

	    if ( addTrans != null )
	    {
		BaseNode[] new_nodes = new BaseNode[addTrans.length + 1];
		for ( int i = 0 ; i < num ; i++ )
		    new_nodes[i] = addTrans[i];
		new_nodes[num] = (BaseNode)trans_node;
		for ( int i = num ; i < addTrans.length ; i++ )
		    new_nodes[i+1] = addTrans[i];
		addTrans = new_nodes;
	    }
	    else
	    {
	        addTrans = new BaseNode[1];
		addTrans[0] = (BaseNode)trans_node;
	    }

	    setGeometry();

	} catch (Exception ex) {
	    System.err.println(ex);
	    ex.printStackTrace();
	}
    }
}

