/*
 * Copyright (C) 1998 Stephen Chenney and Jeffrey Ichnowski
 *
 * This program is free software.  You can redistribute it and/or
 * modify it under the terms of the GNU General Public License version
 * 2 as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
import vrml.*;
import vrml.field.*;
import vrml.node.*;

public class TranslateControl extends Script {
    private boolean active = false;
    private float[] posn = { 0.0f, 0.0f, 0.0f };
    private float[] offset = new float[3];

    private SFVec3f posnField = null;
    private SFVec3f trans_set = null;
    private SFVec3f lag_set = null;
    private SFInt32 num = null;
    private SFInt32 changed = null;
    private SFInt32 addNode = null;
    private SFInt32 liftNode = null;


    public void initialize() {
      try {
	addNode = (SFInt32)getEventOut("addEvent");
	liftNode = (SFInt32)getEventOut("liftEvent");
	trans_set = (SFVec3f)getEventOut("trans_set");
	lag_set = (SFVec3f)getEventOut("lag_set");
	num = (SFInt32)getField("num");
	changed = (SFInt32)getEventOut("controlChanged");
	posnField = (SFVec3f)getField("location");
	posnField.getValue(posn);
	lag_set.setValue(posn);
      } catch (Exception e) {
	System.err.println(e);
	e.printStackTrace();
      }
    }

    public void processEvent(Event e) {
	String e_name = e.getName();

	if ( e_name.equals("set_num") )
	    num.setValue(((ConstSFInt32)e.getValue()).getValue());
	else if ( e_name.equals("addTouch") )
	    addNode.setValue(num.getValue());
	else if ( e_name.equals("liftTouch") )
	    liftNode.setValue(num.getValue());
	else if ( e_name.equals("sensorActive") )
	{
	    boolean new_active = ((ConstSFBool)e.getValue()).getValue();
	    if ( active )
	    {
		if ( ! new_active )
		{
		    posn[0] += offset[0];
		    posn[1] += offset[1];
		    posn[2] += offset[2];
		    lag_set.setValue(posn);
		    posnField.setValue(posn);
		    active = false;
		}
	    }
	    else
	    {
		if ( new_active )
		    active = true;
	    }
	}
	else if ( e_name.equals("xy_changed") )
	{
	    if ( active )
	    {
		((ConstSFVec3f)e.getValue()).getValue(offset);
		trans_set.setValue(posn[0]+offset[0],posn[1]+offset[1],posn[2]);
	    }
	    changed.setValue(num.getValue());
	}
	else if ( e_name.equals("xz_changed") )
	{
	    if ( active )
	    {
		((ConstSFVec3f)e.getValue()).getValue(offset);
		offset[2] = offset[1];
		offset[1] = 0.0f;
		trans_set.setValue(posn[0]+offset[0],posn[1],posn[2]+offset[2]);
	    }
	    changed.setValue(num.getValue());
	}
	else if ( e_name.equals("yz_changed") )
	{
	    if ( active )
	    {
		((ConstSFVec3f)e.getValue()).getValue(offset);
		offset[2] = offset[0];
		offset[0] = 0.0f;
		trans_set.setValue(posn[0],posn[1]+offset[1],posn[2]+offset[2]);
	    }
	    changed.setValue(num.getValue());
	}
	else if ( e_name.equals("orient_changed") )
	    changed.setValue(num.getValue());
    }
}


