/*
 * Copyright (C) 1998 Stephen Chenney and Jeffrey Ichnowski
 *
 * This program is free software.  You can redistribute it and/or
 * modify it under the terms of the GNU General Public License version
 * 2 as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
import vrml.*;
import vrml.field.*;
import vrml.node.*;
import berkeley.cs.dmc.system.DynamicsVariable;

public class DiveBomber extends Script {
  double time = 0.0;
  double last_time = 0.0;
  boolean initialized = false;
  double[] old_params = new double[2];
  double[] seen_params = new double[2];
  double[] new_params = new double[2];
  private DiveBomberVariable phi4 = null;
  private MFFloat phi4_buffer = null;
  private SFFloat phi4_lastTime = null;
  private SFFloat phi4_lastSeen = null;
  private float[] phi4_elements = new float[1];
  private boolean phi4_visible = true;
  private DiveBomberVariable phi3 = null;
  private MFFloat phi3_buffer = null;
  private SFFloat phi3_lastTime = null;
  private SFFloat phi3_lastSeen = null;
  private float[] phi3_elements = new float[1];
  private boolean phi3_visible = true;
  private DiveBomberVariable phi2 = null;
  private MFFloat phi2_buffer = null;
  private SFFloat phi2_lastTime = null;
  private SFFloat phi2_lastSeen = null;
  private float[] phi2_elements = new float[1];
  private boolean phi2_visible = true;
  private DiveBomberVariable phi1 = null;
  private MFFloat phi1_buffer = null;
  private SFFloat phi1_lastTime = null;
  private SFFloat phi1_lastSeen = null;
  private float[] phi1_elements = new float[1];
  private boolean phi1_visible = true;
  private DiveBomberTheta theta = null;
  private MFFloat theta_buffer = null;
  private SFFloat theta_lastTime = null;
  private SFFloat theta_lastSeen = null;
  private float[] theta_elements = new float[1];
  private SFRotation CarRot4_event = null;
  private SFRotation CarRot3_event = null;
  private SFRotation CarRot2_event = null;
  private SFRotation CarRot1_event = null;
  private SFRotation Theta1_event = null;
  public void initialize() {
    if ( initialized ) return;
    phi4 = new DiveBomberVariable();    phi4_buffer = (MFFloat)getField("phi4_buffer");
    phi4_lastTime = (SFFloat)getField("phi4_lastTime");
    phi4_lastSeen = (SFFloat)getField("phi4_lastSeen");
    phi3 = new DiveBomberVariable();    phi3_buffer = (MFFloat)getField("phi3_buffer");
    phi3_lastTime = (SFFloat)getField("phi3_lastTime");
    phi3_lastSeen = (SFFloat)getField("phi3_lastSeen");
    phi2 = new DiveBomberVariable();    phi2_buffer = (MFFloat)getField("phi2_buffer");
    phi2_lastTime = (SFFloat)getField("phi2_lastTime");
    phi2_lastSeen = (SFFloat)getField("phi2_lastSeen");
    phi1 = new DiveBomberVariable();    phi1_buffer = (MFFloat)getField("phi1_buffer");
    phi1_lastTime = (SFFloat)getField("phi1_lastTime");
    phi1_lastSeen = (SFFloat)getField("phi1_lastSeen");
    theta = new DiveBomberTheta();    theta_buffer = (MFFloat)getField("theta_buffer");
    theta_lastTime = (SFFloat)getField("theta_lastTime");
    theta_lastSeen = (SFFloat)getField("theta_lastSeen");
    CarRot4_event = (SFRotation)getEventOut("CarRot4");
    CarRot3_event = (SFRotation)getEventOut("CarRot3");
    CarRot2_event = (SFRotation)getEventOut("CarRot2");
    CarRot1_event = (SFRotation)getEventOut("CarRot1");
    Theta1_event = (SFRotation)getEventOut("Theta1");
    initialized = true;
  }
  public void processEvent( Event e ) {
    String name = e.getName();
    if ( "currentTime".equals( name ) ) {
      time=((ConstSFFloat)e.getValue()).getValue();
    } else if ( "BoundNode".equals( name ) ) {
      phi1_visible = (boolean)((ConstSFBool)e.getValue()).getValue();
      ((SFBool)getEventOut("visible")).setValue(phi1_visible);
      phi2_visible = (boolean)((ConstSFBool)e.getValue()).getValue();
      ((SFBool)getEventOut("visible")).setValue(phi2_visible);
      phi3_visible = (boolean)((ConstSFBool)e.getValue()).getValue();
      ((SFBool)getEventOut("visible")).setValue(phi3_visible);
      phi4_visible = (boolean)((ConstSFBool)e.getValue()).getValue();
      ((SFBool)getEventOut("visible")).setValue(phi4_visible);
    }
  }
  private void timeStep(MFFloat buffer, SFFloat lastTime,
                        SFFloat lastSeen, double bufStep, double phase,
                        DynamicsVariable var, double time, float[] fields,
                        int num_p, int num_b) {
    time += phase;

    double last_time = (double)lastTime.getValue();
    double last_seen = (double)lastSeen.getValue();

    if ( num_b == 1 )
    {
      for ( int i = 0 ; i < num_p ; i++ )
        old_params[i] = (double)buffer.get1Value(i);
      var.newState(last_time, old_params,
                   last_time, old_params,
                   time, new_params);
      for ( int i = 0 ; i < num_p ; i++ )
        buffer.set1Value(i, (float)new_params[i]);
      lastTime.setValue((float)time);
      var.evaluateFields(time, new_params, fields);
      return;
    }

    int    low_index = (int)Math.floor(time / bufStep);
    double low_time = low_index * bufStep;
    int    high_index = ( low_index + 1 ) % num_b;
    int    last_index = (int)Math.floor((last_time + 0.001) / bufStep) % num_b;
    int    seen_index = (int)Math.floor((last_seen + 0.001) / bufStep) % num_b;

    float work_done = 0.0f;

    int base = 0;

    low_index = low_index % num_b;

    base = last_index * num_p;
    for ( int i = 0 ; i < num_p ; i++ )
      old_params[i] = (double)buffer.get1Value(base + i);
    base = seen_index * num_p;
    for ( int i = 0 ; i < num_p ; i++ )
      seen_params[i] = (double)buffer.get1Value(base + i);

    if ( low_time >= last_time )
    {
      work_done = var.newState(last_time, old_params,
                               last_seen, seen_params,
                               time, new_params);
      for ( int i = 0 ; i < num_p ; i++ )
        old_params[i] = new_params[i];
      base = low_index * num_p;
      for ( int i = 0 ; i < num_p ; i++ )
        buffer.set1Value(base + i, (float)new_params[i]);
      last_time = low_time;
      last_index = low_index;
    }
    if ( low_time + bufStep > last_time )
    {
      work_done += var.newState(low_time, old_params,
                                last_seen, seen_params,
                                low_time + bufStep, new_params);
      for ( int i = 0 ; i < num_p ; i++ )
        old_params[i] = new_params[i];
      base = high_index * num_p;
      for ( int i = 0 ; i < num_p ; i++ )
        buffer.set1Value(base + i, (float)new_params[i]);
      last_time = low_time + bufStep;
      last_index = high_index;
    }


    // Guaranteed to have low and high time buffer positions filled.
    int slots_used = last_index - low_index + 1;
    if ( slots_used < 1 )
      slots_used += num_b;
    base = 0;
    while (  work_done < 1.0f && slots_used < num_b )
    {
      work_done += var.newState(last_time, old_params,
                                last_seen, seen_params,
                                last_time + bufStep, new_params);
      last_index = ( last_index + 1 ) % num_b;
      base = last_index * num_p;
      for ( int i = 0 ; i < num_p ; i++ )
        old_params[i] = new_params[i];
      for ( int i = 0 ; i < num_p ; i++ )
        buffer.set1Value(base + i, (float)new_params[i]);
      last_time += bufStep;
      slots_used++;
    }

    lastTime.setValue((float)last_time);
    lastSeen.setValue((float)(low_time + bufStep));

    double fraction = ( time - low_time ) / bufStep;
    int base1 = high_index * num_p;
    base = low_index * num_p;
    for ( int i = 0 ; i < num_p ; i++ )
      new_params[i] = fraction * buffer.get1Value(base1 + i)
                    + ( 1.0 - fraction ) * buffer.get1Value(base + i);
    var.evaluateFields(time, new_params, fields);
  }
  public void eventsProcessed() {
    if ( ! initialized ) return;
    if ( time <= last_time )
        return;
    last_time = time;
    if ( phi4_visible )
      timeStep(phi4_buffer, phi4_lastTime, phi4_lastSeen, 0.1f, 6.0f, phi4, last_time, phi4_elements, 2, 1);
    if ( phi3_visible )
      timeStep(phi3_buffer, phi3_lastTime, phi3_lastSeen, 0.1f, 2.0f, phi3, last_time, phi3_elements, 2, 1);
    if ( phi2_visible )
      timeStep(phi2_buffer, phi2_lastTime, phi2_lastSeen, 0.1f, 4.0f, phi2, last_time, phi2_elements, 2, 1);
    if ( phi1_visible )
      timeStep(phi1_buffer, phi1_lastTime, phi1_lastSeen, 0.1f, 0.0f, phi1, last_time, phi1_elements, 2, 1);
    timeStep(theta_buffer, theta_lastTime, theta_lastSeen, 0.0f, 0.0f, theta, last_time, theta_elements, 1, 1);
    if ( phi4_visible )
      CarRot4_event.setValue(0.0f, 1.0f, 0.0f, phi4_elements[0]);
    if ( phi3_visible )
      CarRot3_event.setValue(0.0f, 1.0f, 0.0f, phi3_elements[0]);
    if ( phi2_visible )
      CarRot2_event.setValue(0.0f, 1.0f, 0.0f, phi2_elements[0]);
    if ( phi1_visible )
      CarRot1_event.setValue(0.0f, 1.0f, 0.0f, phi1_elements[0]);
    Theta1_event.setValue(0.0f, 0.0f, 1.0f, theta_elements[0]);
  }
}

