/*
 * Copyright (C) 1998 Stephen Chenney and Jeffrey Ichnowski
 *
 * This program is free software.  You can redistribute it and/or
 * modify it under the terms of the GNU General Public License version
 * 2 as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
import berkeley.cs.dmc.system.DynamicsVariable;

public class OctopusClosedVars implements DynamicsVariable {

    public int numVariables() { return 2; }

    public int numFields() { return 2; }

    public float bufferStep() { return 0.01f; }

    public int bufferSize() { return 1; }

    public boolean initialCondition(double[] p)
    {
	p[0] = 0.0;
	p[1] = 0.0;
	return false;
    }

    public float newState(double last_time, double[] last_params,
                          double seen_time, double[] seen_params,
                          double new_time, double[] new_params)
    {
	new_params[0] = OctopusCarSystem.OMEGA1 * new_time;
	new_params[1] = OctopusCarSystem.OMEGA2 * new_time;

	return 1.0f;
    }

    public void evaluateFields(double t, double[] p, float[] f) {
	f[0] = (float)p[0];
	f[1] = (float)p[1];
    }
}
