/*
 * Copyright (C) 1998 Stephen Chenney and Jeffrey Ichnowski
 *
 * This program is free software.  You can redistribute it and/or
 * modify it under the terms of the GNU General Public License version
 * 2 as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
import berkeley.cs.dmc.numericalMethods.Integrator;
import berkeley.cs.dmc.numericalMethods.Derivative;

public class Pendulum2BurnIn implements Derivative {

  private static final double OMEGA_DOT = Pendulum2System.OMEGA_RUN
                                        * Pendulum2System.OMEGA_RUN
					/ ( 32.0 * Math.PI );
  private static final double BURN_TIME = Pendulum2System.OMEGA_RUN / OMEGA_DOT;

  private static final double DAMPING = 0.1;

  public final int getDimension() { return 2; }

  public final void derivs( double x, int n, double y[], double dydx[] ) {

    double omega = x * OMEGA_DOT;
    double theta = 0.5 * OMEGA_DOT * x * x;
    
    double sin_tx = Math.sin(theta - y[0]);
    double sin_x = Math.sin(y[0]);
    double cos_tx = Math.cos(theta - y[0]);

    dydx[0] = y[1];
    dydx[1] = ( Pendulum2System.R1 * omega * omega * sin_tx - 9.81 * sin_x
              - Pendulum2System.R1 * OMEGA_DOT * OMEGA_DOT * cos_tx )
              / Pendulum2System.R2
	    - DAMPING * ( y[1] - omega );
  }

  public void burnIn(double[] vars) {
    vars[0] = 0.0;
    vars[1] = 0.0;

    Integrator integ = new Integrator(this);

    integ.ODEIntegrate(vars, 0.0, BURN_TIME, 1.0e-3, 0.1, 1.0e-6);

  }

}


