/*
 * Copyright (C) 1998 Stephen Chenney and Jeffrey Ichnowski
 *
 * This program is free software.  You can redistribute it and/or
 * modify it under the terms of the GNU General Public License version
 * 2 as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
import java.io.PrintWriter;
import java.io.FileWriter;
import java.io.FileReader;
import java.io.StreamTokenizer;
import java.io.IOException;

class MakeRoller {
    static String eoln = System.getProperty("line.separator");

    static String track_header =
        "#VRML V2.0 utf8" + eoln
      + "PROTO Track [] {" + eoln
      + " Group {" + eoln
      + "  children [" + eoln
      + "    Shape {" + eoln
      + "      geometry IndexedLineSet {" + eoln
      + "        coord Coordinate {" + eoln
      + "          point [";

    static String track_tail =
	"        ]" + eoln
      + "      }" + eoln
      + "      appearance Appearance {" + eoln
      + "        material Material {" + eoln
      + "          emissiveColor 1.0 1.0 1.0" + eoln
      + "        }" + eoln
      + "      }" + eoln
      + "    }" + eoln
      + "  ]" + eoln
      + " }" + eoln
      + "}" + eoln;

    static String sys_header =
	"import dynamics.system.DynamicsSystem;" + eoln
      + "import numerical_recipes.Integrator;" + eoln
      + "import numerical_recipes.Derivative;" + eoln
      + "public class RollerSystem implements DynamicsSystem {" + eoln
      + "  BSpline     pspline;" + eoln
      + "  BSpline     nspline;" + eoln
      + "  private Derivative deriv = null;" + eoln
      + "  private Integrator integ = null;" + eoln
      + "  private double[] params = new double[2];" + eoln;

    static String sys_tail =
        "  public int numVariables() { return 2; }" + eoln
      + "  public int numFields() { return 11; }" + eoln
      + "  public int subsetLength() { return 2; }" + eoln
      + "  public int evaluationTechnique(int i) {" + eoln
      + "    if ( i == 0 )" + eoln
      + "      return ITERATIVE;" + eoln
      + "    else" + eoln
      + "      return FIRST_DERIVATIVE;" + eoln
      + "  }" + eoln
      + "  public double period() { return 0.0; }" + eoln
      + "  public boolean isCyclic(int ind) {" + eoln
      + "    if ( ind == 0 )" + eoln
      + "      return true;" + eoln
      + "    else" + eoln
      + "      return false;" + eoln
      + "  }" + eoln
      + "  public double[] getBound(int ind) {" + eoln
      + "    double[] res = new double[2];" + eoln
      + "    if ( ind == 0 ) {" + eoln
      + "      res[0] = 0.0; res[1] = PERIOD;" + eoln
      + "    } else {" + eoln
      + "      res[0] = 0.0; res[1] = 1.0;" + eoln
      + "    }" + eoln
      + "    return res;" + eoln
      + "  }" + eoln
      + "  public boolean initialCondition(double[] p) {" + eoln
      + "    p[0] = 0.0;" + eoln
      + "    double[] dp = pspline.evaluateDpDu(0.0);" + eoln
      + "    p[1] = Math.sqrt(1.0/(dp[0]*dp[0]+dp[1]*dp[1]+dp[2]*dp[2]));"+eoln
      + "    return false;" + eoln
      + "  }" + eoln
      + "  public float evaluate(double t_last, double[] p_last," + eoln
      + "                        double t_now, double[] p_now) {" + eoln
      + "    params[0] = p_last[0];" + eoln
      + "    params[1] = p_last[1];" + eoln
      + "    integ.ODE_Integrate(params, t_last, t_now, 1.0e-4, 0.5, 1.0e-5);"
      + eoln
      + "    p_now[0] = params[0];" + eoln
      + "    p_now[1] = params[1];" + eoln
      + "    return 1.0f;" + eoln
      + "  }" + eoln
      + "  public void evaluateFields(double time," + eoln
      + "                     double[] p_in, float[] f_out) {" + eoln
      + "    double[] a = new double[3];" + eoln
      + "    double[] v = new double[3];" + eoln
      + "    double[] pt = pspline.evaluate(p_in[0]);" + eoln
      + "    f_out[0] = (float)pt[0];" + eoln
      + "    f_out[1] = (float)pt[1];" + eoln
      + "    f_out[2] = (float)pt[2];" + eoln
      + "    double[] t = pspline.evaluateDpDu(p_in[0]);" + eoln
      + "    double l = Math.sqrt(t[0] * t[0] + t[1] * t[1] + t[2] * t[2]);"
      + eoln
      + "    a[0] = 0.0;" + eoln
      + "    a[1] = -t[2] / l;" + eoln
      + "    a[2] = t[1] / l;" + eoln
      + "    double sin_a = Math.sqrt(a[1] * a[1] + a[2] * a[2]);" + eoln
      + "    if ( sin_a < 1.0e-5 && sin_a > -1.0e-5 ) {" + eoln
      + "      f_out[7] = 1.0f;" + eoln
      + "      f_out[8] = 0.0f;" + eoln
      + "      f_out[9] = 0.0f;" + eoln
      + "      f_out[10] = 0.0f;" + eoln
      + "    } else {" + eoln
      + "      f_out[7] = 0.0f;" + eoln
      + "      f_out[8] = (float)(a[1] / sin_a);" + eoln
      + "      f_out[9] = (float)(a[2] / sin_a);" + eoln
      + "      if ( t[0] >= 0.0 )" + eoln
      + "        f_out[10] = (float)Math.asin(sin_a);" + eoln
      + "      else" + eoln
      + "        f_out[10] = (float)(Math.PI - Math.asin(sin_a));" + eoln
      + "    }" + eoln
      + "    double[] n = nspline.evaluate(p_in[0]);" + eoln
      + "    v[0] = n[1] * t[2] - n[2] * t[1];" + eoln
      + "    v[1] = n[2] * t[0] - n[0] * t[2];" + eoln
      + "    v[2] = n[0] * t[1] - n[1] * t[0];" + eoln
      + "    n[0] = t[1] * v[2] - t[2] * v[1];" + eoln
      + "    n[1] = t[2] * v[0] - t[0] * v[2];" + eoln
      + "    n[2] = t[0] * v[1] - t[1] * v[0];" + eoln
      + "    l = Math.sqrt(n[0] * n[0] + n[1] * n[1] + n[2] * n[2]);" + eoln
      + "    n[0] /= l;" + eoln
      + "    n[1] /= l;" + eoln
      + "    n[2] /= l;" + eoln
      + "    double cos_a = ( 1.0 - Math.cos(f_out[10]) );" + eoln
      + "    v[0] = a[1];" + eoln
      + "    v[1] = f_out[8] * f_out[9] * cos_a;" + eoln
      + "    v[2] = 1.0 - f_out[8] * f_out[8] * cos_a;" + eoln
      + "    a[0] = v[1] * n[2] - n[1] * v[2];" + eoln
      + "    a[1] = v[2] * n[0] - n[2] * v[0];" + eoln
      + "    a[2] = v[0] * n[1] - n[0] * v[1];" + eoln
      + "    sin_a = Math.sqrt(a[0] * a[0] + a[1] * a[1] + a[2] * a[2]);" + eoln
      + "    if ( sin_a < 1.0e-5 && sin_a > -1.0e-5 ) {" + eoln
      + "      f_out[3] = 1.0f;" + eoln
      + "      f_out[4] = 0.0f;" + eoln
      + "      f_out[5] = 0.0f;" + eoln
      + "      f_out[6] = 0.0f;" + eoln
      + "    } else {" + eoln
      + "      f_out[3] = (float)(a[0] / sin_a);" + eoln
      + "      f_out[4] = (float)(a[1] / sin_a);" + eoln
      + "      f_out[5] = (float)(a[2] / sin_a);" + eoln
      + "      if ( v[0] * n[0] + v[1] * n[1] + v[2] * n[2] >= 0.0 )" + eoln
      + "        f_out[6] = (float)Math.asin(sin_a);" + eoln
      + "      else" + eoln
      + "        f_out[6] = (float)(Math.PI - Math.asin(sin_a));" + eoln
      + "    }" + eoln
      + "  }" + eoln
      + "  public boolean checkState(double t, double[] p, boolean discard) {"
      + eoln
      + "    return true;" + eoln
      + "  }" + eoln
      + "  public float bufferStep() { return 0.1f; }" + eoln
      + "  public double[] resolutions() {" + eoln
      + "    double[] res = new double[2];" + eoln
      + "    res[0] = 0.2;" + eoln
      + "    res[1] = 0.2;" + eoln
      + "    return res;" + eoln
      + "  }" + eoln
      + "  public double mixTestMax() { return 0.5; }" + eoln
      + "  public double mixDiff() { return 1.0; }" + eoln
      + "}" + eoln + eoln;


    static void readControls(StreamTokenizer in, double[][] tc, double[][] nc,
			     int num)
	throws IOException
    {
	for ( int i = 0 ; i < num ; i++ )
	{
	    tc[i] = new double[3];
	    for ( int j = 0 ; j < 3 ; j++ )
	    {
		in.nextToken();
		tc[i][j] = in.nval;
	    }
	    nc[i] = new double[3];
	    for ( int j = 0 ; j < 3 ; j++ )
	    {
		in.nextToken();
		nc[i][j] = in.nval;
	    }
	}
	for ( int i = num ; i < num + 3 ; i++ )
	{
	    tc[i] = tc[i - num];
	    nc[i] = nc[i - num];
	}
    }

    static void readLifters(StreamTokenizer in, double[][] l, int num)
	throws IOException
    {
	for ( int i = 0 ; i < num ; i++ )
	{
	    l[i] = new double[3];
	    for ( int j = 0 ; j < 3 ; j++ )
	    {
		in.nextToken();
		l[i][j] = in.nval;
	    }
	}
    }

    public static void main(String[] args)
    {
	StreamTokenizer input = null;
	double[][]      t_control_pts = null;
	double[][]      n_control_pts = null;
	double[][]      lifters = null;
	int             num_c = 0;
	int             num_l = 0;

	try {
	    input = new StreamTokenizer(new FileReader(args[0]));
	    input.nextToken();
	    num_c = (int)input.nval;
	    t_control_pts = new double[num_c + 3][];
	    n_control_pts = new double[num_c + 3][];
	    readControls(input, t_control_pts, n_control_pts, num_c);
	    input.nextToken();
	    num_l = (int)input.nval;
	    if ( num_l > 0 )
	    {
		lifters = new double[num_l][];
		readLifters(input, lifters, num_l);
	    }
	} catch (Exception e) {
	    System.err.println(e);
	    System.exit(1);
	}

	PrintWriter trackfile =  null;
	PrintWriter sysfile =  null;

	BSpline     spline = new BSpline(3, num_c + 3, t_control_pts);
	int         count = 0;

	try {
	    trackfile = new PrintWriter(new FileWriter("Track.wrl"));
	    sysfile = new PrintWriter(new FileWriter("RollerSystem.java"));
	} catch (Exception e) {
	    System.err.println(e);
	    System.exit(1);
	}

	trackfile.println(track_header);

	for ( double u = 0.0 ; u < num_c ; u += 0.4 )
	{
	  try {
	    double[]    pt = spline.evaluate(u);
	    trackfile.println("            "+pt[0]+" "+pt[1]+" "+pt[2]);
	    count++;
	  } catch (ArithmeticException e) {
	    trackfile.println(e);
	  }
	}

	trackfile.println("          ]");
	trackfile.println("        }");
	trackfile.println("        coordIndex [");

	for ( int i = 0 ; i < count ; i++ )
	    trackfile.println("          " + i);
	trackfile.println("          0");
	trackfile.println(track_tail);

	sysfile.println(sys_header);

	sysfile.println("  static final double PERIOD = " + num_c + ".0;");
	sysfile.println("  private double[][]   t_control_pts = {");
	for ( int i = 0 ; i < num_c + 2 ; i++ )
	    sysfile.println("    { " + t_control_pts[i][0] + ", "
	                             + t_control_pts[i][1] + ", "
				     + t_control_pts[i][2] + "},");
	sysfile.println("    { "
	              + t_control_pts[num_c+2][0] + ", "
		      + t_control_pts[num_c+2][1] + ", "
		      + t_control_pts[num_c+2][2] + " }");
	sysfile.println("  };");
	sysfile.println("  private double[][]   n_control_pts = {");
	for ( int i = 0 ; i < num_c + 2 ; i++ )
	    sysfile.println("    { " + n_control_pts[i][0] + ", "
	                             + n_control_pts[i][1] + ", "
				     + n_control_pts[i][2] + "},");
	sysfile.println("    { "
	              + n_control_pts[num_c + 2][0] + ", "
		      + n_control_pts[num_c + 2][1] + ", "
		      + n_control_pts[num_c + 2][2] + " }");
	sysfile.println("  };");
	sysfile.println("  public double[][] lifters = "
		      + ( num_l == 0 ? "null;" : "{"));
	if ( num_l > 0 )
	{
	    for ( int i = 0 ; i < num_l - 1 ; i++ )
		sysfile.println("    { " + lifters[i][0] + ", "
					 + lifters[i][1] + ", "
					 + lifters[i][2] + "},");
	    sysfile.println("    { "
			  + lifters[num_l - 1][0] + ", "
			  + lifters[num_l - 1][1] + ", "
			  + lifters[num_l - 1][2] + " }");
	    sysfile.println("  };");
	}
	sysfile.println("  public RollerSystem() {");
	sysfile.println("    pspline = new BSpline(3, " + ( num_c + 3 )
		      + ", t_control_pts);");
	sysfile.println("    nspline = new BSpline(3, " + ( num_c + 3 )
		      + ", n_control_pts);");
	sysfile.println("    deriv = new RollerDerivative(this);");
	sysfile.println("    integ = new Integrator(deriv);");
	sysfile.println("  }");
	sysfile.println(sys_tail);

	try {
	    trackfile.close();
	    sysfile.close();
	} catch (Exception e) {
	}
    }
}

