/*
 * Copyright (C) 1998 Stephen Chenney and Jeffrey Ichnowski
 *
 * This program is free software.  You can redistribute it and/or
 * modify it under the terms of the GNU General Public License version
 * 2 as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
import berkeley.cs.dmc.numericalMethods.Derivative;
import berkeley.cs.dmc.bspline.BSpline;

public class RollerDerivative implements Derivative {

  RollerSystem sys;
  private float[] dp = null;
  private float[] dpdot = null;

  private static final double DAMPING = 0.0;

  public RollerDerivative( RollerSystem car ) {
    sys = car;
    dp = new float[3];
    dpdot = new float[3];
  }

  public final int getDimension() { return 2; }

  private void constantVelDerivs(float u, int n, double y[], double dydx[]) {
    sys.pspline.evaluateDpDu(u, dp);
    sys.pspline.evaluateDpdotDu(u, (float)y[1], dpdot);
    double numer = dpdot[0] * dp[0] + dpdot[1] * dp[1] + dpdot[2] * dp[2];
    double denom = dp[0] * dp[0] + dp[1] * dp[1] + dp[2] * dp[2];

    dydx[0] = y[1];
    dydx[1] = -y[1] * numer / denom;
  }

  public final void derivs( double x, int n, double y[], double dydx[] ) {
    float   u = (float)Math.IEEEremainder(y[0], sys.PERIOD);
    if ( u < 0.0 )
	u += sys.PERIOD;
    if ( u >= sys.PERIOD )
	u -= sys.PERIOD;

    if ( sys.lifters != null )
    {
	for ( int i = 0 ; i < sys.lifters.length ; i++ )
	{
	    if ( u >= sys.lifters[i][0] && u < sys.lifters[i][1]
	      && y[1] < sys.lifters[i][2] )
	    {
	        constantVelDerivs(u, n, y, dydx);
		return;
	    }
	}
    }

    sys.pspline.evaluateDpDu(u, dp);
    sys.pspline.evaluateDpdotDu(u, (float)y[1], dpdot);

    double dot_prod = dpdot[0] * dp[0] + dpdot[1] * dp[1] + dpdot[2] * dp[2];
    double numer = -9.81 * dp[1] - y[1] * dot_prod;
    double denom = dp[0] * dp[0] + dp[1] * dp[1] + dp[2] * dp[2];

    dydx[0] = y[1];
    dydx[1] = numer / denom;

  }
}


