/*
 * Copyright (C) 1998 Stephen Chenney and Jeffrey Ichnowski
 *
 * This program is free software.  You can redistribute it and/or
 * modify it under the terms of the GNU General Public License version
 * 2 as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
import berkeley.cs.dmc.system.DynamicsVariable;

public class TiltThetaVars implements DynamicsVariable {

    public static final double OMEGA_RUN = (6.5 * Math.PI / 30.0);

    public int numVariables() { return 7; }

    public int numFields() { return 7; }

    public float bufferStep() { return 0.01f; }

    public int bufferSize() { return 1; }

    public boolean initialCondition(double[] p)
    {
	p[0] = 0.0;
	p[1] = p[0] + Math.PI / 3.5;
	p[2] = p[1] + Math.PI / 3.5;
	p[3] = p[2] + Math.PI / 3.5;
	p[4] = p[3] + Math.PI / 3.5;
	p[5] = p[4] + Math.PI / 3.5;
	p[6] = p[5] + Math.PI / 3.5;

	return false;
    }

    public float newState( double last_time, double[] last_params,
                           double seen_time, double[] seen_params,
                           double new_time, double[] new_params) {

	new_params[0] = (OMEGA_RUN * new_time);
	new_params[1] = new_params[0] + Math.PI / 3.5;
	new_params[2] = new_params[1] + Math.PI / 3.5;
	new_params[3] = new_params[2] + Math.PI / 3.5;
	new_params[4] = new_params[3] + Math.PI / 3.5;
	new_params[5] = new_params[4] + Math.PI / 3.5;
	new_params[6] = new_params[5] + Math.PI / 3.5;

        return 1.0f;
    }

    public void evaluateFields(double t, double[] p, float[] f) {
	for ( int i = 0 ; i < 7 ; i++ )
	  f[i] = (float)p[i];
    }
}
