/*
 * Copyright (C) 1998 Stephen Chenney and Jeffrey Ichnowski
 *
 * This program is free software.  You can redistribute it and/or
 * modify it under the terms of the GNU General Public License version
 * 2 as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
import vrml.*;
import vrml.field.*;
import vrml.node.*;
import berkeley.cs.dmc.system.DynamicsVariable;

public class Roller extends Script {
  double time = 0.0;
  double last_time = 0.0;
  boolean initialized = false;
  double[] old_params = new double[2];
  double[] seen_params = new double[2];
  double[] new_params = new double[2];
  private RollerVariable car1vars = null;
  private MFFloat car1vars_buffer = null;
  private SFFloat car1vars_lastTime = null;
  private SFFloat car1vars_lastSeen = null;
  private float[] car1vars_elements = new float[11];
  private boolean car1vars_visible = true;
  private RollerVariable car2vars = null;
  private MFFloat car2vars_buffer = null;
  private SFFloat car2vars_lastTime = null;
  private SFFloat car2vars_lastSeen = null;
  private float[] car2vars_elements = new float[11];
  private boolean car2vars_visible = true;
  private SFRotation Car2_R2_event = null;
  private SFRotation Car2_R1_event = null;
  private SFVec3f Car2_T1_event = null;
  private SFRotation Car1_R2_event = null;
  private SFRotation Car1_R1_event = null;
  private SFVec3f Car1_T1_event = null;
  public void initialize() {
    if ( initialized ) return;
    car1vars = new RollerVariable();
    car1vars_buffer = (MFFloat)getField("car1vars_buffer");
    car1vars_lastTime = (SFFloat)getField("car1vars_lastTime");
    car1vars_lastSeen = (SFFloat)getField("car1vars_lastSeen");
    car2vars = new RollerVariable();
    car2vars_buffer = (MFFloat)getField("car2vars_buffer");
    car2vars_lastTime = (SFFloat)getField("car2vars_lastTime");
    car2vars_lastSeen = (SFFloat)getField("car2vars_lastSeen");
    Car2_R2_event = (SFRotation)getEventOut("Car2_R2");
    Car2_R1_event = (SFRotation)getEventOut("Car2_R1");
    Car2_T1_event = (SFVec3f)getEventOut("Car2_T1");
    Car1_R2_event = (SFRotation)getEventOut("Car1_R2");
    Car1_R1_event = (SFRotation)getEventOut("Car1_R1");
    Car1_T1_event = (SFVec3f)getEventOut("Car1_T1");
    initialized = true;
  }
  public void processEvent( Event e ) {
    String name = e.getName();
    if ( "currentTime".equals( name ) ) {
      time=((ConstSFFloat)e.getValue()).getValue();
    } else if ( "RollerBound".equals( name ) ) {
      car1vars_visible = (boolean)((ConstSFBool)e.getValue()).getValue();
      car2vars_visible = (boolean)((ConstSFBool)e.getValue()).getValue();
      ((SFBool)getEventOut("visible")).setValue((boolean)((ConstSFBool)e.getValue()).getValue());
    }
  }
  private void timeStep(MFFloat buffer, SFFloat lastTime,
                        SFFloat lastSeen, double bufStep, double phase,
                        DynamicsVariable var, double time, float[] fields,
                        int num_p, int num_b) {
    time += phase;

    double last_time = (double)lastTime.getValue();
    double last_seen = (double)lastSeen.getValue();

    if ( num_b == 1 )
    {
      for ( int i = 0 ; i < num_p ; i++ )
        old_params[i] = (double)buffer.get1Value(i);
      var.newState(last_time, old_params,
                   last_time, old_params,
                   time, new_params);
      for ( int i = 0 ; i < num_p ; i++ )
        buffer.set1Value(i, (float)new_params[i]);
      lastTime.setValue((float)time);
      var.evaluateFields(time, new_params, fields);
      return;
    }

    int    low_index = (int)Math.floor(time / bufStep);
    double low_time = low_index * bufStep;
    int    high_index = ( low_index + 1 ) % num_b;
    int    last_index = (int)Math.floor((last_time + 0.001) / bufStep) % num_b;
    int    seen_index = (int)Math.floor((last_seen + 0.001) / bufStep) % num_b;

    float work_done = 0.0f;

    int base = 0;

    low_index = low_index % num_b;

    base = last_index * num_p;
    for ( int i = 0 ; i < num_p ; i++ )
      old_params[i] = (double)buffer.get1Value(base + i);
    base = seen_index * num_p;
    for ( int i = 0 ; i < num_p ; i++ )
      seen_params[i] = (double)buffer.get1Value(base + i);

    if ( low_time > last_time + 0.00001 )
    {
      work_done = var.newState(last_time, old_params,
                               last_seen, seen_params,
                               low_time, new_params);
      for ( int i = 0 ; i < num_p ; i++ )
        old_params[i] = new_params[i];
      base = low_index * num_p;
      for ( int i = 0 ; i < num_p ; i++ )
        buffer.set1Value(base + i, (float)new_params[i]);
      last_time = low_time;
      last_index = low_index;
    }
    if ( low_time + bufStep > last_time + 0.00001 )
    {
      work_done += var.newState(low_time, old_params,
                                last_seen, seen_params,
                                low_time + bufStep, new_params);
      for ( int i = 0 ; i < num_p ; i++ )
        old_params[i] = new_params[i];
      base = high_index * num_p;
      for ( int i = 0 ; i < num_p ; i++ )
        buffer.set1Value(base + i, (float)new_params[i]);
      last_time = low_time + bufStep;
      last_index = high_index;
    }


    // Guaranteed to have low and high time buffer positions filled.
    int slots_used = last_index - low_index + 1;
    if ( slots_used < 1 )
      slots_used += num_b;
    base = 0;
    while (  work_done < 1.0f && slots_used < num_b )
    {
      work_done += var.newState(last_time, old_params,
                                last_seen, seen_params,
                                last_time + bufStep, new_params);
      last_index = ( last_index + 1 ) % num_b;
      base = last_index * num_p;
      for ( int i = 0 ; i < num_p ; i++ )
        old_params[i] = new_params[i];
      for ( int i = 0 ; i < num_p ; i++ )
        buffer.set1Value(base + i, (float)new_params[i]);
      last_time += bufStep;
      last_time = Math.floor(last_time / bufStep + 0.1) * bufStep;
      slots_used++;
    }

    lastTime.setValue((float)last_time);
    lastSeen.setValue((float)(low_time + bufStep));

    double fraction = ( time - low_time ) / bufStep;
    int base1 = high_index * num_p;
    base = low_index * num_p;
    for ( int i = 0 ; i < num_p ; i++ )
      new_params[i] = fraction * buffer.get1Value(base1 + i)
                    + ( 1.0 - fraction ) * buffer.get1Value(base + i);
    var.evaluateFields(time, new_params, fields);
  }
  public void eventsProcessed() {
    if ( ! initialized ) return;
    if ( time <= last_time )
        return;
    last_time = time;
    if ( car1vars_visible )
      timeStep(car1vars_buffer, car1vars_lastTime, car1vars_lastSeen, 0.1f, 0.0f, car1vars, last_time, car1vars_elements, 2, 100);
    if ( car2vars_visible )
      timeStep(car2vars_buffer, car2vars_lastTime, car2vars_lastSeen, 0.1f, 4.0f, car2vars, last_time, car2vars_elements, 2, 100);
    if ( car2vars_visible )
      Car2_R2_event.setValue(car2vars_elements[7], car2vars_elements[8], car2vars_elements[9], car2vars_elements[10]);
    if ( car2vars_visible )
      Car2_R1_event.setValue(car2vars_elements[3], car2vars_elements[4], car2vars_elements[5], car2vars_elements[6]);
    if ( car2vars_visible )
      Car2_T1_event.setValue(car2vars_elements[0], car2vars_elements[1], car2vars_elements[2]);
    if ( car1vars_visible )
      Car1_R2_event.setValue(car1vars_elements[7], car1vars_elements[8], car1vars_elements[9], car1vars_elements[10]);
    if ( car1vars_visible )
      Car1_R1_event.setValue(car1vars_elements[3], car1vars_elements[4], car1vars_elements[5], car1vars_elements[6]);
    if ( car1vars_visible )
      Car1_T1_event.setValue(car1vars_elements[0], car1vars_elements[1], car1vars_elements[2]);
  }
}

