/*
 * StreamSplitter.java - OutputStream splitting
 * Copyright (C) 1998 Stephen Chenney and Jeffrey Ichnowski
 *
 * This program is free software.  You can redistribute it and/or
 * modify it under the terms of the GNU General Public License version
 * 2 as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
package berkeley.cs.dmc.compiler; // There is probably a better place for this

import java.io.OutputStream;
import java.io.IOException;

class StreamSplitter extends OutputStream {

    OutputStream a, b;

    StreamSplitter( OutputStream a, OutputStream b ) {
	this.a = a;
	this.b = b;
    }

    public void close() throws IOException {
	try {
	    a.close();
	} finally {
	    b.close();
	}
    }

    public void flush() throws IOException {
	try {
	    a.flush();
	} finally {
	    b.flush();
	}
    }

    public void write( int c ) throws IOException {
	try {
	    a.write(c);
	} finally {
	    b.write(c);
	}
    }

    public void write( byte c[] ) throws IOException {
	try {
	    a.write(c);
	} finally {
	    b.write(c);
	}
    }

    public void write( byte c[], int off, int len ) throws IOException {
	try {
	    a.write(c,off,len);
	} finally {
	    b.write(c,off,len);
	}
    }
}
