package berkeley.cs.dmc.dmod;

import java.util.Enumeration;
import java.util.Hashtable;
import berkeley.cs.dmc.model.*;
import berkeley.cs.dmc.system.DynamicsVariable;

public class Animator implements Runnable {
    private Main parent = null;
    private TreeNode  root = null;
    private Hashtable var_hash = new Hashtable();

    public Animator(Main p)
	throws ClassNotFoundException
    {
	parent = p;
	root = p.getTree();
	Hashtable vars = p.getVars();

	Hashtable systems = new Hashtable();
	Enumeration enum = vars.elements();

	while ( enum.hasMoreElements() )
	{
	    VariableDefinition var = (VariableDefinition)enum.nextElement();
	    Class the_class = null;
	    DynamicsVariable system = null;
	    String name = var.getType();

	    if ( ( system = (DynamicsVariable)systems.get(name) ) == null )
	    {
		try { 
		    the_class = p.dynamicsClassLoader.loadClass(name); // Class.forName(name);
		    system = (DynamicsVariable)the_class.newInstance();
		} catch( ClassNotFoundException e ) {
		    throw new ClassNotFoundException(
						   "Class "+name+" not found.");
		} catch( InstantiationException e ) {
		    throw new ClassNotFoundException(
			     "Could not instantiate class. " + e.getMessage() );
		} catch( IllegalAccessException e ) {
		    throw new ClassNotFoundException( "Illegal access. "
						    + e.getMessage() );
		} catch( ClassCastException e ) {
		    throw new ClassNotFoundException("Class " + name
		      + " does not implement dynamics.system.DynamicsVariable");
		}
		systems.put(var.getName(), system);
	    }
	    var_hash.put(var.getName(), new AnimateVar(system, var.getPhase()));
	}
    }

    private void treeSetFields(TreeNode tree) {
	if ( tree instanceof TranslateNode )
	{
	    TranslateNode node = (TranslateNode)tree;
	    if (  ! ( node.getX() instanceof Number ) )
	    {
		VariableField f = (VariableField)node.getX();
		AnimateVar var = (AnimateVar)var_hash.get(f.getVar().getName());
		f.setValue(var.getField(f.getIndex()));
	    }
	    if (  ! ( node.getY() instanceof Number ) )
	    {
		VariableField f = (VariableField)node.getY();
		AnimateVar var = (AnimateVar)var_hash.get(f.getVar().getName());
		f.setValue(var.getField(f.getIndex()));
	    }
	    if (  ! ( node.getZ() instanceof Number ) )
	    {
		VariableField f = (VariableField)node.getZ();
		AnimateVar var = (AnimateVar)var_hash.get(f.getVar().getName());
		f.setValue(var.getField(f.getIndex()));
	    }
	}

	if ( tree instanceof RotateNode )
	{
	    RotateNode node = (RotateNode)tree;
	    if (  ! ( node.getX() instanceof Number ) )
	    {
		VariableField f = (VariableField)node.getX();
		AnimateVar var = (AnimateVar)var_hash.get(f.getVar().getName());
		f.setValue(var.getField(f.getIndex()));
	    }
	    if (  ! ( node.getY() instanceof Number ) )
	    {
		VariableField f = (VariableField)node.getY();
		AnimateVar var = (AnimateVar)var_hash.get(f.getVar().getName());
		f.setValue(var.getField(f.getIndex()));
	    }
	    if (  ! ( node.getZ() instanceof Number ) )
	    {
		VariableField f = (VariableField)node.getZ();
		AnimateVar var = (AnimateVar)var_hash.get(f.getVar().getName());
		f.setValue(var.getField(f.getIndex()));
	    }
	    if (  ! ( node.getAngle() instanceof Number ) )
	    {
		VariableField f = (VariableField)node.getAngle();
		AnimateVar var = (AnimateVar)var_hash.get(f.getVar().getName());
		f.setValue(var.getField(f.getIndex()));
	    }
	}

	for ( int i = 0 ; i < tree.getChildCount() ; i++ )
	    treeSetFields(tree.getChild(i));
    }

    public void run() {
	long    time = System.currentTimeMillis();
	long    base_time = time;
	boolean drawingBounds = false;

	while ( true ) {
	    // Update all the systems.
	    Enumeration enum = var_hash.elements();
	    while ( enum.hasMoreElements() )
	    {
		AnimateVar var = (AnimateVar)enum.nextElement();
		var.update((time - base_time ) * 0.001);
	    }

	    treeSetFields(root);

	    parent.updateFrame();

	    time = System.currentTimeMillis();
	}
    }
}

