package berkeley.cs.dmc.dmod;

import java.io.*;
import java.util.Hashtable;
import java.util.StringTokenizer;

public class DynamicsClassLoader extends ClassLoader {
 
    Hashtable cache = new Hashtable();
    Main main;

    public DynamicsClassLoader( Main main ) {
	this.main = main;
    }

    private byte[] loadClassData( String classname ) {
	String name = classname.replace( '.', File.separatorChar );
	byte data[];
	File file;

	file = new File( main.directory, name + ".class" );
	if( !file.exists() ) {
	    String classpath = System.getProperty( "java.class.path" );

	    if( classpath == null )
		return null;

	    StringTokenizer st = new StringTokenizer( classpath, File.pathSeparator );
	    do {
		if( !st.hasMoreTokens() )
		    return null;

		file = new File( st.nextToken(), name + ".class" );
	    } while( !file.exists() );
	}

	FileInputStream in = null;

	try {
	    int len = (int)file.length();
	    int off = 0;

	    in = new FileInputStream(file);
	    data = new byte[len];

	    while( len > 0 ) {
		int read = in.read( data, off, len );
		if( read <= 0 )
		    return null;
	
		off += read;
		len -= read;
	    }

	    return data;

	} catch( IOException e ) {
	    return null;
	} finally {
	    if( in != null )
		try { in.close(); } catch( IOException e ) {}
	}
    }

    public synchronized Class loadClass( String name, boolean resolve ) 
	throws ClassNotFoundException
    {
	Class c = (Class)cache.get(name);

	if( c == null ) {
	    try {
		c = Class.forName(name);
	    } catch( ClassNotFoundException e ) {
		System.out.println( "Loading class '"+name+"'" );

		byte data[] = loadClassData(name);
		if( data == null )
		    throw new ClassNotFoundException();

		c = defineClass(name, data, 0, data.length);
		cache.put( name, c );
	    }
	}

	if( resolve )
	    resolveClass(c);

	return c;
    }
}
