package berkeley.cs.dmc.dmod;

import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.net.*;
import java.util.*;

import berkeley.cs.dmc.ji3d.Vec3;
import berkeley.cs.dmc.model.*;
import berkeley.cs.dmc.model.parser.*;
import berkeley.cs.dmc.vrml.RenderState;
import berkeley.cs.dmc.vrml.parser.*;

public class Main extends Frame implements ActionListener, ItemListener {
    Object modelLock = new Object();
    Hashtable modelVars = new Hashtable();
    TreeNode  modelTree = new TreeNode();
    DynamicsClassLoader dynamicsClassLoader;

    public String directory = "";
    ModelCanvas modelCanvas;
    TreePanel treePanel;
    TreeNodePanel treeNodePanel;
    CheckboxMenuItem viewBounds;
    CheckboxMenuItem wireFrame;

    public Main() {
	super( "DMod - Dynamics Modeller" );

	dynamicsClassLoader = new DynamicsClassLoader(this);

	setForeground( SystemColor.controlText );
	setBackground( SystemColor.control );

	MenuBar menu = new MenuBar();
	Menu fileMenu = new Menu( "File" );
	fileMenu.add( new MenuItem( "New..." ) );
	fileMenu.add( new MenuItem( "Load..." ) );
	fileMenu.add( new MenuItem( "Save..." ) );
	fileMenu.add( new MenuItem( "Save As VRML..." ) );
	fileMenu.addSeparator();
	fileMenu.add( new MenuItem( "Exit" ) );

	Menu dynamicsMenu = new Menu( "Dynamics" );
	dynamicsMenu.add( new MenuItem( "Edit Variables..." ) );
	dynamicsMenu.addSeparator();
	dynamicsMenu.add( new MenuItem( "Run!" ) );
	dynamicsMenu.add( new MenuItem( "Stop!" ) );

	Menu optionsMenu = new Menu( "Options" );
	optionsMenu.add( wireFrame = new CheckboxMenuItem( "Wire Frame", false ) );
	optionsMenu.add( viewBounds = new CheckboxMenuItem( "View Bounds", false ) );
	optionsMenu.add( new MenuItem( "Bound Model..." ) );

	Menu helpMenu = new Menu( "Help" );
	helpMenu.add( new MenuItem( "About..." ) );

	linkMenu( fileMenu );
	linkMenu( dynamicsMenu );
	linkMenu( optionsMenu );
	linkMenu( helpMenu );

	viewBounds.addItemListener( this );
	wireFrame.addItemListener( this );
	
	menu.add( fileMenu );
	menu.add( optionsMenu );
	menu.add( dynamicsMenu );
	menu.setHelpMenu( helpMenu );
	setMenuBar( menu );

	modelCanvas = new ModelCanvas(this,
				      new Vec3( 0.0f, 0.0f, 10.0f ),
				      new Vec3( 0.0f, 2.0f, 0.0f ), 
				      new Vec3( 0.0f, 1.0f, 0.0f ) );

	treePanel   = new TreePanel(this);

	setLayout( new BorderLayout() );
	VResizePanel vp = new VResizePanel( VResizePanel.GROW_TOP, 4 );
	add( vp, BorderLayout.CENTER );
	vp.add( modelCanvas, vp.TOP );

	Panel bottom = new Panel( new GridBagLayout() );
	GridBagConstraints c = new GridBagConstraints();
	c.gridx = c.gridy = 0;
	c.weightx = 1.0;
	c.weighty = 1.0;
	c.fill = c.BOTH;
	bottom.add( treePanel, c );
	c.gridx = 1;
	c.weightx = 0.0;
	bottom.add( treeNodePanel = new TreeNodePanel(this), c );
	
	vp.add( bottom, vp.BOTTOM );
	bottom.setSize( treePanel.getPreferredSize() );

	enableEvents( AWTEvent.WINDOW_EVENT_MASK );

	/*
	vp.add( treePanel, vp.BOTTOM );
	treePanel.setSize( treePanel.getPreferredSize() );
	*/

	/*
	setLayout( new GridBagLayout() );
	GridBagConstraints c = new GridBagConstraints();
	c.gridwidth = 2;
	c.gridx = c.gridy = 0;
	c.fill = c.BOTH;
	c.weightx = 1.0;
	c.weighty = 1.0;
	add( modelCanvas, c );
	c.gridy = 1;
	c.gridwidth = 1;
	c.weighty = 0.0;
	add( treePanel, c );
	c.gridx = 1;
	c.weightx = 0.0;
	add( new TreeNodePanel( this, new RotateNode() ), c );
	*/

    }

    void linkMenu( Menu menu ) {
	int num = menu.countItems();
	for( int i=0 ; i<num ; i++ ) {
	    MenuItem mi = menu.getItem(i);
	    if( mi instanceof Menu )
		linkMenu( (Menu)mi );
	    else
		mi.addActionListener( this );
	}
    }

    public Hashtable getVars() {
	return modelVars;
    }

    public TreeNode getTree() {
	return modelTree;
    }

    public VariableDefinition getVar( String name ) {
	VariableDefinition def = (VariableDefinition)modelVars.get(name);
	if( def == null ) {
	    if( ChoiceDialog.choose(this, "Undefined Variable", 
				    "The variable \""+name+"\" is undefined.\n"+
				    "Would you like to define it?",
				    "Yes\nNo") == 1 )
		throw new UndefinedVariableException( name );
	    
	    UnDefVariableDialog dlg = new UnDefVariableDialog( this, name );
	    if( dlg.isCancelled() )
		throw new UndefinedVariableException( name );

	    def = dlg.getVariableDefinition();
	    getVars().put( name, def );
	}
	return def;
    }

    private void getVarRef( Vector refs, TreeNode node, VariableDefinition def ) {
	String fields[] = node.getFieldList();

    mainloop:
	for( int i=0 ; i<fields.length ; i++ ) {
	    Object fld = node.getField(fields[i]);

	    if( fld instanceof VariableField ) {
		if( ((VariableField)fld).getVar() == def ) {
		    refs.addElement(this);
		    break mainloop;
		}
	    } else if( fld == def ) {
		refs.addElement(this);
		break mainloop;
	    } else if( fld instanceof Vector ) {
		Vector v = (Vector)fld;
		for( int j=0 ; j<v.size() ; j++ )
		    if( v.elementAt(j) == def ) {
			refs.addElement(this);
			break mainloop;
		    }
	    }
	}

	for( int i=0 ; i<node.getChildCount() ; i++ )
	    getVarRef( refs, node.getChild(i), def );
    }

    public Vector getVarReference( VariableDefinition def ) {
	Vector vec = new Vector();
	getVarRef( vec, getTree(), def );
	return vec;
    }

    public Object parseVariableField( String text )
	 throws NumberFormatException, UndefinedVariableException
    {
	text = text.trim();

	if( text.length() == 0 )
	    return null;

	try {
	    return new Integer( text );
	} catch( NumberFormatException e ) {}
	try {
	    return new Double( text );
	} catch( NumberFormatException e ) {}

	int ob = text.indexOf('[');
	int cb = text.lastIndexOf(']');
	int index = 0;

	if( ob != -1 && cb != -1 ) {
	    String indexString = text.substring( ob+1, cb ).trim();
	    // throws NumberFormatException...
	    index = Integer.parseInt( indexString ); 
	    text = text.substring( 0, ob ).trim();
	}
		
	Hashtable vars = getVars();
	VariableDefinition def = (VariableDefinition)vars.get( text );
	    
	if( def == null )
	    throw new UndefinedVariableException(text);

	return new VariableField( def, index );
    }

    public void deleteNode( TreeNode node ) {
	synchronized( modelLock ) {
	    TreeNode parent = node.getParent();
	    
	    while( node.getChildCount() > 0 )
		parent.addChild( node.getChild(0) );

	    parent.removeChild(node);
	}
	treePanel.updateDisplay();
	modelCanvas.updateDisplay();
    }

    public void treeChanged() {
	modelCanvas.updateDisplay();
    }

    public void updateFrame() {
	modelCanvas.updateFrame();
    }

    public void nodeChanged( TreeNode node ) {
	if( node instanceof VRMLModelNode ) {
	    VRMLModelNode vmnode = (VRMLModelNode)node;
	    vmnode.scene = loadVRML( vmnode.getURL().toString() );
	}

	modelCanvas.resetBounds();
	modelCanvas.updateDisplay();
	treePanel.updateDisplay();
    }

    public void selectNode( TreeNode node ) {
	treeNodePanel.setTreeNode( node );
    }

    Hashtable vrmlModels = new Hashtable();

    public Scene loadVRML( String filename ) {
	Scene scene;
	// if( (scene = (VRMLScene)vrmlModels.get(filename)) != null )
	// return scene;

	InputStream in = null;
	boolean failed = false;

	// System.out.println( "loading "+filename+"..." );

	File file = new File( directory, filename );

	try {
	    if( file.exists() ) {
		in = new FileInputStream(file);
	    } else {
		in = new URL("http://"+filename).openStream();
	    }
	    VRMLParser parser = new VRMLParser( in );
	    parser.file();
	    scene = parser.getScene().getFirstProto();
	    scene.build();
	    vrmlModels.put( filename, scene );
	    return scene;
	} catch( berkeley.cs.dmc.vrml.parser.ParseException e ) {
	    System.out.println( "ParseError: " );
	    e.printStackTrace();
	    failed = true;
	} catch( MalformedURLException e ) {
	    System.out.println( "Malformed URL: " +e.getMessage() );
	    failed = true;
	} catch( IOException e ) {
	    System.out.println( "IO Error: " + e.getMessage() );
	    failed = true;
	} finally {
	    if( failed )
		System.out.println( filename+" not loaded" );

	    if( in != null ) {
		try { in.close(); } catch( IOException e ) {}
	    }
	}
	return null;
    }

    public void loadDynamicsVRMLModels( TreeNode node ) {
	if( node instanceof VRMLModelNode ) {
	    VRMLModelNode vnode = (VRMLModelNode)node;
	    vnode.scene = loadVRML( vnode.getURL().toString() );
	}

	int num = node.getChildCount();
	for( int i=0 ; i<num ; i++ )
	    loadDynamicsVRMLModels( node.getChild(i) );
    }

    public void loadDynamics( String dir, String filename ) {
	FileInputStream in = null;
	// System.out.println( "loading "+filename+"..." );

	vrmlModels = new Hashtable();  // clear the vrml models

	try {
	    in = new FileInputStream(dir+filename);
	    DynamicsParser parser = new DynamicsParser(in);
	    DynamicsModel model = parser.parse();
	    synchronized( modelLock ) {
		TreeNode tree = model.getTree();
		modelVars = model.getVars();
		if( tree.getChildCount() == 1 && "group".equals( tree.getType() ) ) {
		    modelTree = tree;
		} else {
		    modelTree = new TreeNode();
		    modelTree.setName( "TreeRoot" );
		    modelTree.addChild( tree );
		}
	    }
	    directory = dir;
	    loadDynamicsVRMLModels( getTree() );
	    treePanel.updateDisplay();
	    modelCanvas.resetBounds();
	    modelCanvas.updateDisplay();
	} catch( IOException e ) {
	    ChoiceDialog.info( this, "IO Error", "While loading '"+filename+"'\n"+e.getMessage() );
	} catch( berkeley.cs.dmc.model.parser.ParseException e ) {
	    ChoiceDialog.info( this, "Parse Error", e.getMessage() );
	} finally {
	    if( in != null )
		try { in.close(); } catch( IOException e ) {}
	}
    }
	    
    public void saveDynamics( String filename ) {
	PrintStream out = null;
	try {
	    out = new PrintStream(new FileOutputStream(filename));

	    Hashtable vars = getVars();
	    if( vars != null ) {
		Enumeration enum = vars.elements();
		while( enum.hasMoreElements() ) {
		    VariableDefinition var = (VariableDefinition)enum.nextElement();
		    if( getVarReference(var).size() > 0 )
			var.print(out);
		}
	    }

	    TreeNode root = getTree();
	    if( root != null ) {
		if( root.getChildCount() == 1 )
		    root.getChild(0).print(out);
		else {
		    root.print(out);
		}
	    }
	} catch( IOException e ) {
	    ChoiceDialog.info( this, "IO Error", "While saving '"+filename+"'\n"+e.getMessage() );
	} finally {
	    if( out != null )
		out.close();
	}
    }

    //
    // Event Handlers
    //
    static void centerDialog( Dialog dlg ) {
	Rectangle pb = dlg.getParent().getBounds();
	Rectangle b = dlg.getBounds();
	int x = pb.x + (pb.width - b.width) / 2;
	int y = pb.y + (pb.height - b.height) / 2;
	
	dlg.setLocation( x, y );
    }

    void newEvent() {
	FileDialog dlg = new FileDialog(this, "New Project Directory...", FileDialog.LOAD);
	centerDialog(dlg);
	dlg.setVisible(true);
	String dir = dlg.getDirectory();
	if( dir != null ) {
	    synchronized( modelLock ) {
		modelTree = new TreeNode();
		modelTree.setName( "TreeRoot" );
		modelVars = new Hashtable();
		directory = dir;
	    }
	    treePanel.updateDisplay();
	    modelCanvas.resetBounds();
	    modelCanvas.updateDisplay();
	}
    }

    void loadEvent() {
	FileDialog dlg = new FileDialog(this, "Load Dynamics...", FileDialog.LOAD);
	centerDialog(dlg);
	dlg.setVisible(true);
	String dir = dlg.getDirectory();
	String file = dlg.getFile();
	if( dir != null && file != null ) {
	    loadDynamics( dir, file );
	}
    }

    void saveEvent() {
	FileDialog dlg = new FileDialog(this, "Save Dynamics...", FileDialog.SAVE);
	centerDialog(dlg);
	dlg.setVisible(true);
	String dir = dlg.getDirectory();
	String file = dlg.getFile();
	if( dir != null && file != null )
	    saveDynamics( dir + file );
    }

    void saveVRMLEvent() {
	FileDialog dlg = new FileDialog(this, "Save Dynamics...", FileDialog.SAVE);
	centerDialog(dlg);
	dlg.setVisible(true);
	String dir = dlg.getDirectory();
	String file = dlg.getFile();
	if( dir != null && file != null ) {
	    int idx = file.indexOf('.');
	    if( idx != -1 )
		file = file.substring( 0, idx );

	    try {
		berkeley.cs.dmc.tovrml.Main.dump( file, new DynamicsModel( getVars(), getTree() ) );
	    } catch( java.io.IOException e ) {
		ChoiceDialog.info( this, "File Save Error",
				   "IO Error:\n"+e.getMessage() );
	    }
	}
    }

    Thread dynamicsThread;
    synchronized void runDynamics() {
	if( dynamicsThread == null ) {
	    try {
		Animator animator = new Animator(this);
		dynamicsThread = new Thread(animator, "Animator");
		dynamicsThread.setPriority( Thread.MIN_PRIORITY );
		dynamicsThread.start();
	    } catch( Exception e ) {
		StringWriter sout = new StringWriter();
		PrintWriter out = new PrintWriter( sout );

		out.println( "An exception occured while trying to run the dynamics.\n"+
			     "The stack trace follows:\n" );
		e.printStackTrace( out );
		out.flush();

		ChoiceDialog.info( this, "Dynamics Exception",
				   sout.toString() );
	    }
	}
    }

    synchronized void stopDynamics() {
	if( dynamicsThread != null ) {
	    dynamicsThread.stop();
	    dynamicsThread = null;
	}
    }

    void exitEvent() {
	stopDynamics();
	System.exit(0);
    }
    void aboutEvent() {
	ChoiceDialog.info( this, "About...",
			   "DMod - Dynamics Modeller\n"+
			   "alpha\n"+
			   "-\n"+
			   "Rendering code Copyright (c)1997 Jeffrey Ichnowski\n"+
			   "All Rights Reserved" );
    }

    void editVariables() {
	new VariableEditDialog( this );
    }

    void boundEvent() {
	float bounds[][] = modelCanvas.bounds;

	switch( ChoiceDialog.choose( this, "Bound Model",
				     "Insert a bound node with the current bound volume?\n"+
				     "("+bounds[0][0]+","+bounds[1][0]+","+bounds[2][0]+") - ("+bounds[0][1]+","+bounds[1][1]+","+bounds[2][1]+")", "Yes\nNo" ) ) {
	case 0: // Yes
	    synchronized( modelLock ) {
		BoundNode bound = new BoundNode();
		bound.setX( new Float( bounds[0][1] - bounds[0][0] ) );
		bound.setY( new Float( bounds[1][1] - bounds[1][0] ) );
		bound.setZ( new Float( bounds[2][1] - bounds[2][0] ) );
		bound.setName( "ModelBounds" );

		while( modelTree.getChildCount() > 0 )
		    bound.addChild( modelTree.getChild(0) );

		modelTree.addChild( bound );
		treePanel.updateDisplay();
	    }
	    break;
	case 1: // No
	    break;
	}
    }

    public void itemStateChanged( ItemEvent evt ) {
	// evt.getSource() == wireFrame or viewBounds
	modelCanvas.updateDisplay();
    }

    public void actionPerformed( ActionEvent evt ) {
	String cmd = evt.getActionCommand();
	Object src = evt.getSource();
	
	if( src instanceof MenuItem ) {
	    if( "Load...".equals(cmd) )       loadEvent();
	    else if( "Save...".equals(cmd) )  saveEvent();
	    else if( "Save As VRML...".equals(cmd) ) saveVRMLEvent();
	    else if( "New...".equals(cmd) )   newEvent();
	    else if( "Exit".equals(cmd) )     exitEvent();
	    else if( "About...".equals(cmd) ) aboutEvent();
	    else if( "Run!".equals(cmd) )     runDynamics();
	    else if( "Stop!".equals(cmd) )    stopDynamics();
	    else if( "Edit Variables...".equals(cmd) ) editVariables();
	    else if( "Bound Model...".equals(cmd) ) boundEvent();
	    else
		throw new InternalError( "Unhandled menu item: "+cmd );
	}
    }

    protected void processWindowEvent( WindowEvent evt ) {
	if( evt.getID() == WindowEvent.WINDOW_CLOSING )
	    exitEvent();
    }

    public static void main( String args[] ) {
	Main main = new Main();
	main.setSize(640,480);
	main.validate();
	main.show();
    }
}
