package berkeley.cs.dmc.dmod;

import java.util.Vector;

public class Timer implements Runnable {
    long millis;
    boolean periodic;
    Object cmd;
    Vector listeners = new Vector();

    public Timer( Object cmd ) {
	this.cmd = cmd;
	new Thread(this, "Timer").start();
    }

    public synchronized void addTimerListener( TimerListener tl ) {
	listeners.addElement(tl);
    }

    public synchronized void removeTimerListeners( TimerListener tl ) {
	listeners.removeElement(tl);
    }

    public synchronized boolean isRunning() {
	return millis > 0;
    }

    public synchronized void start( long millis, boolean periodic ) {
	this.millis = millis;
	this.periodic = periodic;
	notify();
    }

    public synchronized void stop() {
	this.millis = 0;
	notify();
    }

    public synchronized void kill() {
	this.millis = -1;
	notify();
    }

    public synchronized void run() {
	while( true ) {
	    try {
		long oldmillis = millis;

		wait( millis );

		if( millis < 0 )
		    break;
		if( millis == oldmillis ) {
		    for( int i=0 ; i<listeners.size() ; i++ ) {
			TimerListener l = (TimerListener)listeners.elementAt(i);
			l.timerEvent( this, cmd );
		    }
		    if( !periodic )
			millis = 0;
		}
	    } catch( InterruptedException e ) {}
	}
    }
}
