package berkeley.cs.dmc.dmod;

import java.awt.*;
import java.awt.event.*;
import java.util.Vector;
import berkeley.cs.dmc.model.*;

public class TreeNodePanel extends Panel
implements TextListener, ActionListener
{
    // there are more but we are only interested in the longest strings.
    private static String ATTRIBS[] = { "angle", "name" }; 

    Main main;

    Label nameLabel;
    TextField name;
    
    transient int labelWidth;
    String    fields[];
    Label     labels[];
    TextField textfields[];
    String    oldstrings[];
    Panel     buttonPanel;

    TreeNode node;
    boolean changed = false;

    Button apply = new Button( "Apply" );
    Button delete = new Button( "Delete" );

    public TreeNodePanel( Main main ) {
	super(null);
	setFont( new Font( "Dialog", Font.PLAIN, 12 ) );
	this.main = main;

	int i;
	labels = new Label[6];
	textfields = new TextField[6];
	oldstrings = new String[6];

	labels[0] = new Label( "Name:" );
	textfields[0] = new TextField(20);
	textfields[0].setEditable(false);

	for( i=1 ; i<6 ; i++ ) {
	    labels[i] = new Label();
	    textfields[i] = new TextField();
	    labels[i].setVisible( false );
	    textfields[i].setVisible( false );
	}

	for( i=0 ; i<6 ; i++ ) {
	    add( labels[i] );
	    add( textfields[i] );
	    textfields[i].addTextListener(this);
	    textfields[i].addActionListener(this);
	}

	buttonPanel = new Panel();
	buttonPanel.add( apply );
	buttonPanel.add( delete );
	
	add( buttonPanel );

	apply.setEnabled( false );
	delete.setEnabled( false );

	apply.addActionListener(this);
	delete.addActionListener(this);

	calcFontSize();
    }

    private void calcFontSize() {
	FontMetrics fm = getFontMetrics( getFont() );
	int width = 0;
	for( int i=0 ; i<ATTRIBS.length ; i++ )
	    width = Math.max( width, fm.stringWidth( ATTRIBS[i]+":" ) );
	labelWidth = width;
    }

    public void setFont( Font font ) {
	super.setFont(font);
	invalidate();
    }

    public Dimension getPreferredSize() {
	Dimension tfs = textfields[0].getPreferredSize();
	Dimension lfs = labels[0].getPreferredSize();

	int width = labelWidth + 2 + tfs.width;
	int height = (Math.max( tfs.height, lfs.height ) + 1) * 6 + 
	    buttonPanel.getPreferredSize().height + 4;

	return new Dimension( width, height );
    }

    public Dimension getMinimumSize() { return getPreferredSize(); }

    public void validate() {
	calcFontSize();
	super.validate();
    }

    public void doLayout() {
	Dimension size = getSize();
	int height = size.height - 8;
	int width = size.width - 8;
	int tfwidth = width - labelWidth - 6;
	int y = 8;

	for( int i=0 ; i<6 ; i++ ) {
	    Dimension lps = labels[i].getPreferredSize();
	    Dimension tps = textfields[i].getPreferredSize();

	    labels[i].setBounds( 6, y, labelWidth, lps.height );
	    textfields[i].setBounds( labelWidth+8, y, tfwidth, tps.height );
	    
	    y += Math.max( lps.height, tps.height ) + 1;
	}

	Dimension bps = buttonPanel.getPreferredSize();
	buttonPanel.setBounds( 8, height - bps.height, width-8, bps.height );
    }

    public static String fieldToString( Object field ) {
	if( field instanceof Vector ) {
	    Vector vec = (Vector)field;
	    StringBuffer bfr = new StringBuffer();
	    for( int i=0 ; i<vec.size() ; i++ ) {
		if( i!=0 )
		    bfr.append(", ");
		bfr.append( vec.elementAt(i).toString() );
	    }
	    return new String(bfr);
	} else {
	    return field.toString();
	}
    }

    public void setTreeNode( TreeNode node ) {
	synchronized( getTreeLock() ) {
	    if( node != null ) {
		this.node = node;
		labels[0].setEnabled(true);
		textfields[0].setEnabled(true);
		textfields[0].setEditable(true);
		textfields[0].setText( oldstrings[0] = node.getName() );
	    
		fields = node.getFieldList();
		int i;
		for( i=0 ; i<fields.length ; i++ ) {
		    labels[i+1].setText( fields[i] );
		    oldstrings[i+1] = fieldToString( node.getField( fields[i] ) );
		    textfields[i+1].setText( oldstrings[i+1] );
		    labels[i+1].setVisible(true);
		    textfields[i+1].setVisible(true);
		}

		for( i++; i<6; i++ ) {
		    labels[i].setVisible(false);
		    textfields[i].setVisible(false);
		}
		delete.setEnabled(true);
	    } else {
		labels[0].setEnabled(false);
		textfields[0].setEnabled(false);
		textfields[0].setEditable(false);
		textfields[0].setText("");

		for( int i=1 ; i<6 ; i++ ) {
		    labels[i].setVisible(false);
		    textfields[i].setVisible(false);
		}
		delete.setEnabled(false);
	    }
	}
	setChanged(false);
    }
    
    private void setChanged(boolean newchange) {
	if( changed != newchange )
	    apply.setEnabled(changed = newchange);
    }

    public boolean isModified() { return changed; }

    public void applyChange() {
	int i;
	boolean errored = false;

	node.setName( oldstrings[0] = textfields[0].getText().trim() );

	for( i=0 ; i<fields.length ; i++ ) {
	    String str = textfields[i+1].getText();
	    if( str != oldstrings[i+1] ) {
		Object val = null;

		try {
		    if( "url".equals(fields[i]) ) {
			val = FieldParser.parseFileSpec(str);
		    } else if( "var".equals(fields[i]) ) {
			Vector vec = FieldParser.parseVariableList(str);
			Vector newvec = new Vector();
			for( int j=0 ; j<vec.size() ; j++ ) {
			    newvec.addElement(main.getVar((String)vec.elementAt(j)));
			}
			val = newvec;
		    } else {
			val = FieldParser.parseField(str);
			if( val instanceof FieldParser.Variable ) {
			    FieldParser.Variable fv = (FieldParser.Variable)val;
			    val = new VariableField( main.getVar(fv.name), fv.index );
			}
		    }
		    node.setField( fields[i], val );
		    oldstrings[i] = str;
		} catch( ParseException e ) {
		    ChoiceDialog.info( main, "Bad Value", e.toString() );
		    errored = true;
		} catch( IllegalArgumentException e ) {
		    ChoiceDialog.info( main, "Illegal Argument", e.toString() );
		    errored = true;
		}
	    }
	}

	main.nodeChanged(node);
	setChanged(errored);
    }

    public void actionPerformed( ActionEvent evt ) {
	Object src = evt.getSource();

	if( src instanceof Button ) {
	    if( src == apply ) {
		applyChange();
	    } else if( src == delete ) {
		setTreeNode(null);
		main.deleteNode(node);
	    }
	} else if( src instanceof TextField ) {
	    applyChange();
	}
    }

    public void textValueChanged( TextEvent evt ) {
	Object src = evt.getSource();
	for( int i=0 ; i<6 ; i++ ) {
	    if( src == textfields[i] ) {
		if( !changed )
		    setChanged( !textfields[i].getText().equals( oldstrings[i] ) );
		break;
	    }
	}
    }

    public Insets getInsets() {
	return new Insets( 4,4,4,4 );
    }

    public void paint( Graphics g ) {
	Dimension size = getSize();
	int width = size.width;
	int height = size.height;

	g.setColor( getBackground() );
	g.draw3DRect( 0,0, width-1, height-1, false );
	g.draw3DRect( 1,1, width-3, height-3, true );
    }
}
