package berkeley.cs.dmc.dmod;

import berkeley.cs.dmc.model.*;
import java.awt.*;
import java.awt.image.*;
import java.awt.event.*;
import java.util.Vector;

public class TreePanel extends Panel
implements AdjustmentListener {

    Main main;

    // Icon file
    // private static final String ICON_FILE = "ModellerIcons.gif";
    private static final int ICON_SIZE = 32;

    // Area constants
    private final int NO_WHERE = -2;
    private final int TREE_AREA = -1;
    private final int GROUP = 0;
    private final int ROTATE = 1;
    private final int TRANSLATE = 2;
    private final int MODEL = 3;
    private final int BOUNDS = 4;
    
    // Icons
    Image groupIcon;
    Image rotatIcon;
    Image transIcon;
    Image modelIcon;
    Image boundIcon;

    // Scrolling
    Scrollbar hscroll = new Scrollbar( Scrollbar.HORIZONTAL );
    Scrollbar vscroll = new Scrollbar( Scrollbar.VERTICAL );
    int scrollX = 0;
    int scrollY = 0;
    int maxScrollX;
    int maxScrollY;
    boolean scrolling = false;

    Vector rootNodes = new Vector();

    // Double buffer stuff
    Image buffer;
    int bwidth = -1;
    int bheight = -1;

    // Event handling
    int mouseOffX, mouseOffY;
    int mouseX, mouseY;
    boolean  newDrag; // Edit when done
    boolean  dragging;
    TreeNode selectedNode;
    int      areaOver;
    TreeNode nodeOver;

    public TreePanel( Main main ) {
	super( null );
	this.main = main;

	Toolkit tk = getToolkit();
	ImageProducer icons = tk.createImage( ModellerIcons.data ).getSource();

	// These should probably be ICON_SIZE rather than 32
	groupIcon = createImage(new FilteredImageSource(icons,new CropImageFilter(0,0,32,32)));
	rotatIcon = createImage(new FilteredImageSource(icons,new CropImageFilter(32,0,32,32)));
	transIcon = createImage(new FilteredImageSource(icons,new CropImageFilter(64,0,32,32)));
	modelIcon = createImage(new FilteredImageSource(icons,new CropImageFilter(96,0,32,32)));
	boundIcon = createImage(new FilteredImageSource(icons,new CropImageFilter(128,0,32,32)));

	MediaTracker tracker = new MediaTracker(this);
	tracker.addImage( groupIcon, 0 );
	tracker.addImage( rotatIcon, 1 );
	tracker.addImage( transIcon, 2 );
	tracker.addImage( modelIcon, 3 );
	tracker.addImage( boundIcon, 4 );

	try {
	    tracker.waitForAll();
	} catch( InterruptedException e ) {
	    throw new InternalError( "Interrupted loading icons for TreePanel" );
	}

	if( tracker.isErrorAny() )
	    throw new InternalError( "Error loading icons" ); // from '"+ICON_FILE+"'" );


	setForeground( SystemColor.controlText );
	setBackground( SystemColor.control );

	hscroll.addAdjustmentListener(this);
	vscroll.addAdjustmentListener(this);
	hscroll.setVisible(false);
	vscroll.setVisible(false);
	add( hscroll );
	add( vscroll );

	hscroll.setUnitIncrement( 8 );
	vscroll.setUnitIncrement( 8 );

	enableEvents( AWTEvent.MOUSE_EVENT_MASK |
		      AWTEvent.MOUSE_MOTION_EVENT_MASK );
    }

    public void updateDisplay() {
	doLayout();
	repaint();
    }

    public void addRootNode( TreeNode node ) {
	TreeNode root = main.getTree();
	// if( root != null )
	root.addChild( node );
    }

    Rectangle groupBounds = new Rectangle( 0,0,36,36 );
    Rectangle rotatBounds = new Rectangle( 0,38,36,36 );
    Rectangle transBounds = new Rectangle( 0,38*2,36,36 );
    Rectangle modelBounds = new Rectangle( 0,38*3,36,36 );
    Rectangle boundBounds = new Rectangle( 0,38*4,36,36 );
    Rectangle buttonBounds;
    {
	buttonBounds = new Rectangle( groupBounds );
	buttonBounds.add( boundBounds );
    }
    Rectangle treeAreaBounds = new Rectangle( 38,0,128,128 );

    public void doLayout() {
	Dimension size = getSize();
	Dimension hsize = hscroll.getPreferredSize();
	Dimension vsize = vscroll.getPreferredSize();
	int width, height;
	int treewidth, treeheight = 0;
	boolean von = false, hon = false;

	TreeNode root = main.getTree();

	int i,x=1,num = root.getChildCount();
	for( i=0 ; i<num ; i++ ) {
	    TreeNode node = root.getChild(i);
	    layoutTree( node, x, 1 );
	    x += node.width + 4;
	    if( treeheight < node.height )
		treeheight = node.height;
	}

	width = size.width - 38;
	height = size.height;

	treewidth = x-4;
	treeheight+=4;

	if( treeheight > height ) {
	    von = true;
	    if( vsize.width + treewidth > width )
		hon = true;
	} else if( treewidth > width ) {
	    hon = true;
	    if( hsize.height + treeheight > height )
		von = true;
	}

	if( hon ) height -= hsize.height;
	if( von ) width  -= vsize.width;

	if( von ) {
	    maxScrollY = Math.max( 0, treeheight - height );
	    scrollY = (scrollY < 0 ? 0 : (scrollY > maxScrollY ? maxScrollY : scrollY ));
	    vscroll.setValues( scrollY, height, 0, treeheight );
	    vscroll.setBounds( 38+width, 0, vsize.width, height );
	    vscroll.setBlockIncrement( height );
	} else {
	    scrollY = 0;
	}

	if( hon ) {
	    maxScrollX = Math.max( 0, treewidth - width ) ;
	    scrollX = (scrollX < 0 ? 0 : (scrollX > maxScrollX ? maxScrollX : scrollX));
	    hscroll.setValues( scrollX, width, 0, treewidth );
	    hscroll.setBounds( 38, height, width, hsize.height );
	    hscroll.setBlockIncrement( width );
	} else {
	    scrollX = -(width - treewidth) / 2;
	}

	treeAreaBounds.setSize( width, height );

	hscroll.setVisible( hon );
	vscroll.setVisible( von );
    }

    void layoutTree( TreeNode node, int x, int y ) {
	int numc = node.getChildCount();

	node.bx = x;
	node.by = y;

	if( numc == 0 || node.collapsed ) {
	    node.width = 32;
	    node.height = 32;
	    node.x = x+2;
	    node.y = y;
	} else {
	    int hgt = 0;
	    int width = 0;
	    int ny = y + 32 + 8;

	    for( int i=0 ; i<numc ; i++ ) {
		TreeNode child = node.getChild(i);
		layoutTree( child, x+width, ny );
		width += child.width + 4;
		if( child.height > hgt )
		    hgt = child.height;
	    }

	    node.width = width - 4;
	    node.height = hgt + 32 + 8;
	    node.x = x + (width-32)/2;
	    node.y = y;
	}
    }

    void drawButton( Graphics g, Image i, Rectangle r, 
		     boolean selected, boolean up,
		     Color l, Color d ) {

	if( selected ) {
	    g.setColor( Color.black );
	    g.drawRect( r.x, r.y, r.width-1, r.height-1 );
	    draw3DBorder( g, r.x+1, r.y+1, r.width-2, r.height-2, up, l, d );
	} else {
	    g.setColor( getBackground() );
	    g.drawRect( r.x, r.y, r.width-1, r.height-1 );
	    g.drawRect( r.x+1, r.y+1, r.width-3, r.height-3 );
	}

	g.drawImage( i, r.x+2, r.y+2, this );
    }

    public void update( Graphics g ) {
	paint(g);
    }

    Image nodeIcon( TreeNode node ) {
	if( node instanceof RotateNode )         return rotatIcon;
	else if( node instanceof TranslateNode ) return transIcon;
	else if( node instanceof ModelNode )     return modelIcon;
	else if( node instanceof BoundNode )     return boundIcon;
	else                                     return groupIcon;
    }

    void paintSelectedTree( Graphics g, TreeNode node, int x, int y ) {
	int numc = node.getChildCount();
	Image icon = nodeIcon( node );

	g.drawImage( icon, node.x+x-scrollX, node.y+y-scrollY, this );
	if( !node.collapsed ) {
	    for( int i=0 ; i<numc ; i++ ) {
		TreeNode child = node.getChild(i);
		g.drawLine( node.x+x+16-scrollX, node.y+y+32-scrollY, 
			    child.x+x+16-scrollX, child.y+y-scrollY );
		paintSelectedTree( g, child, x, y );
	    }
	}
    }

    void paintTree( Graphics g, TreeNode node ) {
	int numc = node.getChildCount();
	Image icon = nodeIcon( node );

	g.drawImage( icon, node.x-scrollX, node.y-scrollY, this );

	if( !node.collapsed ) {
	    for( int i=0 ; i<numc ; i++ ) {
		TreeNode child = node.getChild(i);
		if( !dragging || (child != selectedNode) ) {
		    g.drawLine( node.x+16-scrollX, node.y+32-scrollY, 
				child.x+16-scrollX, child.y-scrollY );
		    paintTree( g, child );
		}
	    }
	}
    }

    void labelNode( Graphics g, TreeNode node ) {
	g.setFont( getFont() );
	FontMetrics fm = g.getFontMetrics();
	String name = node.getName();
	int width = fm.stringWidth( name ) + 4;
	int ht = fm.getHeight()+4;
	int x = node.x - width/2 + 16 - scrollX;
	int y = node.y + 32 - scrollY;

	if( x+width > treeAreaBounds.x+treeAreaBounds.width )
	    x = treeAreaBounds.x+treeAreaBounds.width-width;
	if( x < 0 )
	    x = 0;

	g.setColor( SystemColor.info );
	g.fillRect( x+1, y+1, width-2, ht-2 );
	g.setColor( SystemColor.infoText );
	g.drawRect( x,y, width-1, ht-1 );
	g.drawString( name, x+2, y+fm.getAscent()+2 );
    }

    void highlightNode( Graphics g, TreeNode node ) {
	g.setColor( SystemColor.textHighlight );
	g.drawRect( node.x-1-scrollX, node.y-1-scrollY,
		    33,33 );
    }

    public void paint( Graphics cg ) {
	Dimension size = getSize();
	if( size.width != bwidth || size.height != bheight ) {
	    buffer = createImage( size.width, size.height );
	    bwidth = size.width;
	    bheight = size.height;
	    Graphics g = buffer.getGraphics();
	    g.setColor( getBackground() );
	    g.fillRect( 0,0, bwidth, bheight );
	    g.dispose();
	}

	Graphics g = buffer.getGraphics();

	Color l = getBackground().brighter();
	Color d = getBackground().darker();
	drawButton( g, groupIcon, groupBounds, areaOver == GROUP, true, l, d );
	drawButton( g, rotatIcon, rotatBounds, areaOver == ROTATE, true, l, d );
	drawButton( g, transIcon, transBounds, areaOver == TRANSLATE, true, l, d );
	drawButton( g, modelIcon, modelBounds, areaOver == MODEL, true, l, d );
	drawButton( g, boundIcon, boundBounds, areaOver == BOUNDS, true, l, d );

	g.setColor( getForeground() );
	g.drawRect( treeAreaBounds.x, treeAreaBounds.y, 
		    treeAreaBounds.width-1, treeAreaBounds.height-1 );

	g.setColor( SystemColor.window );
	g.fillRect( treeAreaBounds.x+1, treeAreaBounds.y+1, 
		    treeAreaBounds.width-2, treeAreaBounds.height-2 );

	g.translate( treeAreaBounds.x+1, treeAreaBounds.y+1 );
	g.clipRect( 0,0, treeAreaBounds.width-2, treeAreaBounds.height-2 );

	g.setColor( SystemColor.windowText );
	
	TreeNode treeroot = main.getTree();

	for( int i=0 ; i<treeroot.getChildCount() ; i++ ) {
	    TreeNode n = treeroot.getChild(i);
	    if( !dragging || n != selectedNode )
		paintTree( g, n );
	}

	if( nodeOver != null )
	    labelNode( g, nodeOver );

	if( dragging ) {
	    if( nodeOver != null )
		highlightNode( g, nodeOver );

	    if( selectedNode != null )
		paintSelectedTree( g, selectedNode,
				   mouseX-selectedNode.x+mouseOffX, 
				   mouseY-selectedNode.y+mouseOffY );
	} else if( !newDrag ) {
	    if( selectedNode != null )
		highlightNode( g, selectedNode );
	}

	g.dispose();
	cg.drawImage( buffer, 0,0, this );

	if( scrolling )
	    dragScroll();
    }

    public Dimension getPreferredSize() {
	return new Dimension( 640, 2+38*5 );
    }

    public Dimension getMinimumSize() {
	return new Dimension( 128, 2+38*5 );
    }

    int whatisUnder( int x, int y ) {
	if( buttonBounds.contains(x,y) ) {
	    if( groupBounds.contains(x,y) ) return GROUP;
	    if( rotatBounds.contains(x,y) ) return ROTATE;
	    if( transBounds.contains(x,y) ) return TRANSLATE;
	    if( modelBounds.contains(x,y) ) return MODEL;
	    if( boundBounds.contains(x,y) ) return BOUNDS;
	} else if( treeAreaBounds.contains(x,y) ) {
	    return TREE_AREA;
	}

	return NO_WHERE;
    }

    TreeNode nodeUnder( TreeNode node, int x, int y ) {
	if( x < node.bx || y < node.by ||
	    x > node.bx + node.width ||
	    y > node.by + node.height )
	    return null;

	if( x >= node.x && y >= node.y && x <= node.x + 32 && y <= node.y + 32 )
	    return node;

	int num = node.getChildCount();
	for( int i=0 ; i<num ; i++ ) {
	    TreeNode child = node.getChild(i);
	    if( !dragging || (child != selectedNode) ) {
		child = nodeUnder( child, x, y );
		if( child != null)
		    return child;
	    }
	}

	return null;
    }
    
    TreeNode nodeUnder( int x, int y ) {
	x -= treeAreaBounds.x+1-scrollX;
	y -= treeAreaBounds.y+1-scrollY;
	
	TreeNode root = main.getTree();
	int num = root.getChildCount();
	for( int i=0 ; i<num ; i++ ) {
	    TreeNode node = root.getChild(i);
	    if( !dragging || (node != selectedNode) ) {
		node = nodeUnder( node, x, y );
		if( node != null )
		    return node;
	    }
	}

	return null;
    }

    protected void processMouseEvent( MouseEvent evt ) {
	int x = evt.getX();
	int y = evt.getY();

	mouseX = x;
	mouseY = y;

	switch( evt.getID() ) {
	case MouseEvent.MOUSE_PRESSED:
	    dragging = true;
	    newDrag = true;
	    mouseOffX = -treeAreaBounds.x-16+scrollX;
	    mouseOffY = -treeAreaBounds.y-16+scrollY;
	    switch( whatisUnder( x, y ) ) {
	    case TREE_AREA: 
		selectedNode = nodeUnder(x,y);
		if( selectedNode != null ) {
		    mouseOffX = selectedNode.x - x;
		    mouseOffY = selectedNode.y - y;
		}
	    case NO_WHERE: // fall through
		newDrag = false;
		dragging = false;
		break;
	    case GROUP:     selectedNode = new TreeNode(); break;
	    case ROTATE:    selectedNode = new RotateNode(); break;
	    case TRANSLATE: selectedNode = new TranslateNode(); break;
	    case MODEL:     selectedNode = new VRMLModelNode(); break;
	    case BOUNDS:    selectedNode = new BoundNode(); break;
	    }
	    repaint();
	    break;
	case MouseEvent.MOUSE_RELEASED:
	    scrolling = false;

	    if( selectedNode == null )
		break;

	    if( dragging ) {
		dragging = false;
		if( whatisUnder(x,y) == TREE_AREA ) {
		    nodeOver = nodeUnder(x,y);
		    
		    if( nodeOver == null ) {
			if( selectedNode.getParent() != null )
			    selectedNode.removeParent();

			main.getTree().addChild( selectedNode );
			doLayout();
			main.treeChanged();
			if( newDrag )
			    main.selectNode( selectedNode );

		    } else if( nodeOver != selectedNode && !selectedNode.isParentOf( nodeOver ) ) {
			if( selectedNode.getParent() == null )
			    main.getTree().removeChild(selectedNode);
			
			nodeOver.addChild( selectedNode );
			doLayout();
			main.treeChanged();
			if( newDrag )
			    main.selectNode( selectedNode );

		    }
		}
		selectedNode = null;
		repaint();
	    } else {
		main.selectNode( selectedNode );
	    }
	}
    }

    protected void processMouseMotionEvent( MouseEvent evt ) {
	int x = evt.getX();
	int y = evt.getY();

	mouseX = x;
	mouseY = y;

	switch( evt.getID() ) {
	case MouseEvent.MOUSE_DRAGGED:
	    if( selectedNode == null )
		break;

	    nodeOver = nodeUnder(x,y);
	    dragging = true;
	    if( !scrolling &&
		(((x < treeAreaBounds.x+5 && scrollX > 0) ||
		  (x > treeAreaBounds.x+treeAreaBounds.width-5 &&
		   scrollX < maxScrollX)) &&
		 hscroll.isVisible()) ||
		(((y < treeAreaBounds.y+5 && scrollY > 0) ||
		  (y > treeAreaBounds.y+treeAreaBounds.height-5 &&
		   scrollY < maxScrollY)) &&
		 vscroll.isVisible()) ) {
		scrolling = true;
	    }

	    repaint();
	    break;
	case MouseEvent.MOUSE_MOVED:
	    int area = whatisUnder(x,y);
	    if( area != areaOver ) {
		areaOver = area;
		repaint();
	    }
	    TreeNode node = nodeUnder(x,y);
	    if( node != nodeOver ) {
		nodeOver = node;
		repaint();
	    }
	    break;
	}
    }

    public void adjustmentValueChanged( AdjustmentEvent e ) {
	int val = e.getValue();

	if( e.getSource() == hscroll ) {
	    scrollX = Math.min( maxScrollX, val );
	    hscroll.setValue( scrollX );
	} else {
	    scrollY = Math.min( maxScrollY, val );
	    vscroll.setValue( scrollY );
	}

	repaint();
    }

    private void dragScroll() {
	int nsx, nsy;
	int x = mouseX;
	int y = mouseY;
	int dx, dy;

	if( (dx = (treeAreaBounds.x+5 - x)/2) > 0 )
	    nsx = Math.max( 0, scrollX - dx );
	else if( (dx = (x - (treeAreaBounds.width+treeAreaBounds.x-5))/2) > 0 )
	    nsx = Math.min( maxScrollX, scrollX + dx );
	else
	    nsx = scrollX;

	if( (dy = (treeAreaBounds.y+5 - y)/2) > 0 )
	    nsy = Math.max( 0, scrollY - dy );
	else if( (dy = (y - (treeAreaBounds.height+treeAreaBounds.y-5))/2) > 0 )
	    nsy = Math.min( maxScrollY, scrollY + dy );
	else
	    nsy = scrollY;

	if( nsx == scrollX && nsy == scrollY ) {
	    scrolling = false;
	} else {
	    mouseOffX -= scrollX - nsx;
	    mouseOffY -= scrollY - nsy;
	    hscroll.setValue( scrollX = nsx );
	    vscroll.setValue( scrollY = nsy );
	    // repaint();
	    getToolkit().getSystemEventQueue().postEvent( new java.awt.event.PaintEvent( this, java.awt.event.PaintEvent.PAINT, treeAreaBounds ) );
	}
    }

    // Helper functions...

    public static final void draw3DBorder( Graphics g, int x, int y, int width, int height,
					   boolean up, Color c ) {
	draw3DBorder( g, x, y, width, height, up, c.brighter(), c.darker() );
    }

    public static final void draw3DBorder( Graphics g, int x, int y, int width, int height,
					   boolean up, Color light, Color dark ) {
	int x2 = x+width-1;
	int y2 = y+height-1;

	if( up ) {
	    g.setColor( light );
	    g.drawLine( x,y, x2-1,y );
	    g.drawLine( x,y+1, x, y2 );
	    g.setColor( dark );
	    g.drawLine( x+1, y2-1, x2-1, y2-1 );
	    g.drawLine( x2-1, y+1, x2-1, y2-2 );
	    g.setColor( Color.black );
	    g.drawLine( x, y2, x2, y2 );
	    g.drawLine( x2, y, x2, y2-1 );
	} else {
	    g.setColor( Color.black );
	    g.drawLine( x, y, x2, y );
	    g.drawLine( x, y+1, x, y2 );
	    g.setColor( dark );
	    g.drawLine( x+1, y+1, x2-1, y+1 );
	    g.drawLine( x+1, y+2, x+1, y2-1 );
	    g.setColor( light );
	    g.drawLine( x+1, y2, x2, y2 );
	    g.drawLine( x2, y+1, x2, y2-1 );
	}
    }

    /*
    public static void main( String argv[] ) {
	TreeNode root = new TreeNode();
	TreeNode tran = new TranslateNode();
	TreeNode rota = new RotateNode();
	TreeNode mode = new ModelNode();
	TreeNode boun = new BoundNode();

	root.setName( "Root" );
	tran.setName( "Translate" );
	rota.setName( "Rotate" );
	mode.setName( "Model" );
	boun.setName( "Bound" );

	root.addChild( tran );
	root.addChild( rota );
	root.addChild( mode );
	root.addChild( boun );

	TreePanel treePanel = new TreePanel();
	treePanel.addRootNode( root );

	Frame frame = new Frame( "TreePanel test" );
	frame.add( treePanel);
	frame.pack();
	frame.show();
    }
    */
}
