package berkeley.cs.dmc.dmod;

import java.awt.*;
import java.awt.event.*;
import java.util.StringTokenizer;
import java.util.Vector;
import berkeley.cs.dmc.model.*;

public class UnDefVariableDialog extends BaseDialog
implements ActionListener
{
    Frame frame;

    TextField name;
    TextField type;
    TextField phase;
    TextField args;

    Button ok;
    Button cancel;

    public UnDefVariableDialog( Frame frame, String varname ) {
	super( frame, "Define Variable", true );
	this.frame = frame;

	setLayout( new GridBagLayout() );
	GridBagConstraints c = new GridBagConstraints();
	c.gridx = c.gridy = 0;
	c.fill = c.NONE;
	c.anchor = c.WEST;
	add( new Label("Name: "), c );
	c.gridy = 1;
	add( new Label("Type: "), c );
	c.gridy = 2;
	add( new Label("Phase: "), c );
	c.gridy = 3;
	add( new Label("Args: "), c );
	
	c.gridx = 1;
	c.gridy = 0;
	c.fill = c.HORIZONTAL;
	c.weightx = 1.0;
	add( name = new TextField(15), c );
	c.gridy = 1;
	add( type = new TextField(15), c );
	c.gridy = 2;
	add( phase = new TextField(15), c );
	c.gridy = 3;
	add( args = new TextField(15), c );

	name.setText( varname );
	name.setEditable( false );

	c.gridx = 0;
	c.gridy = 4;
	c.gridwidth = 2;
	Panel buttonPanel = new Panel();
	buttonPanel.add( ok = new Button( "Ok" ) );
	buttonPanel.add( cancel = new Button( "Cancel" ) );
	add( buttonPanel, c );

	ok.addActionListener(this);
	cancel.addActionListener(this);

	pack();
	setVisible(true);
    }

    protected boolean cancelled;

    public boolean isCancelled() { return cancelled; }
    public String getType() { return type.getText(); }
    public float getPhase() {
	return new Float( phase.getText() ).floatValue();
    }
    public VariableDefinition getVariableDefinition() {
	VariableDefinition def = new VariableDefinition( name.getText(), getType() );
	def.setPhase( getPhase() );
	Vector args = getArgs();
	for( int i=0 ; i<args.size() ; i++ )
	    def.addArg( args.elementAt(i) );
	return def;
    }

    public Vector getArgs() {
	StringTokenizer st = new StringTokenizer( args.getText(), ", " );
	Vector vec = new Vector();
	while( st.hasMoreTokens() ) {
	    String tok = st.nextToken();
	    try {
		vec.addElement( new Integer( tok ) );
	    } catch( NumberFormatException e ) {
		try {
		    vec.addElement( new Double( tok ) );
		} catch( NumberFormatException e1 ) {
		    vec.addElement( tok );
		}
	    }
	}
	return vec;
    }

    public boolean checkArgs() {
	StringBuffer str = new StringBuffer();
	try {
	    FieldParser.parseIdentifier( type.getText() );
	} catch( ParseException e ) {
	    str.append( "Illegal type: "+type.getText() );
	}
	try {
	    new Float( phase.getText() );
	} catch( NumberFormatException e ) {
	    if( str.length() != 0 ) str.append('\n');
	    str.append( "Illegal phase: "+phase.getText() );
	}
	if( str.length() != 0 ) {
	    ChoiceDialog.info( frame, "Illegal Arguments", new String(str) );
	    return false;
	} else {
	    return true;
	}
    }

    public void actionPerformed( ActionEvent evt ) {
	Object src = evt.getSource();
	if( src == ok ) {
	    if( checkArgs() ) {
		cancelled = false;
		setVisible(false);
		dispose();
	    }
	} else if( src == cancel ) {
	    cancelled = true;
	    setVisible(false);
	    dispose();
	}
    }
}
