package berkeley.cs.dmc.dmod;

import java.awt.*;
import java.awt.event.*;
import java.util.*;
import berkeley.cs.dmc.model.*;

public class VariableEditDialog extends BaseDialog
implements ActionListener, ItemListener, TextListener
{
    Main main;

    Choice names;
    TextField type;
    TextField phase;
    TextField args;

    Button ok;
    Button apply;
    Button cancel;

    VariableDefinition vardef;
    boolean modified = false;

    public VariableEditDialog( Main main ) {
	super( main, "Edit Variable", true );
	this.main = main;

	setLayout( new GridBagLayout() );
	GridBagConstraints c = new GridBagConstraints();
	c.gridx = c.gridy = 0;
	c.fill = c.NONE;
	c.anchor = c.WEST;
	add( new Label("Name: "), c );
	c.gridy = 1;
	add( new Label("Type: "), c );
	c.gridy = 2;
	add( new Label("Phase: "), c );
	c.gridy = 3;
	add( new Label("Args: "), c );
	
	c.gridx = 1;
	c.gridy = 0;
	c.fill = c.HORIZONTAL;
	c.weightx = 1.0;
	add( names = new Choice(), c );
	c.gridy = 1;
	add( type = new TextField(15), c );
	c.gridy = 2;
	add( phase = new TextField(15), c );
	c.gridy = 3;
	add( args = new TextField(15), c );

	Enumeration enum = main.getVars().keys();
	Vector vec = new Vector();
	while( enum.hasMoreElements() )
	    vec.addElement( enum.nextElement() );
	
	int inc, lim, si, i, j, size = vec.size();

        for( inc=1 ; inc < size ; inc = 3*inc + 1 );

        for( ; inc > 0 ; inc = (inc - 1) / 3 ) {
	    lim = size - inc;

	    for( si=0 ; si<inc ; si++ ) {
		for( i=si ; i<lim ; i+=inc ) {
		    for( j=i ; swapInOrder( vec, j, j+inc ) ; j -= inc ) {
			if( j < inc ) break;
		    }
		}
	    }
        }

	for( i=0 ; i<vec.size() ; i++ )
	    names.addItem( (String)vec.elementAt(i) );

	c.gridx = 0;
	c.gridy = 4;
	c.gridwidth = 2;
	Panel buttonPanel = new Panel();
	buttonPanel.add( ok = new Button( "Ok" ) );
	buttonPanel.add( apply = new Button( "Apply" ) );
	buttonPanel.add( cancel = new Button( "Cancel" ) );
	add( buttonPanel, c );

	names.addItemListener(this);
	ok.addActionListener(this);
	apply.addActionListener(this);
	cancel.addActionListener(this);

	type.addTextListener(this);
	phase.addTextListener(this);
	args.addTextListener(this);

	selectVariable( names.getSelectedItem() );

	pack();
	setVisible(true);
    }

    public static boolean swapInOrder( Vector vec, int i, int j ) {
	String stri = (String)vec.elementAt(i);
	String strj = (String)vec.elementAt(j);
	if( stri.compareTo(strj) <= 0 ) {
	    return false;
	} else {
	    vec.setElementAt(stri,j);
	    vec.setElementAt(strj,i);
	    return true;
	}
    }

    protected boolean cancelled;

    public boolean isCancelled() { return cancelled; }
    public String getType() { return type.getText(); }
    public float getPhase() {
	return new Float( phase.getText() ).floatValue();
    }
    public VariableDefinition getVariableDefinition() {
	VariableDefinition def = new VariableDefinition( names.getSelectedItem(), getType() );
	def.setPhase( getPhase() );
	Vector args = getArgs();
	for( int i=0 ; i<args.size() ; i++ )
	    def.addArg( args.elementAt(i) );
	return def;
    }

    public Vector getArgs() {
	StringTokenizer st = new StringTokenizer( args.getText(), ", " );
	Vector vec = new Vector();
	while( st.hasMoreTokens() ) {
	    String tok = st.nextToken();
	    try {
		vec.addElement( new Integer( tok ) );
	    } catch( NumberFormatException e ) {
		try {
		    vec.addElement( new Double( tok ) );
		} catch( NumberFormatException e1 ) {
		    vec.addElement( tok );
		}
	    }
	}
	return vec;
    }

    public boolean checkArgs() {
	StringBuffer str = new StringBuffer();
	try {
	    FieldParser.parseIdentifier( type.getText() );
	} catch( ParseException e ) {
	    str.append( "Illegal type: "+type.getText() );
	}
	try {
	    new Float( phase.getText() );
	} catch( NumberFormatException e ) {
	    if( str.length() != 0 ) str.append('\n');
	    str.append( "Illegal phase: "+phase.getText() );
	}
	if( str.length() != 0 ) {
	    ChoiceDialog.info( main, "Illegal Arguments", new String(str) );
	    return false;
	} else {
	    return true;
	}
    }

    public static String argsToString( VariableDefinition var ) {
	StringBuffer str = new StringBuffer();

	for( int i=0 ; i<var.getArgCount() ; i++ ) {
	    if( i!=0 ) str.append( ", " );
	    str.append( var.getArg(i).toString() );
	}

	return new String(str);
    }

    boolean applyChanges() {
	if( vardef != null && modified ) {
	    if( !checkArgs() )
		return false;

	    vardef.setType( getType() );
	    vardef.setPhase( getPhase() );
	    vardef.clearArgs();
	    Vector v = getArgs();
	    for( int i=0 ; i<v.size() ; i++ )
		vardef.addArg( v.elementAt(i) );
	}
	return true;
    }

    void closeDialog() {
	setVisible(false);
	dispose();
    }

    void selectVariable( String varname ) {
	VariableDefinition var = (VariableDefinition)main.getVars().get( varname );
	if( var != vardef ) {
	    if( vardef != null && modified &&
		ChoiceDialog.choose( main, "Commit Changes?",
				     "You have modified "+vardef.getName()+".\n"+
				     "Before editing another variable would you like to commit changes?\n"+
				     "(If you select \"No\" you will lose all changes you made to "+vardef.getName()+".)",
				     "Yes\nNo" ) == 0 ) {
		if( !applyChanges() )
		    names.select( vardef.getName() );
	    }
	    vardef = var;
	    type.setText( var.getType() );
	    phase.setText( ""+var.getPhase() );
	    args.setText( argsToString( var ) );
	    modified = false;
	    apply.setEnabled(false);
	}
    }
    
    public void actionPerformed( ActionEvent evt ) {
	Object src = evt.getSource();
	if( src == apply ) {
	    applyChanges();
	} else if( src == ok ) {
	    if( applyChanges() )
		closeDialog();
	} else if( src == cancel ) {
	    closeDialog();
	}
    }

    public void itemStateChanged( ItemEvent evt ) {
	selectVariable( names.getSelectedItem() );
    }

    public void textValueChanged( TextEvent evt ) {
	if( !modified && vardef != null ) {
	    Object src = evt.getSource();
	    if( src == type ) {
		modified = !type.getText().trim().equals( vardef.getType() );
	    } else if( src == phase ) {
		modified = !phase.getText().trim().equals( ""+vardef.getPhase() );
	    } else if( src == args ) {
		modified = !args.getText().trim().equals( argsToString(vardef) );
	    }
	    apply.setEnabled(modified);
	}
    }
}
