/*
 * PointLight.java - Point light source model.
 * Copyright (C) 1998 Jeffrey Ichnowski
 *
 * This program is free software.  You can redistribute it and/or
 * modify it under the terms of the GNU General Public License version
 * 2 as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
package berkeley.cs.dmc.ji3d;

public final class PointLight extends Light
{
    public Vec3  center;
    public float falloff;

    // Vec3 falloff = new Vec3( 0.1f, 0.02f, 0.02f );

    public PointLight( Vec3 color, Vec3 center, float falloff ) {
        super( color );
        this.center = center;
        this.falloff = falloff;
    }

    public PointLight( Vec3 color, Vec3 center ) {
        this( color, center, 1.0f );
    }

    protected float lightDir( Vec3 dir, Vec4 v ) {
        dir.x = v.x - center.x; // dir.sub( v, center );
        dir.y = v.y - center.y;
        dir.z = v.z - center.z;

        float len = dir.length();
        dir.scale( 1.0f / len );
        return len;
    }

    protected float attenuate( float dist ) {
        return 1.0f / (falloff * dist * dist);
    }
}
