/*
 * SurfaceDetail.java - Surface detail polygons
 * Copyright (C) 1998 Jeffrey Ichnowski
 *
 * This program is free software.  You can redistribute it and/or
 * modify it under the terms of the GNU General Public License version
 * 2 as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
package berkeley.cs.dmc.ji3d;

import java.awt.Polygon;

public final class SurfaceDetail extends Polygon
{
    public Vec3 mapped[];
    public Light shadowSource = null;
    public SurfaceDetail next = null;
    public Material material;

    public SurfaceDetail( Vec2 points[], Vec3 base, Vec3 normal, Vec3 up, Material mat )
    {
        super( new int[points.length], new int[points.length], points.length );

        material = mat;

        int i;
        Vec3 u = new Vec3();
        Vec3 v = new Vec3();

        u.cross( up, normal ); u.normalize();
        v.cross( normal, u );  v.normalize();

        Mat4 m = new Mat4( u.x,  v.x,  normal.x,  base.x,
                           u.y,  v.y,  normal.y,  base.y,
                           u.z,  v.z,  normal.z,  base.z,
                           0.0f, 0.0f, 0.0f,      1.0f );

        mapped = new Vec3[ points.length ];

        for( i=0 ; i<points.length; i++ )
            mapped[i] = new Vec3().xform( m, points[i] );
    }

    public void project( Mat4 m, int half_width, int half_height )
    {
        int i;
        Vec4 t = new Vec4();

        for( i=0 ; i<mapped.length ; i++ ) {
            t.xform( m, mapped[i] );
            xpoints[i] = (int)((t.x/t.w) /* half_width */ ) + half_width;
            ypoints[i] = (int)((t.y/t.w) /* half_height */ ) + half_height;
        }
    }
}
