/*
 * BoundNode.java - Bounding node
 * Copyright (C) 1998 Stephen Chenney and Jeffrey Ichnowski
 *
 * This program is free software.  You can redistribute it and/or
 * modify it under the terms of the GNU General Public License version
 * 2 as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
package berkeley.cs.dmc.model;

import java.io.PrintStream;
import java.util.Vector;

public class BoundNode extends TreeNode {
    Vector vars = new Vector();
    Object x = new Integer(-1);
    Object y = new Integer(-1);
    Object z = new Integer(-1);

    public Object getX() { return x; }
    public Object getY() { return y; }
    public Object getZ() { return z; }

    public void setX( Object x ) { this.x = x; }
    public void setY( Object y ) { this.y = y; }
    public void setZ( Object z ) { this.z = z; }

    public void addVar( VariableDefinition var ) {
	this.vars.addElement( var );
    }
    public Vector getVar() { return vars; }
    public VariableDefinition getVar( int index ) {
	return (VariableDefinition)vars.elementAt(index);
    }
    public int getVarCount() {
	return vars.size();
    }

    public String[] getFieldList() {
	return new String[] { "x", "y", "z", "var" };
    }
    public void setField( String field, Object value ) {
	if( field.length() == 1 ) {
	    switch( field.charAt(0) ) {
	    case 'x': setX( value ); return;
	    case 'y': setY( value ); return;
	    case 'z': setZ( value ); return;
	    }
	} else if( "var".equals(field) ) {
	    if( value instanceof Vector ) {
		vars = (Vector)value; // XXX: check that all are VariableDefinitions
	    } else if( value instanceof VariableField ) {
		VariableField vf = (VariableField)value;
		addVar( vf.getVar() );
	    } else {
		throw new IllegalArgumentException( "var must be variable" );
	    }

	    return;
	}
	super.setField( field, value );
    }

    public Object getField( String field ) {
	if( field.length() == 1 ) {
	    switch( field.charAt(0) ) {
	    case 'x': return getX();
	    case 'y': return getY();
	    case 'z': return getZ();
	    }
	} else if( "var".equals(field) )
	    return vars;

	return super.getField( field );
    }

    public String getType() { return "bound"; }
    public void printFields( PrintStream out, String indent ) {
	out.println( indent + "x " + formatValue(x) );
	out.println( indent + "y " + formatValue(y) );
	out.println( indent + "z " + formatValue(z) );
	for( int i=0 ; i<getVarCount() ; i++ )
	    out.println( indent + "var " + formatValue(getVar(i)) );
    }        
}
