/*
 * VariableDefinition.java - A variable definition
 * Copyright (C) 1998 Stephen Chenney and Jeffrey Ichnowski
 *
 * This program is free software.  You can redistribute it and/or
 * modify it under the terms of the GNU General Public License version
 * 2 as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
package berkeley.cs.dmc.model;

import java.io.PrintStream;
import java.util.Vector;

public class VariableDefinition {

    String name;
    String type;
    float  phase = 0.0f;
    Vector initarg;

    public VariableDefinition( String name, String type ) {
	this.name = name;
	this.type = type;
	initarg = new Vector();
    }

    public synchronized String getName() { return name; }
    public synchronized void setName( String name ) { this.name = name; }
    public synchronized String getType() { return type; }
    public synchronized void setType( String type ) { this.type = type; }
    public synchronized void addArg( Object obj ) { initarg.addElement( obj ); }
    public synchronized int getArgCount() { return initarg.size(); }
    public synchronized Object getArg( int i ) { return initarg.elementAt(i); }
    public synchronized void setArg( int i, Object arg ) { initarg.setElementAt( arg, i ); }
    public synchronized void clearArgs() { initarg = new Vector(); }
    public synchronized void setPhase( float p ) { phase = p; }
    public synchronized float getPhase() { return phase; }

    public synchronized void print( PrintStream out ) {
	out.print( name + " " + type + " " + phase );
	int i, size = getArgCount();
	for( i=0 ; i<size ; i++ )
	    out.print( " " + getArg(i) );

	out.println();
    }

    public String toString() {
	return name;
    }
}
