/*
 * Learn.java - learning algorithms
 * Copyright (C) 1998 Stephen Chenney and Jeffrey Ichnowski
 *
 * This program is free software.  You can redistribute it and/or
 * modify it under the terms of the GNU General Public License version
 * 2 as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
package berkeley.cs.dmc.neuralnet;

public abstract class Learn {
    public static final Learn backprop = new Backprop();

    public abstract double learn( NeuralNet net, Pattern pats[] ); // return the net error

    private static class Backprop extends Learn {
	double learning = 0.5;
	double max = 0.02;

	public double learn( NeuralNet net, Pattern pats[] ) {
	    int i, num;
	    double error = 0.0;

	    synchronized( net ) {
		net.topoSort();
		num = pats.length;
		for( i=0 ; i<num ; i++ ) {
		    Pattern pat = pats[i];

		    /* Forward propagation */
		    net.setInput(pat.input);
		    net.propagateForward();

		    /* Backward propagation  */
		    error += net.propagateBackward2( pat, learning, max );
		}
	    }
	    return error;
	}
    }
}
