/*
 * Neuron.java - A single node in a neural network
 * Copyright (C) 1998 Stephen Chenney and Jeffrey Ichnowski
 *
 * This program is free software.  You can redistribute it and/or
 * modify it under the terms of the GNU General Public License version
 * 2 as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
package berkeley.cs.dmc.neuralnet;

import java.util.Vector;

public class Neuron {
    public static final int INPUT = 0;
    public static final int HIDDEN = 1;
    public static final int OUTPUT = 2;

    public int type; // INPUT/HIDDEN/OUTPUT
    public double out; // output
    public double act; // activation
    public double iact; // initial activation
    public double bias;

    public double learn1; // learning parameters
    public boolean visited; // for topo sort
    public NeuralNet net; // the net to which this belongs

    public String name;
    protected Vector inputs = new Vector();
    protected Vector outputs = new Vector();

    public OutFunc outFunc; // output function
    public ActFunc actFunc; // activation function

    public Neuron( String name, int type, OutFunc outf, ActFunc actf ) {
	this.type = type;
	out = 0.0;
	act = 0.0;
	iact = 0.0;
	bias = 0.0;
	this.name = name;
	outFunc = outf;
	actFunc = actf;
    }

    public void eval( double act ) {
	this.act = act;
	out = outFunc.eval(act);
    }

    public void eval() {
	act = actFunc.eval(this);
	out = outFunc.eval(act);
    }

    public int numInputs() { return inputs.size(); }
    public Link getInput( int i ) { return (Link)inputs.elementAt(i); }
    public void addInput( Link s ) { inputs.addElement(s); }
    public void removeInput( int i ) { inputs.removeElementAt(i); }
    public void removeInput( Link s ) { inputs.removeElement(s); }
    
    public int numOutputs() { return outputs.size(); }
    public Link getOutput( int i ) { return (Link)outputs.elementAt(i); }
    public void addOutput( Link t ) { outputs.addElement(t); }
    public void removeOutput( int i ) { outputs.removeElementAt(i); }
    public void removeOutput( Link t ) { outputs.removeElement(t); }

    public int hashCode() {
	return name.hashCode();
    }

    public boolean equals( Object obj ) {
	if( this == obj )
	    return true;
	if( obj instanceof Neuron ) {
	    Neuron n = (Neuron)obj;
	    return name.equals(n.name) && type == n.type &&
		inputs.size() == n.inputs.size() && 
		outputs.size() == n.outputs.size() &&
		actFunc == n.actFunc && 
		outFunc == n.outFunc &&
		bias == n.bias;
	}
	
	return false;
    }

    public String toString() {
	return name + " (" +(type==INPUT?"input":(type==HIDDEN?"hidden":"output")) +
	    ", act="+act+
	    ", out="+out+
	    ", bias="+bias+
	    ", inputs="+numInputs()+
	    ", outputs="+numOutputs()+
	    ", actfunc="+actFunc.getName()+
	    ", outfunc="+outFunc.getName() + ")";
    }
}
