/*
 * Derivative.java - Required functions for integration
 * Copyright (C) 1998 Stephen Chenney and Jeffrey Ichnowski
 *
 * This program is free software.  You can redistribute it and/or
 * modify it under the terms of the GNU General Public License version
 * 2 as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
package berkeley.cs.dmc.numericalMethods;

/**
 * The Derivative interface defines functions required by the numerical
 * integration methods. These functions evaluate the derivatives of
 * the function being integrated, as well as giving the dimension (number
 * of variables).
 *
 * @see Integrator
 *
 * @author Stephen Chenney and Jeffry Ichnowski
 * @version 1.0
 */
public interface Derivative {

    /**
     * Supply the dimension of the system (number of unknown variables).
     *
     * @return The dimension
     */
    public int  getDimension();

    /**
     * The routine to evaluate the derivatives of a function. The derivatives
     * should be placed in dydx.
     *
     * @param x The ordinate value at which to evaluate the derivative.
     *          Frequently this corresponds to time.
     * @param num The number of variables in y, and the number of
     *            derivatives required. This may be the same as the
     *            dimension, but might also be less.
     * @param y The current values for the unknown variables at x.
     * @param dydx Output array for the derivative values. dydx[i] is
     *             the partial derivative of y[i] with respect to x.
     */
    public void derivs( double x, int num, double y[], double dydx[] );
}


