/*
 * MinimizeFunction.java - An interface for function minimization
 * Copyright (C) 1998 Stephen Chenney and Jeffrey Ichnowski
 *
 * This program is free software.  You can redistribute it and/or
 * modify it under the terms of the GNU General Public License version
 * 2 as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
package berkeley.cs.dmc.numericalMethods;

/**
 * An interface for functions seeking to be minimized. The only
 * method is for evaluating the function at a point.
 */
public interface MinimizeFunction {

    /**
     * The method to evaluate the function being minimized.
     *
     * @param x The point at which the function should be evaluated.
     * @param data User data passed in to the Minimize method.
     *
     * @return The function value at x.
     */
    public double evaluate(double x, Object[] data);

}

