/*
 * FxMat.java
 * Copyright (C) 1998 Stephen Chenney and Jeffrey Ichnowski
 *
 * This program is free software.  You can redistribute it and/or
 * modify it under the terms of the GNU General Public License version
 * 2 as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
package berkeley.cs.dmc.rapid2d;

public final class FxMat {
    int xx, xy;
    int yx, yy;

    public FxMat() { identity(); }
    public FxMat( FxMat m ) { set( m.xx, m.xy, m.yx, m.yy ); }
    public FxMat( Mat2D m ) { set( m.xx, m.xy, m.yx, m.yy ); }
    public FxMat( int a, int b, int c, int d ) { set( a,b,c,d ); }
    public FxMat( float a, float b, float c, float d ) { set( a,b,c,d ); }

    public final void identity() { set( 1,0, 0,1 ); }
    public final void set( float a, float b, float c, float d ) {
        xx = FixedPt.toFixed( a );
        xy = FixedPt.toFixed( b );
        yx = FixedPt.toFixed( c );
        yy = FixedPt.toFixed( d );
    }

    public final void set( int a, int b, int c, int d ) {
        xx = a; xy = b; yx = c; yy = d;
    }

    public final void transpose( FxMat v ) { xx=v.xx; xy=v.yx; yx=v.xy; yy=v.yy; }

    public final void mul( FxMat a, FxMat b ) {
        xx = FixedPt.mul( a.xx, b.xx ) + FixedPt.mul( a.xy, b.yx );
        xy = FixedPt.mul( a.xx, b.xy ) + FixedPt.mul( a.xy, b.yy );
        yx = FixedPt.mul( a.yx, b.xx ) + FixedPt.mul( a.yy, b.yx );
        yy = FixedPt.mul( a.yx, b.xy ) + FixedPt.mul( a.yy, b.yy );
    }

    public final void mulTxM( FxMat a, FxMat b ) {
        xx = FixedPt.mul( a.xx, b.xx ) + FixedPt.mul( a.yx, b.yx );
        xy = FixedPt.mul( a.xx, b.xy ) + FixedPt.mul( a.yx, b.yy );
        yx = FixedPt.mul( a.xy, b.xx ) + FixedPt.mul( a.yy, b.yx );
        yy = FixedPt.mul( a.xy, b.xy ) + FixedPt.mul( a.yy, b.yy );
    }

    public final int comp( int r, int c ) {
        if( r==0 ) {
            if     ( c==0 ) return xx;
            else if( c==1 ) return xy;
        } else if( r == 1 ) {
            if     ( c==0 ) return yx;
            else if( c==1 ) return yy;
        }

        throw new ArrayIndexOutOfBoundsException( "bad index (" + r + "," +c+ ")" );
    }

    public final String toString() {
        return "[" + FixedPt.toString( xx ) + ", "
                   + FixedPt.toString( xy ) + " / "
                   + FixedPt.toString( yx ) + ", "
                   + FixedPt.toString( yy ) + "]";
    }

}
