/*
 * FxVec.java
 * Copyright (C) 1998 Stephen Chenney and Jeffrey Ichnowski
 *
 * This program is free software.  You can redistribute it and/or
 * modify it under the terms of the GNU General Public License version
 * 2 as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
package berkeley.cs.dmc.rapid2d;

public final class FxVec
{
  public int x, y;

  /*
  static Allocator allocator = new Allocator( "FxVec" );

  public FxVec() {}
  public static FxVec alloc() {
    FxVec fx = (FxVec)allocator.alloc();
    fx.zero();
    return fx;
  }
  public static FxVec alloc( int x, int y ) {
    FxVec fx = (FxVec)allocator.alloc();
    fx.set( x, y );
    return fx;
  }
  public static FxVec alloc( FxVec v ) {
    FxVec fx = (FxVec)allocator.alloc();
    fx.set( v.x , v.y );
    return fx;
  }

  public static void free( FxVec v ) {
    allocator.free( v );
  }
  public void free() {
    allocator.free( this );
  }

  public Object clone() {
    return alloc( this );
  }
  */

  public FxVec set( int x, int y ) {
    this.x = x;
    this.y = y;
    return this;
  }

  public void add( FxVec a, FxVec b ) {
    x = a.x + b.x;
    y = a.y + b.y;
  }

  public void add( FxVec v ) { add( this, v ); }

  public void sub( FxVec a, FxVec b ) {
    x = a.x - b.x;
    y = a.y - b.y;
  }
  public void sub( FxVec v ) { sub( this, v ); }

  public void mul( FxVec a, FxVec b ) {
    x = FixedPt.mul( a.x, b.x );
    y = FixedPt.mul( a.y, b.y );
  }
  public void mul( FxVec v ) { mul( this, v ); }

  public void scale( FxVec v, int s )  {
    x = FixedPt.mul( v.x, s );
    y = FixedPt.mul( v.y, s );
  }
  public void scale( int s ) { scale( this, s ); }

  public void invScale( FxVec v, int s ) {
    x = FixedPt.div( v.x, s );
    y = FixedPt.div( v.y, s );
  }
  public void invScale( int s ) { invScale( this, s ); }

  public static int dot( FxVec a, FxVec b ) {
    return FixedPt.add( FixedPt.mul(a.x,b.x) , FixedPt.mul(a.y,b.y) );
  }
  public int dot( FxVec v ) { return dot( this, v ); }

  public int length()
  {
    try {
      return FixedPt.sqrt( dot( this, this ) );
    } catch( ArithmeticException e) {
      return FixedPt.ZERO;
    }
  }

  public void normalize( FxVec v ) throws ArithmeticException
  {
    int len = v.length();
    if( len <= 0 ) throw new ArithmeticException( "zero length vector" );
    invScale( len );
  }

  public void normalize() throws ArithmeticException {
    normalize( this );
  }

  public String toString() { return "( " + FixedPt.toString(x) + ", " + FixedPt.toString(y) + " )"; }
  public int hashCode()    { return x ^ y; }

  public boolean equals( Object anObject )
  {
    if( (anObject != null) && (anObject instanceof FxVec) ) {
      FxVec v = (FxVec)anObject;
      return (v.x == x) && (v.y == y);
    } else
      return false;
  }

    public FxVec() {}
    public FxVec( Vec2D v ) { x = FixedPt.toFixed( v.x ); y = FixedPt.toFixed( v.y ); }
    public FxVec( FxVec v ) { x = v.x; y = v.y; }
    public FxVec( int a, int b ) { set( a, b ); }

    public final void zero() { set( 0, 0 ); }

    public final int comp( int i ) {
        switch( i ) {
    	case 0: return x;
    	case 1: return y;
    	default:
    		throw new ArrayIndexOutOfBoundsException();
    	}
    }

    public final void mul( FxMat m, FxVec v ) {
    	set( FixedPt.mul( m.xx, v.x ) + FixedPt.mul( m.xy, v.y ),
    		 FixedPt.mul( m.yx, v.x ) + FixedPt.mul( m.yy, v.y ) );
    }

    public final void mulT( FxMat m, FxVec v ) {
    	set( FixedPt.mul( m.xx, v.x ) + FixedPt.mul( m.yx, v.y ),
    		 FixedPt.mul( m.xy, v.x ) + FixedPt.mul( m.yy, v.y ) );
    }
}
