/*
 * RapidTest.java
 * Copyright (C) 1998 Stephen Chenney and Jeffrey Ichnowski
 *
 * This program is free software.  You can redistribute it and/or
 * modify it under the terms of the GNU General Public License version
 * 2 as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
package berkeley.cs.dmc.rapid2d;

import java.applet.*;
import java.awt.*;
import java.util.Vector;

class BoxCanvas extends Canvas {

    // public Box box = new Box();
    public Vector boxes = new Vector();
    public Vector rotates = new Vector();
    public Vector tranlts = new Vector();
    public Vector contact = new Vector();
    Vec2D trans;
    Vec2D scale;
    Vec2D pscale = new Vec2D();

    public BoxCanvas() {
        super();
        setScale( 0,0 , 1,1 );
    }

    public void setScale( float x1, float y1, float x2, float y2 ) {
        trans = new Vec2D( -x1, -y1 );
        scale = new Vec2D( 1.0f / (x2 - x1), 1.0f / (y2 - y1) );
    }

    final Point xformVec( Vec2D v, Mat2D r, Vec2D t ) {
        Vec2D t1 = new Vec2D();
        Vec2D t2 = new Vec2D();

        t1.mul( r, v );
        t2.add( t1, t );

        t2.add( t2, trans );
        t1.mul( t2, pscale );

        return new Point( (int)t1.x, (int)t1.y );
    }

    public void paintBox( Box b, boolean cont[], Graphics g, Mat2D r, Vec2D t ) {
        if( b == null ) return;

        Point p1, p2, p3, p4;
        Vec2D nt = new Vec2D();
        Mat2D nr = new Mat2D();

        nt.mul( r, b.translate );
        nt.add( nt, t );

        nr.mul( r, b.rotate );

        p1 = xformVec( b.dim, nr, nt );
        p2 = xformVec( new Vec2D( b.dim.x,-b.dim.y ), nr, nt );
        p3 = xformVec( new Vec2D(-b.dim.x,-b.dim.y ), nr, nt );
        p4 = xformVec( new Vec2D(-b.dim.x, b.dim.y ), nr, nt );

        g.setColor( Color.red );
        g.drawLine( p1.x,p1.y , p2.x,p2.y );
        g.drawLine( p2.x,p2.y , p3.x,p3.y );
        g.drawLine( p3.x,p3.y , p4.x,p4.y );
        g.drawLine( p4.x,p4.y , p1.x,p1.y );

        if( b.segs != null ) {
            int i;

            for( i=0 ; i<b.segs.size() ; i++ ) {
                Segment seg = (Segment)b.segs.elementAt(i);

                g.setColor( cont[seg.id] ? Color.yellow : Color.black );

                p1 = xformVec( seg.p1, nr, nt );
                p2 = xformVec( seg.p2, nr, nt );

                g.drawLine( p1.x,p1.y , p2.x,p2.y );
            }
        }

        paintBox( b.P, cont, g, nr, nt );
        paintBox( b.N, cont, g, nr, nt );
    }

    Dimension size = new Dimension(-1,-1);
    Image floatbuffer;

    public void update( Graphics g ) {
        Dimension sz = size();

        if( sz.height != size.height || sz.width != size.width ) {
            size = sz;
            floatbuffer = createImage( sz.width, sz.height );
        }

        Graphics bg = floatbuffer.getGraphics();

        bg.setColor( getBackground() );
        bg.fillRect( 0,0, size.width, size.height );
        bg.setColor( getForeground() );
        paint( bg );
        bg.dispose();

        g.drawImage( floatbuffer, 0,0 , this );
    }

    public void repaintNow() {
        Graphics g = getGraphics();
        if( g != null ) {
            update( g );
            g.dispose();
        }
    }

    public void paint( Graphics g ) {
        Dimension size = size();
        pscale.x = scale.x * size.width;
        pscale.y = scale.y * size.height;

        float x, y;
        int xi, yi;

        g.setColor( Color.blue );
        for( y = -trans.y ; ; y += 1.0 ) {
            yi = (int)((y + trans.y) * pscale.y);
            if( yi > size.height ) break;

            for( x = -trans.x ;; x += 1.0 ) {
                xi = (int)((x + trans.x) * pscale.x);
                if( xi > size.width ) break;

                g.drawLine( xi,yi, xi,yi );
            }
        }

        for( int i=0 ; i < boxes.size() ; i++ ) {
            Box box = (Box)boxes.elementAt(i);
            Mat2D rot = (Mat2D)rotates.elementAt(i);
            Vec2D trn = (Vec2D)tranlts.elementAt(i);
            boolean c[] = (boolean [])contact.elementAt(i);

            paintBox( box, c, g, rot, trn );
        }
    }
}

public class RapidTest extends Applet implements Runnable {

    BoxCanvas canvas;
    Box box;

    static Box makeCircle( int SEGS, float RAD ) {
        Box b = new Box();

        int i;
        float a = 2.0f * (float)Math.PI * (float)(SEGS - 1) / (float)SEGS;
        Vec2D p0, p1;

        p0 = new Vec2D( RAD * (float)Math.cos(a), RAD * (float)Math.sin(a) );

        for( i=0 ; i<SEGS ; i++ ) {
            a = 2.0f * (float)Math.PI * i / (float)SEGS;

            p1 = new Vec2D( RAD * (float)Math.cos(a), RAD * (float)Math.sin(a) );
            b.addSeg( p0, p1, i );
            p0 = new Vec2D( p1 );
        }

        return b;
    }

    Box   b1, b2;
    FxBox fb1, fb2;
    Mat2D m1, m2;
    Vec2D v1, v2;
    boolean c1[], c2[];

    public void init() {
        int i;

        canvas = new BoxCanvas();

        canvas.setScale( -10,-10, 10,10 );
/*
        box.addSeg( new Vec2D( 0,0 ) , new Vec2D( 5,5 ), 0 );
        box.addSeg( new Vec2D( 5,5 ) , new Vec2D( 4,6 ), 1 );
        box.addSeg( new Vec2D( 4,6 ) , new Vec2D(-4,6 ), 2 );
        box.addSeg( new Vec2D(-4,6 ) , new Vec2D(-5,5 ), 3 );
        box.addSeg( new Vec2D(-5,5 ) , new Vec2D( 0,0 ), 4 );
*/
        b1 = makeCircle( 5, 3 );
        b1.buildHierarchy();
        fb1 = new FxBox( b1 );
        fb1.print( System.out );
        canvas.boxes.addElement( b1 );
        canvas.rotates.addElement( m1 = new Mat2D() );
        canvas.tranlts.addElement( v1 = new Vec2D( 2,3 ) );
        canvas.contact.addElement( c1 = new boolean[5] );

        b2 = makeCircle( 128, 4 );
        b2.buildHierarchy();
        fb2 = new FxBox( b2 );
        canvas.boxes.addElement( b2 );
        canvas.rotates.addElement( m2 = new Mat2D() );
        canvas.tranlts.addElement( v2 = new Vec2D( -2,-2 ) );
        canvas.contact.addElement( c2 = new boolean[128] );

        setLayout( new BorderLayout() );
        add( "Center", canvas );
    }

    Thread thread = null;

    public synchronized void start() {
        if( thread == null ) {
            thread = new Thread( this );
            thread.start();
        }
    }

    public synchronized void stop() {
        if( thread != null ) {
            thread.stop();
            thread = null;
        }
    }

    public void run() {
        int i;
        float angle = 0.0f;
        long starttime = System.currentTimeMillis();
        long pausetime;

        while( true ) {

            m1.xx = m1.yy = (float)Math.cos( angle );
            m1.yx = -(m1.xy = (float)Math.sin( angle ));

            m2.xx = m2.yy = (float)Math.cos( angle * -.5 );
            m2.yx = -(m2.xy = (float)Math.sin( angle * -.5 ));

            angle += (2.0 * Math.PI / 90);

            if( angle > (4.0 * Math.PI) )
                angle -= (4.0 * Math.PI);

            Vector cont = new Vector();
            FxBox.collide( cont, new FxMat(m1), new FxVec(v1), fb1,
                                 new FxMat(m2), new FxVec(v2), fb2 );

            for( i=0 ; i<c1.length ; i++ ) c1[i] = false;
            for( i=0 ; i<c2.length ; i++ ) c2[i] = false;

            for( i=0 ; i<cont.size() ; i++ ) {
                Contact c = (Contact)cont.elementAt(i);

                // System.out.println( " box1." + c.id1 + " <-> box2." + c.id2 );
                c1[c.id1] = true;
                c2[c.id2] = true;
            }

            pausetime = 33 - (System.currentTimeMillis() - starttime);
            if( pausetime > 0 ) {
                try { Thread.sleep( pausetime ); } catch( InterruptedException e ) {}
            }
            starttime = System.currentTimeMillis();

            canvas.repaintNow();
        }
    }
}
