/*
 * Vec2D.java
 * Copyright (C) 1998 Stephen Chenney and Jeffrey Ichnowski
 *
 * This program is free software.  You can redistribute it and/or
 * modify it under the terms of the GNU General Public License version
 * 2 as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
package berkeley.cs.dmc.rapid2d;

public final class Vec2D {
    public float x, y;

    public Vec2D() { set( 0, 0 ); }
	public Vec2D( Vec2D v ) { set( v.x, v.y ); }
	public Vec2D( float a, float b ) { set( a, b ); }

	public final void set( float x, float y ) { this.x = x; this.y = y; }
	public final void zero() { set( 0, 0 ); }

	public final float comp( int i ) {
	    switch( i ) {
		case 0: return x;
		case 1: return y;
		default:
			throw new ArrayIndexOutOfBoundsException();
		}
	}

	public final void add( Vec2D a, Vec2D b )    { x = a.x + b.x;  y = a.y + b.y; }
	public final void sub( Vec2D a, Vec2D b )    { x = a.x - b.x;  y = a.y - b.y; }
	public final void mul( Vec2D a, Vec2D b )    { x = a.x * b.x;  y = a.y * b.y; }
	public final void scale( Vec2D a, float s ) { x = a.x * s;    y = a.y * s; }

	public final void mul( Mat2D m, Vec2D v ) {
		set( m.xx * v.x + m.xy * v.y,
			 m.yx * v.x + m.yy * v.y );
	}

	public final void mulT( Mat2D m, Vec2D v ) {
		set( m.xx * v.x + m.yx * v.y,
			 m.xy * v.x + m.yy * v.y );
	}

	public final String toString() {
		return "( " + (float)x + ", " + (float)y + " )";
	}
}
