/*
 * SysToVar.java
 * Copyright (C) 1998 Stephen Chenney and Jeffrey Ichnowski
 *
 * This program is free software.  You can redistribute it and/or
 * modify it under the terms of the GNU General Public License version
 * 2 as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
package berkeley.cs.dmc.system;

import java.io.PrintWriter;
import java.io.FileWriter;


/**
 * Application class to convert a DynamicsSystem directly
 * into a DynamicsVariable. For use in debugging, primarily.
 *
 * @author Stephen Chenney
 * @version 1.0
 */
public class SysToVar {

    private static String getName(String s) {
	int old_index = s.indexOf("System");
	if ( old_index == -1 )
	    return null;
	return s.substring(0, old_index) + "Variable"
	       + s.substring(old_index + 6, s.length());
    }

    /**
     * Main routine. Should be invoked as:
     *
     * <PRE>
     * java berkeley.cs.dmc.system.SysToVar <DynamicsSystem>
     * </PRE>
     *
     * DynamicsSystem must be th ename of a class that implements
     * the DynamicsSystem interface, and must contain the string "System"
     * in its name.
     *
     * A new class will be created, that implements the DynamicsVariable
     * interface, and with a name derived from DynamicsSystem by
     * replacing the string "System" with the string "Variable".
     */
    public static void main(String[] args) {
	if ( args.length < 1 )
	{
	    System.err.println("Usage: java "
	              + "berkeley.cs.dmc.system.SysToVar <DynamicsSystem>");
	    System.exit(1);
	}

	try {
	    String new_name = getName(args[0]);
	    if ( new_name == null )
	    {
		System.err.println(
		    "System class name must contain the string \"System\"");
		System.exit(1);
	    }

	    PrintWriter out = new PrintWriter(new FileWriter(new_name+".java"));

	    out.println("import berkeley.cs.dmc.system.DynamicsVariable;");
	    out.println("public class " + new_name + " extends " + args[0]
	              + " implements DynamicsVariable {");
	    out.println("    public float newState(double last_time,");
	    out.println("                          double[] last_params,");
	    out.println("                          double last_seen,");
	    out.println("                          double[] seen_params,");
	    out.println("                          double new_time,");
	    out.println("                          double[] new_params) {");
	    out.println("        super.evaluate(last_time, last_params,");
	    out.println("                       new_time, new_params);");
	    out.println("        return 1.0f;");
	    out.println("    }");
	    out.println("}");

	    out.close();

	} catch (Exception e) {
	    System.err.println(e);
	    System.exit(1);
	}

	System.exit(0);
    }
}

