/*
 * ClassData.java
 * Copyright (C) 1998 Stephen Chenney and Jeffrey Ichnowski
 *
 * This program is free software.  You can redistribute it and/or
 * modify it under the terms of the GNU General Public License version
 * 2 as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
package berkeley.cs.dmc.tovrml;

import java.util.Vector;
import berkeley.cs.dmc.system.DynamicsVariable;

class ClassData {
    private String           name = null;
    private DynamicsVariable instance = null;

    ClassData(String name) {
	this.name = name;

	Class the_class = null;
	
	try { 
	    the_class = Class.forName(name);
	    instance = (DynamicsVariable)the_class.newInstance();
	} catch( ClassNotFoundException e ) {
	    System.err.println( "Class "+ name + " not found." );
	    System.exit(1);
	} catch( InstantiationException e ) {
	    System.err.println( "Could not instantiate class. "
			      + e.getMessage() );
	    System.exit(2);
	} catch( IllegalAccessException e ) {
	    System.err.println( "Illegal access. " + e.getMessage() );
	    System.exit(3);
	} catch( ClassCastException e ) {
	    System.err.println("Class " + name
	             + " does not implement dynamics.system.DynamicsVariable");
	    System.exit(4);
	}
    }

    String getName() {
	return name;
    }

    DynamicsVariable getInstance() {
	return instance;
    }
}


