/*
 * ConstMFVec3f.java
 * Copyright (C) 1998 Stephen Chenney and Jeffrey Ichnowski
 *
 * This program is free software.  You can redistribute it and/or
 * modify it under the terms of the GNU General Public License version
 * 2 as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
package berkeley.cs.dmc.vrml.field;

import berkeley.cs.dmc.vrml.ConstMField;

public class ConstMFVec3f extends ConstMField {
    private float array[];

    public ConstMFVec3f(float vec3s[][]) {
	int size = vec3s.length;
	float c[] = new float[size*3];
	int i,j;

	for( i=j=0 ; i<size ; i++ ) {
	    c[j++] = vec3s[i][0];
	    c[j++] = vec3s[i][1];
	    c[j++] = vec3s[i][2];
	}
	
	this.array = c;
    }

    public ConstMFVec3f(float vec3s[]) {
	this( vec3s.length / 3, vec3s );
    }

    public ConstMFVec3f(int size, float vec3s[]) {
	System.arraycopy( vec3s, 0, this.array = new float[size*3], 0, size*3 );
    }

    public int getSize() { return array.length / 3; }

    public void getValue(float vec3s[][]) {
	int i,j;
	float v[] = array;
	int numc = Math.min( vec3s.length, getSize() );

	for( i=j=0; i<numc ; i++ ) {
	    vec3s[i][0] = v[j++];
	    vec3s[i][1] = v[j++];
	    vec3s[i][2] = v[j++];
	}
    }

    public void getValue(float vec3s[]) {
	System.arraycopy( array, 0, vec3s, 0,
			  Math.min( vec3s.length, array.length ) );
    }

    public void get1Value(int index, float vec3s[]) {
	int i = index*3;
	float v[] = this.array;

	vec3s[0] = v[i];
	vec3s[1] = v[i+1];
	vec3s[2] = v[i+2];
    }
	
    public void get1Value(int index, SFVec3f vec ) {
	int i = index*3;
	float v[] = this.array;
	vec.setValue(v[i], v[i+1], v[i+2]);
    }

    public String toString() {
	int count = array.length;

        if( count == 0 )
            return "[]";
        else {
            StringBuffer str = new StringBuffer();
            str.append( "[ " );
            for( int i=0 ; i<count ; i++ )
                str.append( array[i] ).append( " " );
            str.append( "]" );
            return new String( str );
        }
    }
}
