/*
 * MFColor.java
 * Copyright (C) 1998 Stephen Chenney and Jeffrey Ichnowski
 *
 * This program is free software.  You can redistribute it and/or
 * modify it under the terms of the GNU General Public License version
 * 2 as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
package berkeley.cs.dmc.vrml.field;

public class MFColor extends berkeley.cs.dmc.vrml.MField {

    int count;
    float array[];


    private final void boundcheck( int i ) {
        if( i >= count )
            throw new ArrayIndexOutOfBoundsException( "index " + i + " >= " + count );
    }

    public MFColor() { clear(); }
    public MFColor( float colors[][] ) { setValue( colors ); }
    public MFColor( float colors[] ) { setValue( colors ); }
    public MFColor( int size, float colors[] ) { setValue( size, colors ); }

    public synchronized int getSize() { return count; }
    public synchronized void clear() { count = 0; array = new float[1]; }
    public synchronized void delete( int index ) {
        boundcheck( index );
        count--;

        System.arraycopy( array, (index+1)*3, array, index*3, (count - index) * 3 );
    }


    public synchronized void getValue( float colors[][] ) {
        int i,j;
        for( i=j=0 ; i<count ; i++ ) {
            colors[i][0] = array[j++];
            colors[i][1] = array[j++];
            colors[i][2] = array[j++];
        }
    }

    public synchronized void getValue( float colors[] ) {
        System.arraycopy( array, 0, colors, 0, count*3 );
    }

    public synchronized void get1Value( int index, float colors[] ) {
        boundcheck( index );
        index *= 3;
        colors[0] = array[index];
        colors[1] = array[index+1];
        colors[2] = array[index+2];
    }

    public synchronized void get1Value( int index, SFColor color ) {
        boundcheck( index );
        index *= 3;
        color.setValue( array[index],
                        array[index+1],
                        array[index+2] );
    }

    public synchronized void setValue( float colors[][] ) {
        count = colors.length;

        array = new float[ count*3];
        int i,j;
        for( i=j=0 ; i<count ; i++ ) {
            array[j++] = colors[i][0];
            array[j++] = colors[i][1];
            array[j++] = colors[i][2];
        }
    }

    public synchronized void setValue( float colors[] ) {
        setValue( colors.length, colors );
    }

    public synchronized void setValue( int size, float colors[] ) {
        count = colors.length / 3;

        if( array.length < count*3 )
            array = new float[ count*3 ];

        System.arraycopy( colors, 0, array, 0, count*3 );
    }

    public synchronized void setValue( MFColor colors ) {
        count = colors.count;

        if( array.length < count*3 )
            array = new float[ count*3 ];

        System.arraycopy( colors.array, 0, array, 0, count*3 );
    }

    public synchronized void setValue( ConstMFColor colors ) {
	count = colors.getSize();
	if( array.length < count*3 )
	    array = new float[count*3];
	colors.getValue(array);
    }       

    public void set1Value( int index, ConstSFColor color ) {
	set1Value( index, color.getRed(), color.getGreen(), color.getBlue() );
    }

    public synchronized void set1Value( int index, SFColor color ) {
        set1Value( index, color.getRed(), color.getGreen(), color.getBlue() );
    }

    public synchronized void set1Value( int index, float red, float green, float blue ) {
        boundcheck( index );
        index *= 3;
        array[index] = red;
        array[index+1] = green;
        array[index+2] = blue;
    }

    public void addValue( ConstSFColor color ) {
	addValue( color.getRed(), color.getGreen(), color.getBlue() );
    }

    public synchronized void addValue( SFColor color ) {
        addValue( color.getRed(), color.getGreen(), color.getBlue() );
    }

    public synchronized void addValue( float red, float green, float blue ) {
        int i = count * 3;

        if( i+3 > array.length ) {
            float tmp[] = new float[i*2 + 3];
            System.arraycopy( array, 0, tmp, 0, i );
            array = tmp;
        }

        array[i] = red;
        array[i+1] = green;
        array[i+2] = blue;

        count++;
    }

    public void insertValue( int index, ConstSFColor color ) {
	insertValue( index, color.getRed(), color.getGreen(), color.getBlue() );
    }

    public synchronized void insertValue( int index, SFColor color ) {
        insertValue( index, color.getRed(), color.getGreen(), color.getBlue() );
    }

    public synchronized void insertValue( int index, float red, float green, float blue ) {
        if( index > count )
            throw new ArrayIndexOutOfBoundsException( "index " + index + " > " + count );

        int i = index*3;

        if( (count*3 + 3) <= array.length ) {
            System.arraycopy( array, i, array, i+3, (count*3) - i );
        } else {
            float tmp[] = new float[ count*6 + 3 ];
            System.arraycopy( array, 0, tmp, 0, i );
            System.arraycopy( array, i, tmp, i+3, (count*3) - i );
            array = tmp;
        }

        array[i] = red;
        array[i+1] = green;
        array[i+2] = blue;
        count++;
    }

    public synchronized String toString() {
        if( count == 0 )
            return "[]";
        else {
            StringBuffer str = new StringBuffer();
            str.append( "[ " );
            for( int i=0 ; i<count*3 ; i++ ) {
                str.append( array[i] );
                str.append( " " );
            }
            str.append( "]" );
            return new String( str );
        }
    }

    public Object clone() {
        MFColor c = (MFColor)super.clone();
        c.array = new float[count];
        System.arraycopy( array, 0, c.array, 0, count*3 );
        return c;
    }
}
