/*
 * MFFloat.java
 * Copyright (C) 1998 Stephen Chenney and Jeffrey Ichnowski
 *
 * This program is free software.  You can redistribute it and/or
 * modify it under the terms of the GNU General Public License version
 * 2 as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
package berkeley.cs.dmc.vrml.field;

public class MFFloat extends berkeley.cs.dmc.vrml.MField {

    int count;
    float array[];

    public MFFloat() { clear(); }
    public MFFloat( int size, float values[] ) { setValue( size, values ); }
    public MFFloat( float values[] ) { setValue( values.length, values ); }

    private final void boundcheck( int i ) {
        if( i >= count )
            throw new ArrayIndexOutOfBoundsException( "index " + i + " >= " + count );
    }

    public int getSize() { return count; }
    public synchronized void clear() { count = 0; array = new float[1]; }
    public synchronized void delete( int index ) {
        boundcheck(index);
        count--;
        System.arraycopy( array, index+1, array, index, count - index );
    }

    public void getValue( float values[] ) {
        System.arraycopy( array, 0, values, 0, count );
    }

    public synchronized float get1Value( int index ) {
        boundcheck(index);
        return array[index];
    }

    public void setValue( float values[] ) {
        setValue( values.length, values );
    }

    public synchronized void setValue( int size, float values[] ) {
        if( size > array.length )   // only allocate a new array if we need to.
            array = new float[count = size];

        System.arraycopy( values, 0, array, 0, count );
    }

    public void setValue( MFFloat value ) {
        setValue( value.count, value.array );
    }

    public synchronized void setValue( ConstMFFloat value ) {
	count = value.getSize();
	if( count > array.length )
	    array = new float[count];
	value.getValue(array);
    }

    public synchronized void set1Value( int index, float f ) {
        boundcheck( index );
        array[index]=f;
    }

    public void set1Value( int index, ConstSFFloat f ) {
	set1Value( index, f.getValue() );
    }

    public void set1Value( int index, SFFloat f ) {
        set1Value( index, f.getValue() );
    }

    public synchronized void addValue( float f ) {
        if( count == array.length ) {
            float tmp[] = new float[count*2 + 1];
            System.arraycopy( array, 0, tmp, 0, count );
            array = tmp;
        }
        array[count++] = f;
    }

    public void addValue( ConstSFFloat f ) {
	addValue( f.getValue() );
    }

    public void addValue( SFFloat f ) {
        addValue( f.getValue() );
    }

    public synchronized void insertValue( int index, float f ) {
        if( index > count )
            throw new ArrayIndexOutOfBoundsException( "index " + index + " > " + count );

        if( count == array.length ) {
            float tmp[] = new float[ count*2 + 1 ];
            System.arraycopy( array, 0, tmp, 0, index );
            System.arraycopy( array, index, tmp, index+1, count - index );
            array = tmp;
        } else {
            System.arraycopy( array, index, array, index+1, count - index );
        }

        array[index] = f;
        count++;
    }

    public void insertValue( int index, ConstSFFloat f ) {
	insertValue( index, f.getValue() );
    }

    public void insertValue( int index, SFFloat f ) {
        insertValue( index, f.getValue() );
    }

    public synchronized String toString() {
        if( count == 0 )
            return "[]";
        else {
            StringBuffer str = new StringBuffer();
            str.append( "[ " );
            for( int i=0 ; i<count ; i++ )
                str.append( array[i] ).append( " " );
            str.append( "]" );
            return new String( str );
        }
    }

    public synchronized Object clone() {
        MFFloat f = (MFFloat)super.clone();
        f.array = new float[count];
        System.arraycopy( array, 0, f.array, 0, count );
        return f;
    }
}
