/*
 * MFInt32.java
 * Copyright (C) 1998 Stephen Chenney and Jeffrey Ichnowski
 *
 * This program is free software.  You can redistribute it and/or
 * modify it under the terms of the GNU General Public License version
 * 2 as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
package berkeley.cs.dmc.vrml.field;

public class MFInt32 extends berkeley.cs.dmc.vrml.MField {

    int count;
    int array[];

    public MFInt32() { clear(); }
    public MFInt32( int size, int values[] ) { setValue( size, values ); }
    public MFInt32( int values[] ) { setValue( values.length, values ); }

    private final void boundcheck( int i ) {
        if( i >= count )
            throw new ArrayIndexOutOfBoundsException( "index " + i + " >= " + count );
    }

    public int getSize() { return count; }
    public synchronized void clear() { count = 0; array = new int[1]; }
    public synchronized void delete( int index ) {
        boundcheck(index);
        count--;
        System.arraycopy( array, index+1, array, index, count - index );
    }

    public void getValue( int values[] ) {
        System.arraycopy( array, 0, values, 0, count );
    }

    public synchronized int get1Value( int index ) {
        boundcheck(index);
        return array[index];
    }

    public void setValue( int values[] ) {
        setValue( values.length, values );
    }

    public synchronized void setValue( int size, int values[] ) {
        if( size > array.length )   // only allocate a new array if we need to.
            array = new int[count = size];

        System.arraycopy( values, 0, array, 0, count );
    }

    public void setValue( MFInt32 value ) {
        setValue( value.count, value.array );
    }

    public synchronized void setValue( ConstMFInt32 value ) {
	count = value.getSize();
	if( count > array.length )
	    array = new int[count];
	value.getValue( array );
    }

    public synchronized void set1Value( int index, int i ) {
        boundcheck( index );
        array[index]=i;
    }

    public void set1Value( int index, ConstSFInt32 i ) {
	set1Value( index, i.getValue() );
    }

    public void set1Value( int index, SFInt32 i ) {
        set1Value( index, i.getValue() );
    }

    public synchronized void addValue( int i ) {
        if( count == array.length ) {
            int tmp[] = new int[count*2 + 1];
            System.arraycopy( array, 0, tmp, 0, count );
            array = tmp;
        }
        array[count++] = i;
    }

    public void addValue( ConstSFInt32 i ) {
	addValue( i.getValue() );
    }

    public void addValue( SFInt32 i ) {
        addValue( i.getValue() );
    }

    public synchronized void insertValue( int index, int i ) {
        if( index > count )
            throw new ArrayIndexOutOfBoundsException( "index " + index + " > " + count );

        if( count == array.length ) {
            int tmp[] = new int[ count*2 + 1 ];
            System.arraycopy( array, 0, tmp, 0, index );
            System.arraycopy( array, index, tmp, index+1, count - index );
            array = tmp;
        } else {
            System.arraycopy( array, index, array, index+1, count - index );
        }

        array[index] = i;
        count++;
    }

    public void insertValue( int index, ConstSFInt32 i ) {
	insertValue( index, i.getValue() );
    }

    public void insertValue( int index, SFInt32 i ) {
        insertValue( index, i.getValue() );
    }

    public synchronized String toString() {
        if( count == 0 )
            return "[]";
        else {
            StringBuffer str = new StringBuffer();
            str.append( "[ " );
            for( int i=0 ; i<count ; i++ )
                str.append( array[i] ).append( " " );
            str.append( "]" );
            return new String( str );
        }
    }

    public synchronized Object clone() {
        MFInt32 i = (MFInt32)super.clone();
        i.array = new int[count];
        System.arraycopy( array, 0, i.array, 0, count );
        return i;
    }
}
