/*
 * MFNode.java
 * Copyright (C) 1998 Stephen Chenney and Jeffrey Ichnowski
 *
 * This program is free software.  You can redistribute it and/or
 * modify it under the terms of the GNU General Public License version
 * 2 as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
package berkeley.cs.dmc.vrml.field;

import berkeley.cs.dmc.vrml.node.BaseNode;

public class MFNode extends berkeley.cs.dmc.vrml.MField {

    int count;
    BaseNode array[];

    public MFNode() { clear(); }
    public MFNode( int size, BaseNode node[] ) { setValue( size, node ); }
    public MFNode( BaseNode node[] ) { setValue( node.length, node ); }

    private final void boundcheck( int i ) {
        if( i >= count )
            throw new ArrayIndexOutOfBoundsException( "index " + i + " >= " + count );
    }

    public int getSize() { return count; }
    public synchronized void clear() { count = 0; array = new BaseNode[1]; }
    public synchronized void delete( int index ) {
        boundcheck(index);
        count--;
        System.arraycopy( array, index+1, array, index, count - index );
    }

    public void getValue( BaseNode node[] ) {
        System.arraycopy( array, 0, node, 0, count );
    }

    public synchronized BaseNode get1Value( int index ) {
        boundcheck(index);
        return array[index];
    }

    public void setValue( BaseNode node[] ) {
        setValue( node.length, node );
    }

    public synchronized void setValue( int size, BaseNode node[] ) {
        if( size > array.length )   // only allocate a new array if we need to.
            array = new BaseNode[count = size];

        System.arraycopy( node, 0, array, 0, count );
    }

    public void setValue( MFNode node ) {
        setValue( node.count, node.array );
    }

    public synchronized void setValue( ConstMFNode node ) {
	count = node.getSize();

	if( count > array.length )
	    array = new BaseNode[count];
	else { // allow garbage collection
	    for( int i=count ; i<array.length ; i++ )
		array[i] = null;
	}

	node.getValue(array);
    }

    public synchronized void set1Value( int index, BaseNode node ) {
        boundcheck( index );
        array[index]=node;
    }

    public void set1Value( int index, ConstSFNode node ) {
	set1Value( index, node.getValue() );
    }

    public void set1Value( int index, SFNode node ) {
        set1Value( index, node.getValue() );
    }

    public synchronized void addValue( BaseNode node ) {
        if( count == array.length ) {
            BaseNode tmp[] = new BaseNode[count*2 + 1];
            System.arraycopy( array, 0, tmp, 0, count );
            array = tmp;
        }
        array[count++] = node;
    }

    public void addValue( ConstSFNode node ) {
	addValue( node.getValue() );
    }

    public void addValue( SFNode node ) {
        addValue( node.getValue() );
    }

    public synchronized void insertValue( int index, BaseNode node ) {
        if( index > count )
            throw new ArrayIndexOutOfBoundsException( "index " + index + " > " + count );

        if( count == array.length ) {
            BaseNode tmp[] = new BaseNode[ count*2 + 1 ];
            System.arraycopy( array, 0, tmp, 0, index );
            System.arraycopy( array, index, tmp, index+1, count - index );
            array = tmp;
        } else {
            System.arraycopy( array, index, array, index+1, count - index );
        }

        array[index] = node;
        count++;
    }

    public void insertValue( int index, ConstSFNode node ) {
	insertValue( index, node.getValue() );
    }

    public void insertValue( int index, SFNode node ) {
        insertValue( index, node.getValue() );
    }

    public synchronized String toString() {
        if( count == 0 )
            return "[]";
        else {
            StringBuffer str = new StringBuffer();
            str.append( "[ " );
            for( int i=0 ; i<count ; i++ )
                str.append( array[i] ).append( " " );
            str.append( "]" );
            return new String( str );
        }
    }

    public synchronized Object clone() {
        MFNode node = (MFNode)super.clone();
        node.array = new BaseNode[count];
        System.arraycopy( array, 0, node.array, 0, count );
        return node;
    }
}
