/*
 * MFRotation.java
 * Copyright (C) 1998 Stephen Chenney and Jeffrey Ichnowski
 *
 * This program is free software.  You can redistribute it and/or
 * modify it under the terms of the GNU General Public License version
 * 2 as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
package berkeley.cs.dmc.vrml.field;

public class MFRotation extends berkeley.cs.dmc.vrml.MField {

    int count;
    float array[];

    private final void boundcheck( int i ) {
        if( i >= count )
            throw new ArrayIndexOutOfBoundsException( "index " + i + " >= " + count );
    }

    public MFRotation() { clear(); }
    public MFRotation( float rotations[][] ) { setValue( rotations ); }
    public MFRotation( float rotations[] ) { setValue( rotations ); }
    public MFRotation( int size, float rotations[] ) { setValue( size, rotations ); }

    public synchronized int getSize() { return count; }
    public synchronized void clear() { count = 0; array = new float[1]; }
    public synchronized void delete( int index ) {
        boundcheck( index );
        count--;

        System.arraycopy( array, (index+1)*4, array, index*4, (count - index) * 4 );
    }


    public synchronized void getValue( float rotations[][] ) {
        int i,j;
        for( i=j=0 ; i<count ; i++ ) {
            rotations[i][0] = array[j++];
            rotations[i][1] = array[j++];
            rotations[i][2] = array[j++];
            rotations[i][3] = array[j++];
        }
    }

    public synchronized void getValue( float rotations[] ) {
        System.arraycopy( array, 0, rotations, 0, count*4 );
    }

    public synchronized void get1Value( int index, float rotations[] ) {
        boundcheck( index );
        index *= 4;
        rotations[0] = array[index];
        rotations[1] = array[index+1];
        rotations[2] = array[index+2];
	rotations[3] = array[index+3];
    }

    public synchronized void get1Value( int index, SFRotation rotation ) {
        boundcheck( index );
        index *= 4;
        rotation.setValue( array[index],
                           array[index+1],
                           array[index+2],
			   array[index+4] );
    }

    public synchronized void setValue( float rotations[][] ) {
        count = rotations.length;

        array = new float[ count*4];
        int i,j;
        for( i=j=0 ; i<count ; i++ ) {
            array[j++] = rotations[i][0];
            array[j++] = rotations[i][1];
            array[j++] = rotations[i][2];
            array[j++] = rotations[i][3];
        }
    }

    public synchronized void setValue( float rotations[] ) {
        setValue( rotations.length, rotations );
    }

    public synchronized void setValue( int size, float rotations[] ) {
        count = rotations.length / 4;

        if( array.length < count*4 )
            array = new float[ count*4 ];

        System.arraycopy( rotations, 0, array, 0, count*4 );
    }

    public synchronized void setValue( MFRotation rotations ) {
        count = rotations.count;

        if( array.length < count*4 )
            array = new float[ count*4 ];

        System.arraycopy( rotations.array, 0, array, 0, count*4 );
    }

    public synchronized void setValue( ConstMFRotation rotations ) {
	count = rotations.getSize();

	if( array.length < count*4 )
	    array = new float[count*4];

	rotations.getValue( array );
    }

    public void set1Value( int index, ConstSFRotation rotation ) {
	float r[] = new float[4];
        rotation.getValue( r );
        set1Value( index, r[0], r[1], r[2], r[3] );
    }

    public synchronized void set1Value( int index, SFRotation rotation ) {
        float r[] = new float[4];
        rotation.getValue( r );
        set1Value( index, r[0], r[1], r[2], r[3] );
    }

    public synchronized void set1Value( int index, float axisX, float axisY, float axisZ, float angle ) {
        boundcheck( index );
        index *= 4;
        array[index] = axisX;
        array[index+1] = axisY;
        array[index+2] = axisZ;
        array[index+3] = angle;
    }

    public void addValue( ConstSFRotation rotation ) {
	float r[] = new float[4];
        rotation.getValue( r );
        addValue( r[0], r[1], r[2], r[3] );
    }

    public synchronized void addValue( SFRotation rotation ) {
        float r[] = new float[4];
        rotation.getValue( r );
        addValue( r[0], r[1], r[2], r[3] );
    }

    public synchronized void addValue( float axisX, float axisY, float axisZ, float angle ) {
        int i = count * 4;

        if( i+4 > array.length ) {
            float tmp[] = new float[i*2 + 4];
            System.arraycopy( array, 0, tmp, 0, i );
            array = tmp;
        }

        array[i] = axisX;
        array[i+1] = axisY;
        array[i+2] = axisZ;
        array[i+3] = angle;

        count++;
    }

    public void insertValue( int index, ConstSFRotation rotation ) {
	float r[] = new float[4];
        rotation.getValue( r );
        insertValue( index, r[0], r[1], r[2], r[3] );
    }

    public synchronized void insertValue( int index, SFRotation rotation ) {
        float r[] = new float[4];
        rotation.getValue( r );
        insertValue( index, r[0], r[1], r[2], r[3] );
    }

    public synchronized void insertValue( int index, float axisX, float axisY, float axisZ, float angle ) {
        if( index > count )
            throw new ArrayIndexOutOfBoundsException( "index " + index + " > " + count );

        int i = index*4;

        if( (count*4 + 4) <= array.length ) {
            System.arraycopy( array, i, array, i+4, (count*4) - i );
        } else {
            float tmp[] = new float[ count*8 + 4 ];
            System.arraycopy( array, 0, tmp, 0, i );
            System.arraycopy( array, i, tmp, i+4, (count*4) - i );
            array = tmp;
        }

        array[i] = axisX;
        array[i+1] = axisY;
        array[i+2] = axisZ;
        array[i+3] = angle;
        count++;
    }

    public synchronized String toString() {
        if( count == 0 )
            return "[]";
        else {
            StringBuffer str = new StringBuffer();
            str.append( "[ " );
            for( int i=0 ; i<count*4 ; i++ ) {
                str.append( array[i] );
                str.append( " " );
            }
            str.append( "]" );
            return new String( str );
        }
    }

    public Object clone() {
        MFRotation c = (MFRotation)super.clone();
        c.array = new float[count];
        System.arraycopy( array, 0, c.array, 0, count*4 );
        return c;
    }
}
