/*
 * MFTime.java
 * Copyright (C) 1998 Stephen Chenney and Jeffrey Ichnowski
 *
 * This program is free software.  You can redistribute it and/or
 * modify it under the terms of the GNU General Public License version
 * 2 as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
package berkeley.cs.dmc.vrml.field;

public class MFTime extends berkeley.cs.dmc.vrml.MField {

    int count;
    double array[];

    public MFTime() { clear(); }
    public MFTime( int size, double values[] ) { setValue( size, values ); }
    public MFTime( double values[] ) { setValue( values.length, values ); }

    private final void boundcheck( int i ) {
        if( i >= count )
            throw new ArrayIndexOutOfBoundsException( "index " + i + " >= " + count );
    }

    public int getSize() { return count; }
    public void clear() { count = 0; array = new double[1]; }
    public synchronized void delete( int index ) {
        boundcheck(index);
        count--;
        System.arraycopy( array, index+1, array, index, count - index );
    }

    public void getValue( double values[] ) {
        System.arraycopy( array, 0, values, 0, count );
    }

    public synchronized double get1Value( int index ) {
        boundcheck(index);
        return array[index];
    }

    public void setValue( double values[] ) {
        setValue( values.length, values );
    }

    public synchronized void setValue( int size, double values[] ) {
        if( size > array.length )   // only allocate a new array if we need to.
            array = new double[count = size];

        System.arraycopy( values, 0, array, 0, count );
    }

    public void setValue( MFTime value ) {
        setValue( value.count, value.array );
    }

    public synchronized void setValue( ConstMFTime value ) {
	count = value.getSize();
	if( count > array.length )
	    array = new double[count];
	value.getValue( array );
    }

    public synchronized void set1Value( int index, double f ) {
        boundcheck( index );
        array[index]=f;
    }

    public void set1Value( int index, ConstSFTime f ) {
	set1Value( index, f.getValue() );
    }

    public void set1Value( int index, SFTime f ) {
        set1Value( index, f.getValue() );
    }

    public synchronized void addValue( double f ) {
        if( count == array.length ) {
            double tmp[] = new double[count*2 + 1];
            System.arraycopy( array, 0, tmp, 0, count );
            array = tmp;
        }
        array[count++] = f;
    }

    public void addValue( ConstSFTime f ) {
	addValue( f.getValue() );
    }

    public void addValue( SFTime f ) {
        addValue( f.getValue() );
    }

    public synchronized void insertValue( int index, double f ) {
        if( index > count )
            throw new ArrayIndexOutOfBoundsException( "index " + index + " > " + count );

        if( count == array.length ) {
            double tmp[] = new double[ count*2 + 1 ];
            System.arraycopy( array, 0, tmp, 0, index );
            System.arraycopy( array, index, tmp, index+1, count - index );
            array = tmp;
        } else {
            System.arraycopy( array, index, array, index+1, count - index );
        }

        array[index] = f;
        count++;
    }

    public void insertValue( int index, ConstSFTime f ) {
	insertValue( index, f.getValue() );
    }

    public void insertValue( int index, SFTime f ) {
        insertValue( index, f.getValue() );
    }

    public synchronized String toString() {
        if( count == 0 )
            return "[]";
        else {
            StringBuffer str = new StringBuffer();
            str.append( "[ " );
            for( int i=0 ; i<count ; i++ )
                str.append( array[i] ).append( " " );
            str.append( "]" );
            return new String( str );
        }
    }

    public Object clone() {
        MFTime f = (MFTime)super.clone();
        f.array = new double[count];
        System.arraycopy( array, 0, f.array, 0, count );
        return f;
    }
}
