/*
 * MFVec2f.java
 * Copyright (C) 1998 Stephen Chenney and Jeffrey Ichnowski
 *
 * This program is free software.  You can redistribute it and/or
 * modify it under the terms of the GNU General Public License version
 * 2 as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
package berkeley.cs.dmc.vrml.field;

public class MFVec2f extends berkeley.cs.dmc.vrml.MField {

    int count;
    float array[];


    private final void boundcheck( int i ) {
        if( i >= count )
            throw new ArrayIndexOutOfBoundsException( "index " + i + " >= " + count );
    }

    public MFVec2f() { clear(); }
    public MFVec2f( float vec2s[][] ) { setValue( vec2s ); }
    public MFVec2f( float vec2s[] ) { setValue( vec2s ); }
    public MFVec2f( int size, float vec2s[] ) { setValue( size, vec2s ); }

    public synchronized int getSize() { return count; }
    public synchronized void clear() { count = 0; array = new float[1]; }
    public synchronized void delete( int index ) {
        boundcheck( index );
        count--;

        System.arraycopy( array, (index+1)*2, array, index*2, (count - index) * 2 );
    }


    public synchronized void getValue( float vec2s[][] ) {
        int i,j;
        for( i=j=0 ; i<count ; i++ ) {
            vec2s[i][0] = array[j++];
            vec2s[i][1] = array[j++];
        }
    }

    public synchronized void getValue( float vec2s[] ) {
        System.arraycopy( array, 0, vec2s, 0, count*2 );
    }

    public synchronized void get1Value( int index, float vec2s[] ) {
        boundcheck( index );
        index *= 2;
        vec2s[0] = array[index];
        vec2s[1] = array[index+1];
    }

    public synchronized void get1Value( int index, SFVec2f vec2s ) {
        boundcheck( index );
        index *= 2;
        vec2s.setValue( array[index],
                        array[index+1] );
    }

    public synchronized void setValue( float vec2s[][] ) {
        count = vec2s.length;

        array = new float[ count*2];
        int i,j;
        for( i=j=0 ; i<count ; i++ ) {
            array[j++] = vec2s[i][0];
            array[j++] = vec2s[i][1];
        }
    }

    public synchronized void setValue( float vec2s[] ) {
        setValue( vec2s.length, vec2s );
    }

    public synchronized void setValue( int size, float vec2s[] ) {
        count = vec2s.length / 2;

        if( array.length < count*2 )
            array = new float[ count*2 ];

        System.arraycopy( vec2s, 0, array, 0, count*2 );
    }

    public synchronized void setValue( MFVec2f vec2s ) {
        count = vec2s.count;

        if( array.length < count*2 )
            array = new float[ count*2 ];

        System.arraycopy( vec2s.array, 0, array, 0, count*2 );
    }

    public synchronized void setValue( ConstMFVec2f vec2s ) {
	count = vec2s.getSize();

	if( array.length < count*2 )
            array = new float[ count*2 ];

	vec2s.getValue( array );
    }

    public void set1Value( int index, ConstSFVec2f vec2s ) {
	set1Value( index, vec2s.getX(), vec2s.getY() );
    }

    public synchronized void set1Value( int index, SFVec2f vec2s ) {
        set1Value( index, vec2s.getX(), vec2s.getY() );
    }

    public synchronized void set1Value( int index, float x, float y ) {
        boundcheck( index );
        index *= 2;
        array[index] = x;
        array[index+1] = y;
    }

    public void addValue( ConstSFVec2f vec2s ) {
	addValue( vec2s.getX(), vec2s.getY() );
    }

    public synchronized void addValue( SFVec2f vec2s ) {
        addValue( vec2s.getX(), vec2s.getY() );
    }

    public synchronized void addValue( float x, float y ) {
        int i = count * 2;

        if( i+2 > array.length ) {
            float tmp[] = new float[i*2 + 2];
            System.arraycopy( array, 0, tmp, 0, i );
            array = tmp;
        }

        array[i] = x;
        array[i+1] = y;

        count++;
    }

    public void insertValue( int index, ConstSFVec2f vec2s ) {
	insertValue( index, vec2s.getX(), vec2s.getY() );
    }

    public synchronized void insertValue( int index, SFVec2f vec2s ) {
        insertValue( index, vec2s.getX(), vec2s.getY() );
    }

    public synchronized void insertValue( int index, float x, float y ) {
        if( index > count )
            throw new ArrayIndexOutOfBoundsException( "index " + index + " > " + count );

        int i = index*2;

        if( (count*2 + 2) <= array.length ) {
            System.arraycopy( array, i, array, i+2, (count*2) - i );
        } else {
            float tmp[] = new float[ count*4 + 2 ];
            System.arraycopy( array, 0, tmp, 0, i );
            System.arraycopy( array, i, tmp, i+2, (count*2) - i );
            array = tmp;
        }

        array[i] = x;
        array[i+1] = y;
        count++;
    }

    public synchronized String toString() {
        if( count == 0 )
            return "[]";
        else {
            StringBuffer str = new StringBuffer();
            str.append( "[ " );
            for( int i=0 ; i<count*2 ; i++ ) {
                str.append( array[i] );
                str.append( " " );
            }
            str.append( "]" );
            return new String( str );
        }
    }

    public Object clone() {
        MFVec2f c = (MFVec2f)super.clone();
        c.array = new float[count];
        System.arraycopy( array, 0, c.array, 0, count*2 );
        return c;
    }
}
