/*
 * MFVec3f.java
 * Copyright (C) 1998 Stephen Chenney and Jeffrey Ichnowski
 *
 * This program is free software.  You can redistribute it and/or
 * modify it under the terms of the GNU General Public License version
 * 2 as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
package berkeley.cs.dmc.vrml.field;

public class MFVec3f extends berkeley.cs.dmc.vrml.MField {

    int count;
    float array[];

    private final void boundcheck( int i ) {
        if( i >= count )
            throw new ArrayIndexOutOfBoundsException( "index " + i + " >= " + count );
    }

    public MFVec3f() { clear(); }
    public MFVec3f( float vec3s[][] ) { setValue( vec3s ); }
    public MFVec3f( float vec3s[] ) { setValue( vec3s ); }
    public MFVec3f( int size, float vec3s[] ) { setValue( size, vec3s ); }

    public synchronized int getSize() { return count; }
    public synchronized void clear() { count = 0; array = new float[1]; }
    public synchronized void delete( int index ) {
        boundcheck( index );
        count--;

        System.arraycopy( array, (index+1)*3, array, index*3, (count - index) * 3 );
    }


    public synchronized void getValue( float vec3s[][] ) {
        int i,j;
        for( i=j=0 ; i<count ; i++ ) {
            vec3s[i][0] = array[j++];
            vec3s[i][1] = array[j++];
            vec3s[i][2] = array[j++];
        }
    }

    public synchronized void getValue( float vec3s[] ) {
        System.arraycopy( array, 0, vec3s, 0, count*3 );
    }

    public synchronized void get1Value( int index, float vec3s[] ) {
        boundcheck( index );
        index *= 3;
        vec3s[0] = array[index];
        vec3s[1] = array[index+1];
        vec3s[2] = array[index+2];
    }

    public synchronized void get1Value( int index, SFVec3f vec ) {
        boundcheck( index );
        index *= 3;
        vec.setValue( array[index],
                      array[index+1],
                      array[index+2] );
    }

    public synchronized void setValue( float vec3s[][] ) {
        count = vec3s.length;

        array = new float[ count*3];
        int i,j;
        for( i=j=0 ; i<count ; i++ ) {
            array[j++] = vec3s[i][0];
            array[j++] = vec3s[i][1];
            array[j++] = vec3s[i][2];
        }
    }

    public synchronized void setValue( float vec3s[] ) {
        setValue( vec3s.length, vec3s );
    }

    public synchronized void setValue( int size, float vec3s[] ) {
        count = vec3s.length / 3;

        if( array.length < count*3 )
            array = new float[ count*3 ];

        System.arraycopy( vec3s, 0, array, 0, count*3 );
    }

    public synchronized void setValue( MFVec3f vec3s ) {
        count = vec3s.count;

        if( array.length < count*3 )
            array = new float[ count*3 ];

        System.arraycopy( vec3s.array, 0, array, 0, count*3 );
    }

    public synchronized void setValue( ConstMFVec3f vec3s ) {
	count = vec3s.getSize();
	if( array.length < count*3 )
	    array = new float[count*3];

	vec3s.getValue( array );
    }

    public void set1Value( int index, ConstSFVec3f vec ) {
	set1Value( index, vec.getX(), vec.getY(), vec.getZ() );
    }

    public synchronized void set1Value( int index, SFVec3f vec ) {
        set1Value( index, vec.getX(), vec.getY(), vec.getZ() );
    }

    public synchronized void set1Value( int index, float x, float y, float z ) {
        boundcheck( index );
        index *= 3;
        array[index] = x;
        array[index+1] = y;
        array[index+2] = z;
    }

    public void addValue( ConstSFVec3f vec ) {
        addValue( vec.getX(), vec.getY(), vec.getZ() );
    }

    public synchronized void addValue( SFVec3f vec ) {
        addValue( vec.getX(), vec.getY(), vec.getZ() );
    }

    public synchronized void addValue( float x, float y, float z ) {
        int i = count * 3;

        if( i+3 > array.length ) {
            float tmp[] = new float[i*2 + 3];
            System.arraycopy( array, 0, tmp, 0, i );
            array = tmp;
        }

        array[i] = x;
        array[i+1] = y;
        array[i+2] = z;

        count++;
    }

    public void insertValue( int index, ConstSFVec3f vec ) {
	insertValue( index, vec.getX(), vec.getY(), vec.getZ() );
    }

    public synchronized void insertValue( int index, SFVec3f vec ) {
        insertValue( index, vec.getX(), vec.getY(), vec.getZ() );
    }

    public synchronized void insertValue( int index, float x, float y, float z ) {
        if( index > count )
            throw new ArrayIndexOutOfBoundsException( "index " + index + " > " + count );

        int i = index*3;

        if( (count*3 + 3) <= array.length ) {
            System.arraycopy( array, i, array, i+3, (count*3) - i );
        } else {
            float tmp[] = new float[ count*6 + 3 ];
            System.arraycopy( array, 0, tmp, 0, i );
            System.arraycopy( array, i, tmp, i+3, (count*3) - i );
            array = tmp;
        }

        array[i] = x;
        array[i+1] = y;
        array[i+2] = z;
        count++;
    }

    public synchronized String toString() {
        if( count == 0 )
            return "[]";
        else {
            StringBuffer str = new StringBuffer();
            str.append( "[ " );
            for( int i=0 ; i<count*3 ; i++ ) {
                str.append( array[i] );
                str.append( " " );
            }
            str.append( "]" );
            return new String( str );
        }
    }

    public Object clone() {
        MFVec3f c = (MFVec3f)super.clone();
        c.array = new float[count*3];
        System.arraycopy( array, 0, c.array, 0, count*3 );
        return c;
    }
}
