/*
 * SFImage.java
 * Copyright (C) 1998 Stephen Chenney and Jeffrey Ichnowski
 *
 * This program is free software.  You can redistribute it and/or
 * modify it under the terms of the GNU General Public License version
 * 2 as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
package berkeley.cs.dmc.vrml.field;

public class SFImage extends berkeley.cs.dmc.vrml.Field {
    int w, h;
    int c;
    byte pix[];

    public SFImage() { w=h=c=0; pix = new byte[0]; }
    public SFImage( int width, int height, int components, byte pixels[] ) {
        setValue( width, height, components, pixels );
    }

    public int getWidth() { return w; }
    public int getHeight() { return h; }
    public int getComponents() { return c; }
    public synchronized void getPixels( byte pixels[] ) {
        System.arraycopy( pix, 0, pixels, 0, pix.length );
    }

    public synchronized void setValue( int width, int height, int components, byte pixels[] ) {
        w = width;
        h = height;
        c = components;
        pix = new byte[w*h*c];
        System.arraycopy( pixels, 0, pix, 0, Math.min(pix.length, pixels.length) );
    }
    public synchronized void setValue( ConstSFImage image ) {
	w = image.getWidth();
	h = image.getHeight();
	c = image.getComponents();
	image.getPixels( pix = new byte[w*h*c] );
    }

    public synchronized void setValue( SFImage image ) {
        setValue( image.w, image.h, image.c, image.pix );
    }

    public synchronized String toString() {
        StringBuffer str = new StringBuffer();
        str.append( w ).append( " " ).append( h ).append( " " ).append( c );

        if( c == 3 ) {
            for( int i=0 ; i<pix.length ; ) {
                int v = ((pix[i] << 16) & 0xff0000) | ((pix[i+1] << 8) & 0xff00) | (pix[i+2] & 0xff);
                str.append( " 0x" ).append( Integer.toString( v, 16 ) );
            }
        } else {
            for( int i=0 ; i<pix.length ; i++ )
                str.append( " 0x" ).append( Integer.toString( pix[0], 16 ) );
        }

        return new String( str );
    }
}
