/*
 * Script.java
 * Copyright (C) 1998 Stephen Chenney and Jeffrey Ichnowski
 *
 * This program is free software.  You can redistribute it and/or
 * modify it under the terms of the GNU General Public License version
 * 2 as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
package berkeley.cs.dmc.vrml.node;

import berkeley.cs.dmc.vrml.Field;
import berkeley.cs.dmc.vrml.Event;

public abstract class Script extends BaseNode
{ 
    // This method is called before any event is generated
    public abstract void initialize();
    
    // Get a Field by name.
    //   Throws an InvalidFieldException if fieldName isn't a valid
    //   field name for a node of this type.
    protected final Field getField(String fieldName) { return null; }

    // Get an EventOut by name.
    //   Throws an InvalidEventOutException if eventOutName isn't a valid
    //   eventOut name for a node of this type.
    protected final Field getEventOut(String eventOutName) { return null; }
    
    // Get an EventIn by name.
    //   Throws an InvalidEventInException if eventInName isn't a valid
    //   eventIn name for a node of this type.
    protected final Field getEventIn(String eventInName) { return null; }
    
    // processEvents() is called automatically when the script receives 
    //   some set of events. It shall not be called directly except by its subclass.
    //   count indicates the number of events delivered.
    public abstract void processEvents(int count, Event events[]);
    
    // processEvent() is called automatically when the script receives 
    // an event. 
    public abstract void processEvent(Event event);
    
    // eventsProcessed() is called after every invocation of processEvents().
    public abstract void eventsProcessed();
	 
    // shutdown() is called when this Script node is deleted.
    public abstract void shutdown();
	 
    public abstract String toString();   // This overrides a method in Object
}
