/*
 * ParserNode.java
 * Copyright (C) 1998 Stephen Chenney and Jeffrey Ichnowski
 *
 * This program is free software.  You can redistribute it and/or
 * modify it under the terms of the GNU General Public License version
 * 2 as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
package berkeley.cs.dmc.vrml.parser;

import berkeley.cs.dmc.vrml.Field;
import berkeley.cs.dmc.vrml.field.*;
import berkeley.cs.dmc.vrml.RenderState;
import java.util.Hashtable;
import java.util.Vector;
import berkeley.cs.dmc.ji3d.*;

public abstract class ParserNode extends berkeley.cs.dmc.vrml.node.Node implements Cloneable {
    public abstract int getFieldType(String n);
    public abstract void setField(String n, Field f);
    public ParserNode cloneThru( Hashtable ismap, Hashtable newmap ) {
	try {
	    ParserNode n = (ParserNode)clone();
	    Vector vis = (Vector)ismap.get(this);
	    if( vis != null ) {
		for( int i=0 ; i<vis.size() ; i++ ) {
		    Is is = (Is)vis.elementAt(i);
		    Vector v = (Vector)newmap.get(is.protoField);
		    if( v == null ) {
			v = new Vector();
			newmap.put( is.protoField, v );
		    }
		    v.addElement( new Is( is.protoField, n, is.field ) );
		}
	    }
	    return n;
	} catch( CloneNotSupportedException e ) {
	    throw new InternalError();
	}
    }

    public abstract void dump( String indent, java.io.PrintStream out );
    public static void dumpField( String indent, java.io.PrintStream out,
				  String name, Field field ) {
	out.print(indent+name+" = ");

	if( field instanceof SFNode ) {
	    ParserNode n = (ParserNode)((SFNode)field).getValue();
	    if( n == null )
		out.println("NULL");
	    else {
		out.println();
		n.dump( indent+"  ", out );
	    }
	} else if( field instanceof MFNode ) {
	    MFNode mfn = (MFNode)field;
	    out.print("[");
	    if( mfn.getSize() == 0 )
		out.println("]");
	    else {
		out.println();
		for( int i=0 ; i<mfn.getSize() ; i++ ) {
		    ParserNode n = (ParserNode)mfn.get1Value(i);
		    n.dump( indent+"  ", out );
		}
		out.println(indent+"]");
	    }
	} else {
	    out.println( field );
	}
    }

    public static SFNode cloneSFNode( SFNode node, Hashtable ismap,
				      Hashtable newmap ) {
	if( node == null ) return null;
	ParserNode n = (ParserNode)node.getValue();
	return new SFNode( (n==null)?null:n.cloneThru( ismap, newmap ) );
    }
    public static MFNode cloneMFNode( MFNode node, Hashtable ismap,
				      Hashtable newmap ) {
	if( node == null )
	    return null;

	MFNode n = new MFNode();
	int num = node.getSize();
	for( int i=0 ; i<num ; i++ ) {
	    ParserNode pn = (ParserNode)node.get1Value(i);
	    n.addValue( pn.cloneThru( ismap, newmap ) );
	}
	return n;
    }
    public static Field cloneField( Field field ) {
	return field==null ? null : (Field)field.clone();
    }

    public static void build( MFNode mn ) {
        int num = mn.getSize();
	for( int i=0 ; i<num ; i++ ) {
	    ParserNode pn = (ParserNode)mn.get1Value(i);
	    if( pn != null )
	        pn.build();
	}
    }

    public static void build( SFNode node ) {
        ParserNode pn = (ParserNode)node.getValue();
	if( pn != null )
	    pn.build();
    }

    public static Vec3[] build( MFVec3f vec ) {
        int i,j,size = vec.getSize();
	float tmp[] = new float[size*3];
	Vec3 points[] = new Vec3[size];

	vec.getValue(tmp);

	for( i=j=0 ; i<size ; i++, j+=3 )
	    points[i] = new Vec3( tmp[j], tmp[j+1], tmp[j+2] );

	return points;
    }
    
    public static void render( MFNode mn, RenderState state ) {
        int num = mn.getSize();
	for( int i=0 ; i<num ; i++ ) {
	    ParserNode pn = (ParserNode)mn.get1Value(i);
	    if( pn != null )
	        pn.render( state );
	}
    }

    public static void render( SFNode node, RenderState state ) {
        ParserNode pn = (ParserNode)node.getValue();
	if( pn != null )
	    pn.render( state );
    }
    
    public void build() {
        System.err.println( "Warning: "+getClass().getName()+
			    ".build() not implemented" );
    }

    public void render( RenderState state ) {
        System.err.println( "Warning: "+getClass().getName()+
			    ".render() not implemented" );
    }
}
