/*
 * Proto.java
 * Copyright (C) 1998 Stephen Chenney and Jeffrey Ichnowski
 *
 * This program is free software.  You can redistribute it and/or
 * modify it under the terms of the GNU General Public License version
 * 2 as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
package berkeley.cs.dmc.vrml.parser;

import java.io.PrintStream;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Vector;
import berkeley.cs.dmc.vrml.Field;

public class Proto extends Scene {
    public String name;    
    
    Hashtable fields = new Hashtable(); /* maps names to InterfaceDefs */
    Hashtable nodeIs = new Hashtable(); /* maps ParserNodes to Vector of Iss */

    public Proto( String name ) {
	this.name = name;
    }

    public void addEventIn( Token t, Token n ) {
	fields.put( n.image, new EventDef( t.kind, n.image, false )  );
    }

    public void addEventOut( Token t, Token n ) {
	fields.put( n.image, new EventDef( t.kind, n.image, true ) );
    }

    public void addField( Token t, Token n, Field val ) {
	fields.put( n.image, new FieldDef( t.kind, n.image, val, false ) );
    }

    public void addExposedField( Token t, Token n, Field val ) {
	fields.put( n.image, new FieldDef( t.kind, n.image, val, true ) );
    }

    public int getFieldType( String name ) {
	try {
	    FieldDef field = (FieldDef)fields.get(name);
	    return (field == null) ? -1 : field.type;
	} catch( ClassCastException e ) {
	    return -1;
	}
    }

    void makeInstance( Hashtable f, Vector n ) {
	for( int i=0 ; i<nodes.size() ; i++ ) {
	    ParserNode node = (ParserNode)nodes.elementAt(i);
	    n.addElement( node.cloneThru( nodeIs, f ) );
	}
    }

    // overrides is() method in Scene
    public void is( VRMLParser parser, ParserNode node, Token id, Token is ) {
	// XXX: This needs to be modified to handle EventIn's and Out's.
	if( node != null ) {
	    InterfaceDef field = (InterfaceDef)fields.get( is.image );
	    if( field == null ) {
		parser.warning( is, "'"+is.image+"' not defined in PROTO" );
		return;
	    }

	    if( node.getFieldType( id.image ) != field.type ) {
		parser.warning( is, "'IS' field type mismatch" );
		return;
	    }

	    Is i = new Is( is.image, node, id.image );
	    field.isNodes.addElement(i);
	    Vector v = (Vector)nodeIs.get(node);
	    if( v == null )
		nodeIs.put(node, v = new Vector());
	    v.addElement(i);

	    node.setField( id.image, ((FieldDef)field).value );
	}
    }

    public boolean equals( Object obj ) {
	try {
	    Proto proto = (Proto)obj;
	    Hashtable pfields = proto.fields;
	    if( pfields.size() != fields.size() )
		return false;

	    Enumeration enum = fields.keys();
	    while( enum.hasMoreElements() ) {
		Object key = enum.nextElement();
		InterfaceDef a = (InterfaceDef)fields.get(key);
		InterfaceDef b = (InterfaceDef)pfields.get(key);
		if( a.type != b.type ||
		    !a.name.equals(b.name) )
		    return false;
		if( a instanceof EventDef ) {
		    if( ((EventDef)a).eventOut != ((EventDef)b).eventOut )
			return false;
		} else {
		    if( ((FieldDef)a).exposed != ((FieldDef)b).exposed )
			return false;
		}
	    }
	    return true;
	} catch( ClassCastException e ) {
	    return false;
	}
    }

    protected void dumpTable( String indent, String header, Hashtable table,
			      PrintStream out ) {

	Enumeration enum = table.elements();

	while( enum.hasMoreElements() ) {
	    Object o = enum.nextElement();
	    if( o instanceof FieldDef ) {
		FieldDef f = (FieldDef)o;
		out.println( indent+(f.exposed?"exposedF":"f")+"ield "+f.type+" "+f.name );
		// if( f.value != null )
		//     f.value.dump( indent, out );
	    } else {
		EventDef e = (EventDef)o;
		out.println( indent+"event"+(e.eventOut?"Out":"In")+" "+e.type+" "+e.name );
	    }
	}
    }
    
    protected void dumpInterface( String indent, PrintStream out ) {
	String indent2 = indent+"  ";
	out.println( indent+"[" );
	out.println( indent+"]" );
    }
    
    public void dump( String indent, PrintStream out ) {
	String indent2 = indent+"  ";
	out.println( indent+"PROTO "+name );
	dumpInterface( indent, out );
	out.println( indent+"{" );
	super.dump( indent2, out );
	out.println( indent+"}" );
    }
}
