/*
 * ProtoInstance.java
 * Copyright (C) 1998 Stephen Chenney and Jeffrey Ichnowski
 *
 * This program is free software.  You can redistribute it and/or
 * modify it under the terms of the GNU General Public License version
 * 2 as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
package berkeley.cs.dmc.vrml.parser;

import berkeley.cs.dmc.vrml.Field;
import java.util.Hashtable;
import java.util.Vector;

public class ProtoInstance extends ParserNode {
    Proto proto;

    Hashtable fields = new Hashtable(); // maps proto interface names to Vectors of Iss
    Vector nodes = new Vector();

    public ProtoInstance( Proto proto ) {
	this.proto = proto;
	proto.makeInstance(fields, nodes);
    }

    public int getFieldType( String field ) { return proto.getFieldType(field); }
    public void setField( String field, Field value ) {
	Vector iss = (Vector)fields.get( field );
	if( iss != null ) {
	    for( int i=0 ; i<iss.size() ; i++ ) {
		Is is = (Is)iss.elementAt(i);
		is.node.setField( is.field, value );
	    }
	}
    }
    public String getType() { return proto.name; }

    public void dump( String indent, java.io.PrintStream out ) {
	String indent2 = indent+"  ";
	java.util.Enumeration enum = fields.keys();

	out.println( indent+proto.name+" (proto) {" );
	for(int i=0 ; i<nodes.size() ; i++ ) {
	    ParserNode n = (ParserNode)nodes.elementAt(i);
	    n.dump(indent2,out);
	}
	out.println( indent+"}" );
    }

    public void build() {
	for( int i=0 ; i<nodes.size() ; i++ )
	    ((ParserNode)nodes.elementAt(i)).build();
    }

    public void render( berkeley.cs.dmc.vrml.RenderState state ) {
	for( int i=0 ; i<nodes.size() ; i++ )
	    ((ParserNode)nodes.elementAt(i)).render(state);
    }
}
