/*
 * Scene.java
 * Copyright (C) 1998 Stephen Chenney and Jeffrey Ichnowski
 *
 * This program is free software.  You can redistribute it and/or
 * modify it under the terms of the GNU General Public License version
 * 2 as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
package berkeley.cs.dmc.vrml.parser;

import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Vector;
import berkeley.cs.dmc.vrml.RenderState;

public class Scene {

    protected Hashtable defs = new Hashtable();

    protected Vector routes = new Vector();
    protected Hashtable protos = new Hashtable();
    protected Proto firstProto = null; // for EXTERNPROTO
    protected Vector nodes = new Vector();

    public void addNode( ParserNode n ) {
	if( n != null )
	    nodes.addElement(n);
    }

    public void addProto( Proto p ) {
	if( p != null ) {
	    protos.put( p.name, p );
	    if( firstProto == null )
		firstProto = p;
	}
    }

    public Proto getFirstProto() { return firstProto; }
    public Proto getProto( String name ) {
	return (Proto)protos.get( name );
    }

    public void addRoute( Route r ) {
	if( r != null )
	    routes.addElement(r);
    }

    public boolean def( Token name, ParserNode n ) {
	return defs.put( name.image, n ) == null;
    }

    public ParserNode use( Token name ) {
	ParserNode node = (ParserNode)defs.get( name.image );
	if( node == null )
	    return null;
	else
	    return new USENode( name.image, node );
    }

    public void is( VRMLParser parser, ParserNode node, Token id, Token is ) {
	parser.warning( is, "'IS' not inside PROTO" );
    }

    public void dump( String indent, java.io.PrintStream out ) {
	int i, num;
	Enumeration enum;

	for( enum = protos.elements() ; enum.hasMoreElements() ; )
	    ((Proto)enum.nextElement()).dump( indent, out );

	for( i=0, num = nodes.size() ; i<num ; i++ )
	    ((ParserNode)nodes.elementAt(i)).dump( indent, out );

	for( i=0, num = routes.size() ; i<num ; i++ )
	    ((Route)routes.elementAt(i)).dump( indent, out );
    }

    public void build() {
	int num = nodes.size();
	for( int i=0 ; i<num ; i++ )
	    ((ParserNode)nodes.elementAt(i)).build();
    }

    public void render( RenderState state ) {
	int num = nodes.size();
	for( int i=0 ; i<num ; i++ )
	    ((ParserNode)nodes.elementAt(i)).render(state);
    }

}

