/*
 * ScriptNode.java
 * Copyright (C) 1998 Stephen Chenney and Jeffrey Ichnowski
 *
 * This program is free software.  You can redistribute it and/or
 * modify it under the terms of the GNU General Public License version
 * 2 as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
package berkeley.cs.dmc.vrml.parser;

import java.util.Vector;
import berkeley.cs.dmc.vrml.Field;
import berkeley.cs.dmc.vrml.field.*;

public class ScriptNode extends ParserNode {
    public MFString url = null;
    public SFBool directOutput = null;
    public SFBool mustEvaluate = null;

    public Vector eventIns = new Vector();
    public Vector eventOuts = new Vector();
    public Vector fields = new Vector();

    public String getType() { return "Script"; }

    protected static class SEvent {
	String type;
	String name;
	String is;
	public SEvent( String type, String name, String is ) {
	    this.type = type;
	    this.name = name;
	    this.is = is;
	}

	public String toString() {
	    String str = type + " " + name;
	    if( is != null )
		str += " IS "+is;
	    return str;
	}
    }

    protected static class SField {
	String type;
	String name;
	Field value;
	String is;
	public SField( String type, String name, String is ) {
	    this.type = type;
	    this.name = name;
	    this.value = null;
	    this.is = is;
	}
	public SField( String type, String name, Field value ) {
	    this.type = type;
	    this.name = name;
	    this.value = value;
	    this.is = null;
	}
	/*
	public void dump( String indent, java.io.PrintStream out ) {
	    out.print( indent+"field "+type+" "+name );
	    if( value != null ) {
		out.println();
		value.dump( indent + INDENT_LEVEL, out );
	    } else {
		out.println( " IS "+is );
	    }
	}
	*/
    }

    public int getFieldType( String field ) {
	if( "url".equals(field) ) return VRMLParserConstants.MFSTRING;
	if( "directOutput".equals(field) ) return VRMLParserConstants.SFBOOL;
	if( "mustEvaluate".equals(field) ) return VRMLParserConstants.SFBOOL;
	return -1;
    }

    public void setField( String name, Field val ) {
	if( "url".equals(name) ) url = (MFString)val;
	else if( "directOutput".equals(name) ) directOutput = (SFBool)val;
	else if( "mustEvaluate".equals(name) ) mustEvaluate = (SFBool)val;
    }
    /*
    public void parseField( VRMLParser parser, Token field )
    throws ParseException {
	String name=field.image;

	if( "url".equals(name) )
	    url = parser.mfstring();
	else if( "directOutput".equals(name) )
	    directOutput = parser.sfbool();
	else if( "mustEvaluate".equals(name) )
	    mustEvaluate = parser.sfbool();
	else
	    super.parseField( parser, field );
    }
    */
    public void addEventIn( Token type, Token name, Token is ) {
	eventIns.addElement( new SEvent( type.image, name.image, is==null?null:is.image ) );
    }
    public void addEventOut( Token type, Token name, Token is ) {
	eventOuts.addElement( new SEvent( type.image, name.image, is==null?null:is.image ) );
    }

    public void addField( Token type, Token name, Token is ) {
	fields.addElement( new SField(type.image,name.image,is.image) );
    }
    
    public void addField( Token type, Token name, Field initial ) {
	fields.addElement( new SField(type.image,name.image,initial) );
    }

    public void dump( String indent, java.io.PrintStream out ) {
	String indent2 = indent + "  ";
	int i;

	out.println( indent + "Script {" );
	/*
	if( url != null ) {
	    out.println( indent + "url" );
	    url.dump( indent2, out );
	}
	if( directOutput != null ) {
	    out.println( indent + "directOutput" );
	    directOutput.dump( indent2, out );
	}
	if( mustEvaluate != null ) {
	    out.println( indent + "mustEvaluate" );
	    mustEvaluate.dump( indent2, out );
	}

	for( i=0 ; i<eventIns.size() ; i++ )
	    out.println( indent+"eventIn "+eventIns.elementAt(i) );

	for( i=0 ; i<eventOuts.size() ; i++ )
	    out.println( indent+"eventOut "+eventOuts.elementAt(i) );

	for( i=0 ; i<fields.size() ; i++ )
	    ((SField)fields.elementAt(i)).dump( indent, out );
	    */
	out.println( indent + "}" );
    }
}
