/*
 * Copyright (C) 1998 Stephen Chenney and Jeffrey Ichnowski
 *
 * This program is free software.  You can redistribute it and/or
 * modify it under the terms of the GNU General Public License version
 * 2 as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
package berkeley.cs.dmc.vrml.parser.node;

import java.util.Hashtable;
import berkeley.cs.dmc.vrml.parser.*;
import berkeley.cs.dmc.vrml.field.*;
import berkeley.cs.dmc.vrml.Field;

public class Appearance extends ParserNode {
    SFNode material = new SFNode();
    SFNode texture = new SFNode();
    SFNode textureTransform = new SFNode();
    public int getFieldType(String n) {
        if("material".equals(n)) return 28;
        if("texture".equals(n)) return 28;
        if("textureTransform".equals(n)) return 28;
        return -1;
    }
    public void setField(String n, Field f) {
        if("material".equals(n)) material=(SFNode)f;
        else if("texture".equals(n)) texture=(SFNode)f;
        else if("textureTransform".equals(n)) textureTransform=(SFNode)f;
    }
    public ParserNode cloneThru(Hashtable i, Hashtable f) {
        Appearance n = (Appearance)super.cloneThru(i,f);
        n.material = cloneSFNode(material,i,f);
        n.texture = cloneSFNode(texture,i,f);
        n.textureTransform = cloneSFNode(textureTransform,i,f);
        return n;
    }
    public void dump(String indent, java.io.PrintStream out) {
        String i2 = indent + "  ";
        out.println(indent+"Appearance {");
        dumpField(i2,out,"material",material);
        dumpField(i2,out,"texture",texture);
        dumpField(i2,out,"textureTransform",textureTransform);
        out.println(indent+"}");
    }
    public String getType() { return "Appearance"; }

    // Additions follow
    public void build() {
        build( material );
	build( texture );
	build( textureTransform );
    }

    public void render( berkeley.cs.dmc.vrml.RenderState state ) {
        render( material, state );
	render( texture, state );
	render( textureTransform, state );
    }
}
