/*
 * Copyright (C) 1998 Stephen Chenney and Jeffrey Ichnowski
 *
 * This program is free software.  You can redistribute it and/or
 * modify it under the terms of the GNU General Public License version
 * 2 as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
package berkeley.cs.dmc.vrml.parser.node;

import java.util.Hashtable;
import berkeley.cs.dmc.vrml.parser.*;
import berkeley.cs.dmc.vrml.field.*;
import berkeley.cs.dmc.vrml.Field;

public class Background extends ParserNode {
    MFFloat groundAngle = new MFFloat();
    MFColor groundColor = new MFColor();
    MFString backUrl = new MFString();
    MFString bottomUrl = new MFString();
    MFString frontUrl = new MFString();
    MFString leftUrl = new MFString();
    MFString rightUrl = new MFString();
    MFString topUrl = new MFString();
    MFFloat skyAngle = new MFFloat();
    MFColor skyColor = new MFColor(); {
        skyColor.addValue(0f, 0f, 0f);
    }
    public int getFieldType(String n) {
        if("groundAngle".equals(n)) return 16;
        if("groundColor".equals(n)) return 15;
        if("backUrl".equals(n)) return 20;
        if("bottomUrl".equals(n)) return 20;
        if("frontUrl".equals(n)) return 20;
        if("leftUrl".equals(n)) return 20;
        if("rightUrl".equals(n)) return 20;
        if("topUrl".equals(n)) return 20;
        if("skyAngle".equals(n)) return 16;
        if("skyColor".equals(n)) return 15;
        return -1;
    }
    public void setField(String n, Field f) {
        if("groundAngle".equals(n)) groundAngle=(MFFloat)f;
        else if("groundColor".equals(n)) groundColor=(MFColor)f;
        else if("backUrl".equals(n)) backUrl=(MFString)f;
        else if("bottomUrl".equals(n)) bottomUrl=(MFString)f;
        else if("frontUrl".equals(n)) frontUrl=(MFString)f;
        else if("leftUrl".equals(n)) leftUrl=(MFString)f;
        else if("rightUrl".equals(n)) rightUrl=(MFString)f;
        else if("topUrl".equals(n)) topUrl=(MFString)f;
        else if("skyAngle".equals(n)) skyAngle=(MFFloat)f;
        else if("skyColor".equals(n)) skyColor=(MFColor)f;
    }
    public ParserNode cloneThru(Hashtable i, Hashtable f) {
        Background n = (Background)super.cloneThru(i,f);
        n.groundAngle = (MFFloat)cloneField(groundAngle);
        n.groundColor = (MFColor)cloneField(groundColor);
        n.backUrl = (MFString)cloneField(backUrl);
        n.bottomUrl = (MFString)cloneField(bottomUrl);
        n.frontUrl = (MFString)cloneField(frontUrl);
        n.leftUrl = (MFString)cloneField(leftUrl);
        n.rightUrl = (MFString)cloneField(rightUrl);
        n.topUrl = (MFString)cloneField(topUrl);
        n.skyAngle = (MFFloat)cloneField(skyAngle);
        n.skyColor = (MFColor)cloneField(skyColor);
        return n;
    }
    public void dump(String indent, java.io.PrintStream out) {
        String i2 = indent + "  ";
        out.println(indent+"Background {");
        dumpField(i2,out,"groundAngle",groundAngle);
        dumpField(i2,out,"groundColor",groundColor);
        dumpField(i2,out,"backUrl",backUrl);
        dumpField(i2,out,"bottomUrl",bottomUrl);
        dumpField(i2,out,"frontUrl",frontUrl);
        dumpField(i2,out,"leftUrl",leftUrl);
        dumpField(i2,out,"rightUrl",rightUrl);
        dumpField(i2,out,"topUrl",topUrl);
        dumpField(i2,out,"skyAngle",skyAngle);
        dumpField(i2,out,"skyColor",skyColor);
        out.println(indent+"}");
    }
    public String getType() { return "Background"; }
}
