/*
 * Copyright (C) 1998 Stephen Chenney and Jeffrey Ichnowski
 *
 * This program is free software.  You can redistribute it and/or
 * modify it under the terms of the GNU General Public License version
 * 2 as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
package berkeley.cs.dmc.vrml.parser.node;

import java.util.Hashtable;
import berkeley.cs.dmc.vrml.parser.*;
import berkeley.cs.dmc.vrml.field.*;
import berkeley.cs.dmc.vrml.Field;

public class Billboard extends ParserNode {
    SFVec3f bboxCenter = new SFVec3f(0f, 0f, 0f);
    SFVec3f bboxSize = new SFVec3f(-1f, -1f, -1f);
    SFVec3f axisOfRotation = new SFVec3f(0f, 1f, 0f);
    MFNode children = new MFNode();
    public int getFieldType(String n) {
        if("bboxCenter".equals(n)) return 33;
        if("bboxSize".equals(n)) return 33;
        if("axisOfRotation".equals(n)) return 33;
        if("children".equals(n)) return 18;
        return -1;
    }
    public void setField(String n, Field f) {
        if("bboxCenter".equals(n)) bboxCenter=(SFVec3f)f;
        else if("bboxSize".equals(n)) bboxSize=(SFVec3f)f;
        else if("axisOfRotation".equals(n)) axisOfRotation=(SFVec3f)f;
        else if("children".equals(n)) children=(MFNode)f;
    }
    public ParserNode cloneThru(Hashtable i, Hashtable f) {
        Billboard n = (Billboard)super.cloneThru(i,f);
        n.bboxCenter = (SFVec3f)cloneField(bboxCenter);
        n.bboxSize = (SFVec3f)cloneField(bboxSize);
        n.axisOfRotation = (SFVec3f)cloneField(axisOfRotation);
        n.children = cloneMFNode(children,i,f);
        return n;
    }
    public void dump(String indent, java.io.PrintStream out) {
        String i2 = indent + "  ";
        out.println(indent+"Billboard {");
        dumpField(i2,out,"bboxCenter",bboxCenter);
        dumpField(i2,out,"bboxSize",bboxSize);
        dumpField(i2,out,"axisOfRotation",axisOfRotation);
        dumpField(i2,out,"children",children);
        out.println(indent+"}");
    }
    public String getType() { return "Billboard"; }
}
