/*
 * Copyright (C) 1998 Stephen Chenney and Jeffrey Ichnowski
 *
 * This program is free software.  You can redistribute it and/or
 * modify it under the terms of the GNU General Public License version
 * 2 as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
package berkeley.cs.dmc.vrml.parser.node;

import java.util.Hashtable;
import berkeley.cs.dmc.vrml.parser.*;
import berkeley.cs.dmc.vrml.field.*;
import berkeley.cs.dmc.vrml.Field;

public class Collision extends ParserNode {
    SFVec3f bboxCenter = new SFVec3f(0f, 0f, 0f);
    SFVec3f bboxSize = new SFVec3f(-1f, -1f, -1f);
    SFNode proxy = new SFNode();
    MFNode children = new MFNode();
    SFBool collide = new SFBool(true);
    public int getFieldType(String n) {
        if("bboxCenter".equals(n)) return 33;
        if("bboxSize".equals(n)) return 33;
        if("proxy".equals(n)) return 28;
        if("children".equals(n)) return 18;
        if("collide".equals(n)) return 23;
        return -1;
    }
    public void setField(String n, Field f) {
        if("bboxCenter".equals(n)) bboxCenter=(SFVec3f)f;
        else if("bboxSize".equals(n)) bboxSize=(SFVec3f)f;
        else if("proxy".equals(n)) proxy=(SFNode)f;
        else if("children".equals(n)) children=(MFNode)f;
        else if("collide".equals(n)) collide=(SFBool)f;
    }
    public ParserNode cloneThru(Hashtable i, Hashtable f) {
        Collision n = (Collision)super.cloneThru(i,f);
        n.bboxCenter = (SFVec3f)cloneField(bboxCenter);
        n.bboxSize = (SFVec3f)cloneField(bboxSize);
        n.proxy = cloneSFNode(proxy,i,f);
        n.children = cloneMFNode(children,i,f);
        n.collide = (SFBool)cloneField(collide);
        return n;
    }
    public void dump(String indent, java.io.PrintStream out) {
        String i2 = indent + "  ";
        out.println(indent+"Collision {");
        dumpField(i2,out,"bboxCenter",bboxCenter);
        dumpField(i2,out,"bboxSize",bboxSize);
        dumpField(i2,out,"proxy",proxy);
        dumpField(i2,out,"children",children);
        dumpField(i2,out,"collide",collide);
        out.println(indent+"}");
    }
    public String getType() { return "Collision"; }
}
