/*
 * Copyright (C) 1998 Stephen Chenney and Jeffrey Ichnowski
 *
 * This program is free software.  You can redistribute it and/or
 * modify it under the terms of the GNU General Public License version
 * 2 as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
package berkeley.cs.dmc.vrml.parser.node;

import java.util.Hashtable;
import berkeley.cs.dmc.vrml.parser.*;
import berkeley.cs.dmc.vrml.field.*;
import berkeley.cs.dmc.vrml.Field;

public class DirectionalLight extends ParserNode {
    SFFloat ambientIntensity = new SFFloat(0f);
    SFColor color = new SFColor(1f, 1f, 1f);
    SFVec3f direction = new SFVec3f(0f, 0f, -1f);
    SFFloat intensity = new SFFloat(1f);
    SFBool on = new SFBool(true);
    public int getFieldType(String n) {
        if("ambientIntensity".equals(n)) return 25;
        if("color".equals(n)) return 24;
        if("direction".equals(n)) return 33;
        if("intensity".equals(n)) return 25;
        if("on".equals(n)) return 23;
        return -1;
    }
    public void setField(String n, Field f) {
        if("ambientIntensity".equals(n)) ambientIntensity=(SFFloat)f;
        else if("color".equals(n)) color=(SFColor)f;
        else if("direction".equals(n)) direction=(SFVec3f)f;
        else if("intensity".equals(n)) intensity=(SFFloat)f;
        else if("on".equals(n)) on=(SFBool)f;
    }
    public ParserNode cloneThru(Hashtable i, Hashtable f) {
        DirectionalLight n = (DirectionalLight)super.cloneThru(i,f);
        n.ambientIntensity = (SFFloat)cloneField(ambientIntensity);
        n.color = (SFColor)cloneField(color);
        n.direction = (SFVec3f)cloneField(direction);
        n.intensity = (SFFloat)cloneField(intensity);
        n.on = (SFBool)cloneField(on);
        return n;
    }
    public void dump(String indent, java.io.PrintStream out) {
        String i2 = indent + "  ";
        out.println(indent+"DirectionalLight {");
        dumpField(i2,out,"ambientIntensity",ambientIntensity);
        dumpField(i2,out,"color",color);
        dumpField(i2,out,"direction",direction);
        dumpField(i2,out,"intensity",intensity);
        dumpField(i2,out,"on",on);
        out.println(indent+"}");
    }
    public String getType() { return "DirectionalLight"; }
}
