/*
 * Copyright (C) 1998 Stephen Chenney and Jeffrey Ichnowski
 *
 * This program is free software.  You can redistribute it and/or
 * modify it under the terms of the GNU General Public License version
 * 2 as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
package berkeley.cs.dmc.vrml.parser.node;

import java.util.Hashtable;
import berkeley.cs.dmc.vrml.parser.*;
import berkeley.cs.dmc.vrml.field.*;
import berkeley.cs.dmc.vrml.Field;

public class ElevationGrid extends ParserNode {
    MFFloat height = new MFFloat();
    SFBool ccw = new SFBool(true);
    SFBool colorPerVertex = new SFBool(true);
    SFFloat creaseAngle = new SFFloat(0f);
    SFBool normalPerVertex = new SFBool(true);
    SFBool solid = new SFBool(true);
    SFInt32 xDimension = new SFInt32(0);
    SFFloat xSpacing = new SFFloat(1.0f);
    SFInt32 zDimension = new SFInt32(0);
    SFFloat zSpacing = new SFFloat(1.0f);
    SFNode color = new SFNode();
    SFNode normal = new SFNode();
    SFNode texCoord = new SFNode();
    public int getFieldType(String n) {
        if("height".equals(n)) return 16;
        if("ccw".equals(n)) return 23;
        if("colorPerVertex".equals(n)) return 23;
        if("creaseAngle".equals(n)) return 25;
        if("normalPerVertex".equals(n)) return 23;
        if("solid".equals(n)) return 23;
        if("xDimension".equals(n)) return 27;
        if("xSpacing".equals(n)) return 25;
        if("zDimension".equals(n)) return 27;
        if("zSpacing".equals(n)) return 25;
        if("color".equals(n)) return 28;
        if("normal".equals(n)) return 28;
        if("texCoord".equals(n)) return 28;
        return -1;
    }
    public void setField(String n, Field f) {
        if("height".equals(n)) height=(MFFloat)f;
        else if("ccw".equals(n)) ccw=(SFBool)f;
        else if("colorPerVertex".equals(n)) colorPerVertex=(SFBool)f;
        else if("creaseAngle".equals(n)) creaseAngle=(SFFloat)f;
        else if("normalPerVertex".equals(n)) normalPerVertex=(SFBool)f;
        else if("solid".equals(n)) solid=(SFBool)f;
        else if("xDimension".equals(n)) xDimension=(SFInt32)f;
        else if("xSpacing".equals(n)) xSpacing=(SFFloat)f;
        else if("zDimension".equals(n)) zDimension=(SFInt32)f;
        else if("zSpacing".equals(n)) zSpacing=(SFFloat)f;
        else if("color".equals(n)) color=(SFNode)f;
        else if("normal".equals(n)) normal=(SFNode)f;
        else if("texCoord".equals(n)) texCoord=(SFNode)f;
    }
    public ParserNode cloneThru(Hashtable i, Hashtable f) {
        ElevationGrid n = (ElevationGrid)super.cloneThru(i,f);
        n.height = (MFFloat)cloneField(height);
        n.ccw = (SFBool)cloneField(ccw);
        n.colorPerVertex = (SFBool)cloneField(colorPerVertex);
        n.creaseAngle = (SFFloat)cloneField(creaseAngle);
        n.normalPerVertex = (SFBool)cloneField(normalPerVertex);
        n.solid = (SFBool)cloneField(solid);
        n.xDimension = (SFInt32)cloneField(xDimension);
        n.xSpacing = (SFFloat)cloneField(xSpacing);
        n.zDimension = (SFInt32)cloneField(zDimension);
        n.zSpacing = (SFFloat)cloneField(zSpacing);
        n.color = cloneSFNode(color,i,f);
        n.normal = cloneSFNode(normal,i,f);
        n.texCoord = cloneSFNode(texCoord,i,f);
        return n;
    }
    public void dump(String indent, java.io.PrintStream out) {
        String i2 = indent + "  ";
        out.println(indent+"ElevationGrid {");
        dumpField(i2,out,"height",height);
        dumpField(i2,out,"ccw",ccw);
        dumpField(i2,out,"colorPerVertex",colorPerVertex);
        dumpField(i2,out,"creaseAngle",creaseAngle);
        dumpField(i2,out,"normalPerVertex",normalPerVertex);
        dumpField(i2,out,"solid",solid);
        dumpField(i2,out,"xDimension",xDimension);
        dumpField(i2,out,"xSpacing",xSpacing);
        dumpField(i2,out,"zDimension",zDimension);
        dumpField(i2,out,"zSpacing",zSpacing);
        dumpField(i2,out,"color",color);
        dumpField(i2,out,"normal",normal);
        dumpField(i2,out,"texCoord",texCoord);
        out.println(indent+"}");
    }
    public String getType() { return "ElevationGrid"; }
}
