/*
 * Copyright (C) 1998 Stephen Chenney and Jeffrey Ichnowski
 *
 * This program is free software.  You can redistribute it and/or
 * modify it under the terms of the GNU General Public License version
 * 2 as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
package berkeley.cs.dmc.vrml.parser.node;

import java.util.Hashtable;
import berkeley.cs.dmc.vrml.parser.*;
import berkeley.cs.dmc.vrml.field.*;
import berkeley.cs.dmc.vrml.Field;

public class Extrusion extends ParserNode {
    SFBool beginCap = new SFBool(true);
    SFBool ccw = new SFBool(true);
    SFBool convex = new SFBool(true);
    SFFloat creaseAngle = new SFFloat(0f);
    MFVec2f crossSection = new MFVec2f(); {
        crossSection.addValue(1f, 1f);
        crossSection.addValue(1f, -1f);
        crossSection.addValue(-1f, -1f);
        crossSection.addValue(-1f, 1f);
        crossSection.addValue(1f, 1f);
    }
    SFBool endCap = new SFBool(true);
    MFRotation orientation = new MFRotation(); {
        orientation.addValue(0f, 0f, 1f, 0f);
    }
    MFVec2f scale = new MFVec2f(); {
        scale.addValue(1f, 1f);
    }
    SFBool solid = new SFBool(true);
    MFVec3f spine = new MFVec3f(); {
        spine.addValue(0f, 0f, 0f);
        spine.addValue(0f, 1f, 0f);
    }
    public int getFieldType(String n) {
        if("beginCap".equals(n)) return 23;
        if("ccw".equals(n)) return 23;
        if("convex".equals(n)) return 23;
        if("creaseAngle".equals(n)) return 25;
        if("crossSection".equals(n)) return 21;
        if("endCap".equals(n)) return 23;
        if("orientation".equals(n)) return 19;
        if("scale".equals(n)) return 21;
        if("solid".equals(n)) return 23;
        if("spine".equals(n)) return 22;
        return -1;
    }
    public void setField(String n, Field f) {
        if("beginCap".equals(n)) beginCap=(SFBool)f;
        else if("ccw".equals(n)) ccw=(SFBool)f;
        else if("convex".equals(n)) convex=(SFBool)f;
        else if("creaseAngle".equals(n)) creaseAngle=(SFFloat)f;
        else if("crossSection".equals(n)) crossSection=(MFVec2f)f;
        else if("endCap".equals(n)) endCap=(SFBool)f;
        else if("orientation".equals(n)) orientation=(MFRotation)f;
        else if("scale".equals(n)) scale=(MFVec2f)f;
        else if("solid".equals(n)) solid=(SFBool)f;
        else if("spine".equals(n)) spine=(MFVec3f)f;
    }
    public ParserNode cloneThru(Hashtable i, Hashtable f) {
        Extrusion n = (Extrusion)super.cloneThru(i,f);
        n.beginCap = (SFBool)cloneField(beginCap);
        n.ccw = (SFBool)cloneField(ccw);
        n.convex = (SFBool)cloneField(convex);
        n.creaseAngle = (SFFloat)cloneField(creaseAngle);
        n.crossSection = (MFVec2f)cloneField(crossSection);
        n.endCap = (SFBool)cloneField(endCap);
        n.orientation = (MFRotation)cloneField(orientation);
        n.scale = (MFVec2f)cloneField(scale);
        n.solid = (SFBool)cloneField(solid);
        n.spine = (MFVec3f)cloneField(spine);
        return n;
    }
    public void dump(String indent, java.io.PrintStream out) {
        String i2 = indent + "  ";
        out.println(indent+"Extrusion {");
        dumpField(i2,out,"beginCap",beginCap);
        dumpField(i2,out,"ccw",ccw);
        dumpField(i2,out,"convex",convex);
        dumpField(i2,out,"creaseAngle",creaseAngle);
        dumpField(i2,out,"crossSection",crossSection);
        dumpField(i2,out,"endCap",endCap);
        dumpField(i2,out,"orientation",orientation);
        dumpField(i2,out,"scale",scale);
        dumpField(i2,out,"solid",solid);
        dumpField(i2,out,"spine",spine);
        out.println(indent+"}");
    }
    public String getType() { return "Extrusion"; }
}
