/*
 * Copyright (C) 1998 Stephen Chenney and Jeffrey Ichnowski
 *
 * This program is free software.  You can redistribute it and/or
 * modify it under the terms of the GNU General Public License version
 * 2 as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
package berkeley.cs.dmc.vrml.parser.node;

import java.util.Hashtable;
import berkeley.cs.dmc.vrml.parser.*;
import berkeley.cs.dmc.vrml.field.*;
import berkeley.cs.dmc.vrml.Field;

public class FontStyle extends ParserNode {
    MFString family = new MFString(); {
        family.addValue("SERIF");
    }
    SFBool horizontal = new SFBool(true);
    MFString justify = new MFString(); {
        justify.addValue("BEGIN");
    }
    SFString language = new SFString();
    SFBool leftToRight = new SFBool(true);
    SFFloat size = new SFFloat(1.0f);
    SFFloat spacing = new SFFloat(1.0f);
    SFString style = new SFString("PLAIN");
    SFBool topToBottom = new SFBool(true);
    public int getFieldType(String n) {
        if("family".equals(n)) return 20;
        if("horizontal".equals(n)) return 23;
        if("justify".equals(n)) return 20;
        if("language".equals(n)) return 30;
        if("leftToRight".equals(n)) return 23;
        if("size".equals(n)) return 25;
        if("spacing".equals(n)) return 25;
        if("style".equals(n)) return 30;
        if("topToBottom".equals(n)) return 23;
        return -1;
    }
    public void setField(String n, Field f) {
        if("family".equals(n)) family=(MFString)f;
        else if("horizontal".equals(n)) horizontal=(SFBool)f;
        else if("justify".equals(n)) justify=(MFString)f;
        else if("language".equals(n)) language=(SFString)f;
        else if("leftToRight".equals(n)) leftToRight=(SFBool)f;
        else if("size".equals(n)) size=(SFFloat)f;
        else if("spacing".equals(n)) spacing=(SFFloat)f;
        else if("style".equals(n)) style=(SFString)f;
        else if("topToBottom".equals(n)) topToBottom=(SFBool)f;
    }
    public ParserNode cloneThru(Hashtable i, Hashtable f) {
        FontStyle n = (FontStyle)super.cloneThru(i,f);
        n.family = (MFString)cloneField(family);
        n.horizontal = (SFBool)cloneField(horizontal);
        n.justify = (MFString)cloneField(justify);
        n.language = (SFString)cloneField(language);
        n.leftToRight = (SFBool)cloneField(leftToRight);
        n.size = (SFFloat)cloneField(size);
        n.spacing = (SFFloat)cloneField(spacing);
        n.style = (SFString)cloneField(style);
        n.topToBottom = (SFBool)cloneField(topToBottom);
        return n;
    }
    public void dump(String indent, java.io.PrintStream out) {
        String i2 = indent + "  ";
        out.println(indent+"FontStyle {");
        dumpField(i2,out,"family",family);
        dumpField(i2,out,"horizontal",horizontal);
        dumpField(i2,out,"justify",justify);
        dumpField(i2,out,"language",language);
        dumpField(i2,out,"leftToRight",leftToRight);
        dumpField(i2,out,"size",size);
        dumpField(i2,out,"spacing",spacing);
        dumpField(i2,out,"style",style);
        dumpField(i2,out,"topToBottom",topToBottom);
        out.println(indent+"}");
    }
    public String getType() { return "FontStyle"; }
}
