/*
 * Copyright (C) 1998 Stephen Chenney and Jeffrey Ichnowski
 *
 * This program is free software.  You can redistribute it and/or
 * modify it under the terms of the GNU General Public License version
 * 2 as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
package berkeley.cs.dmc.vrml.parser.node;

import java.util.Hashtable;
import berkeley.cs.dmc.vrml.parser.*;
import berkeley.cs.dmc.vrml.field.*;
import berkeley.cs.dmc.vrml.Field;
import berkeley.cs.dmc.ji3d.*;

public class IndexedFaceSet extends ParserNode {
    SFBool ccw = new SFBool(true);
    MFInt32 colorIndex = new MFInt32();
    SFBool colorPerVertex = new SFBool(true);
    SFBool convex = new SFBool(true);
    MFInt32 coordIndex = new MFInt32();
    SFFloat creaseAngle = new SFFloat(0f);
    MFInt32 normalIndex = new MFInt32();
    SFBool normalPerVertex = new SFBool(true);
    SFBool solid = new SFBool(true);
    MFInt32 texCoordIndex = new MFInt32();
    SFNode color = new SFNode();
    SFNode coord = new SFNode();
    SFNode normal = new SFNode();
    SFNode texCoord = new SFNode();
    public int getFieldType(String n) {
        if("ccw".equals(n)) return 23;
        if("colorIndex".equals(n)) return 17;
        if("colorPerVertex".equals(n)) return 23;
        if("convex".equals(n)) return 23;
        if("coordIndex".equals(n)) return 17;
        if("creaseAngle".equals(n)) return 25;
        if("normalIndex".equals(n)) return 17;
        if("normalPerVertex".equals(n)) return 23;
        if("solid".equals(n)) return 23;
        if("texCoordIndex".equals(n)) return 17;
        if("color".equals(n)) return 28;
        if("coord".equals(n)) return 28;
        if("normal".equals(n)) return 28;
        if("texCoord".equals(n)) return 28;
        return -1;
    }
    public void setField(String n, Field f) {
        if("ccw".equals(n)) ccw=(SFBool)f;
        else if("colorIndex".equals(n)) colorIndex=(MFInt32)f;
        else if("colorPerVertex".equals(n)) colorPerVertex=(SFBool)f;
        else if("convex".equals(n)) convex=(SFBool)f;
        else if("coordIndex".equals(n)) coordIndex=(MFInt32)f;
        else if("creaseAngle".equals(n)) creaseAngle=(SFFloat)f;
        else if("normalIndex".equals(n)) normalIndex=(MFInt32)f;
        else if("normalPerVertex".equals(n)) normalPerVertex=(SFBool)f;
        else if("solid".equals(n)) solid=(SFBool)f;
        else if("texCoordIndex".equals(n)) texCoordIndex=(MFInt32)f;
        else if("color".equals(n)) color=(SFNode)f;
        else if("coord".equals(n)) coord=(SFNode)f;
        else if("normal".equals(n)) normal=(SFNode)f;
        else if("texCoord".equals(n)) texCoord=(SFNode)f;
    }
    public ParserNode cloneThru(Hashtable i, Hashtable f) {
        IndexedFaceSet n = (IndexedFaceSet)super.cloneThru(i,f);
        n.ccw = (SFBool)cloneField(ccw);
        n.colorIndex = (MFInt32)cloneField(colorIndex);
        n.colorPerVertex = (SFBool)cloneField(colorPerVertex);
        n.convex = (SFBool)cloneField(convex);
        n.coordIndex = (MFInt32)cloneField(coordIndex);
        n.creaseAngle = (SFFloat)cloneField(creaseAngle);
        n.normalIndex = (MFInt32)cloneField(normalIndex);
        n.normalPerVertex = (SFBool)cloneField(normalPerVertex);
        n.solid = (SFBool)cloneField(solid);
        n.texCoordIndex = (MFInt32)cloneField(texCoordIndex);
        n.color = cloneSFNode(color,i,f);
        n.coord = cloneSFNode(coord,i,f);
        n.normal = cloneSFNode(normal,i,f);
        n.texCoord = cloneSFNode(texCoord,i,f);
        return n;
    }
    public void dump(String indent, java.io.PrintStream out) {
        String i2 = indent + "  ";
        out.println(indent+"IndexedFaceSet {");
        dumpField(i2,out,"ccw",ccw);
        dumpField(i2,out,"colorIndex",colorIndex);
        dumpField(i2,out,"colorPerVertex",colorPerVertex);
        dumpField(i2,out,"convex",convex);
        dumpField(i2,out,"coordIndex",coordIndex);
        dumpField(i2,out,"creaseAngle",creaseAngle);
        dumpField(i2,out,"normalIndex",normalIndex);
        dumpField(i2,out,"normalPerVertex",normalPerVertex);
        dumpField(i2,out,"solid",solid);
        dumpField(i2,out,"texCoordIndex",texCoordIndex);
        dumpField(i2,out,"color",color);
        dumpField(i2,out,"coord",coord);
        dumpField(i2,out,"normal",normal);
        dumpField(i2,out,"texCoord",texCoord);
        out.println(indent+"}");
    }
    public String getType() { return "IndexedFaceSet"; }

    // Additions follow
    Obj obj;
  
    public void build() {
        build( color );
	build( coord );
	build( normal );
	build( texCoord );

	boolean ccw   = this.ccw.getValue();
	boolean solid = this.solid.getValue();

	Coordinate cn = (Coordinate)coord.getValue();
	Vec3 points[] = (cn == null) ? new Vec3[0] : cn.points;
	int faceIdx[][];
	Mat4 mat = new Mat4(); mat.identity();

	int numci = coordIndex.getSize();
	int ci[] = new int[numci];
	int i, j, facesize;
	int faces = 0;

	coordIndex.getValue(ci);
	for( i=0 ; i<numci ; i=j+1 ) {
	    faces++;
	    for( j=i+3 ; j<numci && ci[j] != -1 ; j++ );
	}

	if( !solid ) faces *= 2;

	faceIdx = new int[faces][];
	faces = 0;
	for( i=0 ; i<numci ; ) {
	    for( j=i+3 ; j<numci && ci[j] != -1 ; j++ );

	    facesize = j-i;
	    if( !solid || ccw ) {
	        faceIdx[faces] = new int[facesize];
		for( j=0 ; j<facesize ; j++ )
		    faceIdx[faces][j] = ci[i+facesize-j-1];
		faces++;
	    }
	    if( !solid || !ccw ) {
	        faceIdx[faces] = new int[facesize];
		for( j=0 ; j<facesize ; j++ )
		    faceIdx[faces][j] = ci[i+j];
		faces++;
	    }

	    i+=facesize+1;
	}

	obj = new Obj( points, faceIdx, mat, null );
    }
    public void render( berkeley.cs.dmc.vrml.RenderState state ) {
        obj.modelMatrix = state.model;
	obj.material = state.material;
	obj.appendProjectedFacesToFaceList( state.view, state.faceList );
    }
}
