/*
 * Copyright (C) 1998 Stephen Chenney and Jeffrey Ichnowski
 *
 * This program is free software.  You can redistribute it and/or
 * modify it under the terms of the GNU General Public License version
 * 2 as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
package berkeley.cs.dmc.vrml.parser.node;

import java.util.Hashtable;
import berkeley.cs.dmc.vrml.parser.*;
import berkeley.cs.dmc.vrml.field.*;
import berkeley.cs.dmc.vrml.Field;
import berkeley.cs.dmc.ji3d.*;

public class IndexedLineSet extends ParserNode {
    MFInt32 colorIndex = new MFInt32();
    SFBool colorPerVertex = new SFBool(true);
    MFInt32 coordIndex = new MFInt32();
    SFNode color = new SFNode();
    SFNode coord = new SFNode();
    public int getFieldType(String n) {
        if("colorIndex".equals(n)) return 17;
        if("colorPerVertex".equals(n)) return 23;
        if("coordIndex".equals(n)) return 17;
        if("color".equals(n)) return 28;
        if("coord".equals(n)) return 28;
        return -1;
    }
    public void setField(String n, Field f) {
        if("colorIndex".equals(n)) colorIndex=(MFInt32)f;
        else if("colorPerVertex".equals(n)) colorPerVertex=(SFBool)f;
        else if("coordIndex".equals(n)) coordIndex=(MFInt32)f;
        else if("color".equals(n)) color=(SFNode)f;
        else if("coord".equals(n)) coord=(SFNode)f;
    }
    public ParserNode cloneThru(Hashtable i, Hashtable f) {
        IndexedLineSet n = (IndexedLineSet)super.cloneThru(i,f);
        n.colorIndex = (MFInt32)cloneField(colorIndex);
        n.colorPerVertex = (SFBool)cloneField(colorPerVertex);
        n.coordIndex = (MFInt32)cloneField(coordIndex);
        n.color = cloneSFNode(color,i,f);
        n.coord = cloneSFNode(coord,i,f);
        return n;
    }
    public void dump(String indent, java.io.PrintStream out) {
        String i2 = indent + "  ";
        out.println(indent+"IndexedLineSet {");
        dumpField(i2,out,"colorIndex",colorIndex);
        dumpField(i2,out,"colorPerVertex",colorPerVertex);
        dumpField(i2,out,"coordIndex",coordIndex);
        dumpField(i2,out,"color",color);
        dumpField(i2,out,"coord",coord);
        out.println(indent+"}");
    }
    public String getType() { return "IndexedLineSet"; }

    // Additions follow...
    Obj obj;

    public void build() {
        build( color );
	build( coord );

	Coordinate cn = (Coordinate)coord.getValue();
	Vec3 points[] = (cn == null) ? new Vec3[0] : cn.points;
	int faceIdx[][];
	Mat4 mat = new Mat4(); mat.identity();
	int numci = coordIndex.getSize();
	int ci[] = new int[numci];
	int i, linesize;
	int numlines = 0;

	coordIndex.getValue(ci);
	for( i=1 ; i<numci ; i++ ) {
	    if( ci[i] != -1 )
	        numlines++;
	}

	faceIdx = new int[numlines][3];
	int lineno = 0;
	for( i=1 ; i<numci ; i++ ) {
	    if( ci[i] != -1 ) {
	        faceIdx[lineno][0] = ci[i];
	        faceIdx[lineno][1] = ci[i];
		faceIdx[lineno][2] = ci[i-1];
		lineno++;
	    }
	}

	obj = new Obj( points, faceIdx, mat, null );
    }

    public void render( berkeley.cs.dmc.vrml.RenderState state ) {
        obj.modelMatrix = state.model;
	obj.material = state.material;
	obj.appendProjectedFacesToFaceList( state.view, state.faceList );
    }
}
