/*
 * Copyright (C) 1998 Stephen Chenney and Jeffrey Ichnowski
 *
 * This program is free software.  You can redistribute it and/or
 * modify it under the terms of the GNU General Public License version
 * 2 as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
package berkeley.cs.dmc.vrml.parser.node;

import java.util.Hashtable;
import berkeley.cs.dmc.vrml.parser.*;
import berkeley.cs.dmc.vrml.field.*;
import berkeley.cs.dmc.vrml.Field;
import berkeley.cs.dmc.ji3d.*;

public class Material extends ParserNode {
    SFFloat ambientIntensity = new SFFloat(0.2f);
    SFColor diffuseColor = new SFColor(0.8f, 0.8f, 0.8f);
    SFColor emissiveColor = new SFColor(0f, 0f, 0f);
    SFFloat shininess = new SFFloat(0.2f);
    SFColor specularColor = new SFColor(0f, 0f, 0f);
    SFFloat transparency = new SFFloat(0f);
    public int getFieldType(String n) {
        if("ambientIntensity".equals(n)) return 25;
        if("diffuseColor".equals(n)) return 24;
        if("emissiveColor".equals(n)) return 24;
        if("shininess".equals(n)) return 25;
        if("specularColor".equals(n)) return 24;
        if("transparency".equals(n)) return 25;
        return -1;
    }
    public void setField(String n, Field f) {
        if("ambientIntensity".equals(n)) ambientIntensity=(SFFloat)f;
        else if("diffuseColor".equals(n)) diffuseColor=(SFColor)f;
        else if("emissiveColor".equals(n)) emissiveColor=(SFColor)f;
        else if("shininess".equals(n)) shininess=(SFFloat)f;
        else if("specularColor".equals(n)) specularColor=(SFColor)f;
        else if("transparency".equals(n)) transparency=(SFFloat)f;
    }
    public ParserNode cloneThru(Hashtable i, Hashtable f) {
        Material n = (Material)super.cloneThru(i,f);
        n.ambientIntensity = (SFFloat)cloneField(ambientIntensity);
        n.diffuseColor = (SFColor)cloneField(diffuseColor);
        n.emissiveColor = (SFColor)cloneField(emissiveColor);
        n.shininess = (SFFloat)cloneField(shininess);
        n.specularColor = (SFColor)cloneField(specularColor);
        n.transparency = (SFFloat)cloneField(transparency);
        return n;
    }
    public void dump(String indent, java.io.PrintStream out) {
        String i2 = indent + "  ";
        out.println(indent+"Material {");
        dumpField(i2,out,"ambientIntensity",ambientIntensity);
        dumpField(i2,out,"diffuseColor",diffuseColor);
        dumpField(i2,out,"emissiveColor",emissiveColor);
        dumpField(i2,out,"shininess",shininess);
        dumpField(i2,out,"specularColor",specularColor);
        dumpField(i2,out,"transparency",transparency);
        out.println(indent+"}");
    }
    public String getType() { return "Material"; }
  
    // Additions follow
    berkeley.cs.dmc.ji3d.Material material;
  
    public void build() {
        float ai = ambientIntensity.getValue();
	float dr = diffuseColor.getRed();
	float dg = diffuseColor.getGreen();
	float db = diffuseColor.getBlue();
	material = new berkeley.cs.dmc.ji3d.Material( new Vec3( dr*ai, dg*ai, db*ai ),
					   new Vec3( dr, dg, db ),
					   new Vec3( specularColor.getRed(),
						     specularColor.getGreen(),
						     specularColor.getBlue() ),
					   shininess.getValue() );
    }

    public void render( berkeley.cs.dmc.vrml.RenderState state ) {
        state.material = material;
    }
}
