/*
 * Copyright (C) 1998 Stephen Chenney and Jeffrey Ichnowski
 *
 * This program is free software.  You can redistribute it and/or
 * modify it under the terms of the GNU General Public License version
 * 2 as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
package berkeley.cs.dmc.vrml.parser.node;

import java.util.Hashtable;
import berkeley.cs.dmc.vrml.parser.*;
import berkeley.cs.dmc.vrml.field.*;
import berkeley.cs.dmc.vrml.Field;

public class NavigationInfo extends ParserNode {
    MFFloat avatarSize = new MFFloat(); {
        avatarSize.addValue(0.25f);
        avatarSize.addValue(1.6f);
        avatarSize.addValue(0.75f);
    }
    SFBool headlight = new SFBool(true);
    SFFloat speed = new SFFloat(1.0f);
    MFString type = new MFString(); {
        type.addValue("WALK");
        type.addValue("ANY");
    }
    SFFloat visibilityLimit = new SFFloat(0.0f);
    public int getFieldType(String n) {
        if("avatarSize".equals(n)) return 16;
        if("headlight".equals(n)) return 23;
        if("speed".equals(n)) return 25;
        if("type".equals(n)) return 20;
        if("visibilityLimit".equals(n)) return 25;
        return -1;
    }
    public void setField(String n, Field f) {
        if("avatarSize".equals(n)) avatarSize=(MFFloat)f;
        else if("headlight".equals(n)) headlight=(SFBool)f;
        else if("speed".equals(n)) speed=(SFFloat)f;
        else if("type".equals(n)) type=(MFString)f;
        else if("visibilityLimit".equals(n)) visibilityLimit=(SFFloat)f;
    }
    public ParserNode cloneThru(Hashtable i, Hashtable f) {
        NavigationInfo n = (NavigationInfo)super.cloneThru(i,f);
        n.avatarSize = (MFFloat)cloneField(avatarSize);
        n.headlight = (SFBool)cloneField(headlight);
        n.speed = (SFFloat)cloneField(speed);
        n.type = (MFString)cloneField(type);
        n.visibilityLimit = (SFFloat)cloneField(visibilityLimit);
        return n;
    }
    public void dump(String indent, java.io.PrintStream out) {
        String i2 = indent + "  ";
        out.println(indent+"NavigationInfo {");
        dumpField(i2,out,"avatarSize",avatarSize);
        dumpField(i2,out,"headlight",headlight);
        dumpField(i2,out,"speed",speed);
        dumpField(i2,out,"type",type);
        dumpField(i2,out,"visibilityLimit",visibilityLimit);
        out.println(indent+"}");
    }
    public String getType() { return "NavigationInfo"; }
}
