/*
 * Copyright (C) 1998 Stephen Chenney and Jeffrey Ichnowski
 *
 * This program is free software.  You can redistribute it and/or
 * modify it under the terms of the GNU General Public License version
 * 2 as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
package berkeley.cs.dmc.vrml.parser.node;

import java.util.Hashtable;
import berkeley.cs.dmc.vrml.parser.*;
import berkeley.cs.dmc.vrml.field.*;
import berkeley.cs.dmc.vrml.Field;
import berkeley.cs.dmc.ji3d.*;
import berkeley.cs.dmc.vrml.RenderState;

public class Shape extends ParserNode {
    SFNode appearance = new SFNode();
    SFNode geometry = new SFNode();
    public int getFieldType(String n) {
        if("appearance".equals(n)) return 28;
        if("geometry".equals(n)) return 28;
        return -1;
    }
    public void setField(String n, Field f) {
        if("appearance".equals(n)) appearance=(SFNode)f;
        else if("geometry".equals(n)) geometry=(SFNode)f;
    }
    public ParserNode cloneThru(Hashtable i, Hashtable f) {
        Shape n = (Shape)super.cloneThru(i,f);
        n.appearance = cloneSFNode(appearance,i,f);
        n.geometry = cloneSFNode(geometry,i,f);
        return n;
    }
    public void dump(String indent, java.io.PrintStream out) {
        String i2 = indent + "  ";
        out.println(indent+"Shape {");
        dumpField(i2,out,"appearance",appearance);
        dumpField(i2,out,"geometry",geometry);
        out.println(indent+"}");
    }
    public String getType() { return "Shape"; }
  
    // Additions follow
    RenderState renderState;
    public void build() {
        renderState = new RenderState();
        build( appearance );
	build( geometry );
    }
    public void render( RenderState state ) {
        renderState.copyFrom( state );     // push
	render( appearance, renderState ); // sets the material
	render( geometry, renderState );   // render using that material
    }
}
