/*
 * Copyright (C) 1998 Stephen Chenney and Jeffrey Ichnowski
 *
 * This program is free software.  You can redistribute it and/or
 * modify it under the terms of the GNU General Public License version
 * 2 as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
package berkeley.cs.dmc.vrml.parser.node;

import java.util.Hashtable;
import berkeley.cs.dmc.vrml.parser.*;
import berkeley.cs.dmc.vrml.field.*;
import berkeley.cs.dmc.vrml.Field;

public class Sound extends ParserNode {
    SFBool spatialize = new SFBool(true);
    SFVec3f direction = new SFVec3f(0f, 0f, 1f);
    SFFloat intensity = new SFFloat(1f);
    SFVec3f location = new SFVec3f(0f, 0f, 0f);
    SFFloat maxBack = new SFFloat(10f);
    SFFloat maxFront = new SFFloat(10f);
    SFFloat minBack = new SFFloat(1f);
    SFFloat minFront = new SFFloat(1f);
    SFFloat priority = new SFFloat(0f);
    SFNode source = new SFNode();
    public int getFieldType(String n) {
        if("spatialize".equals(n)) return 23;
        if("direction".equals(n)) return 33;
        if("intensity".equals(n)) return 25;
        if("location".equals(n)) return 33;
        if("maxBack".equals(n)) return 25;
        if("maxFront".equals(n)) return 25;
        if("minBack".equals(n)) return 25;
        if("minFront".equals(n)) return 25;
        if("priority".equals(n)) return 25;
        if("source".equals(n)) return 28;
        return -1;
    }
    public void setField(String n, Field f) {
        if("spatialize".equals(n)) spatialize=(SFBool)f;
        else if("direction".equals(n)) direction=(SFVec3f)f;
        else if("intensity".equals(n)) intensity=(SFFloat)f;
        else if("location".equals(n)) location=(SFVec3f)f;
        else if("maxBack".equals(n)) maxBack=(SFFloat)f;
        else if("maxFront".equals(n)) maxFront=(SFFloat)f;
        else if("minBack".equals(n)) minBack=(SFFloat)f;
        else if("minFront".equals(n)) minFront=(SFFloat)f;
        else if("priority".equals(n)) priority=(SFFloat)f;
        else if("source".equals(n)) source=(SFNode)f;
    }
    public ParserNode cloneThru(Hashtable i, Hashtable f) {
        Sound n = (Sound)super.cloneThru(i,f);
        n.spatialize = (SFBool)cloneField(spatialize);
        n.direction = (SFVec3f)cloneField(direction);
        n.intensity = (SFFloat)cloneField(intensity);
        n.location = (SFVec3f)cloneField(location);
        n.maxBack = (SFFloat)cloneField(maxBack);
        n.maxFront = (SFFloat)cloneField(maxFront);
        n.minBack = (SFFloat)cloneField(minBack);
        n.minFront = (SFFloat)cloneField(minFront);
        n.priority = (SFFloat)cloneField(priority);
        n.source = cloneSFNode(source,i,f);
        return n;
    }
    public void dump(String indent, java.io.PrintStream out) {
        String i2 = indent + "  ";
        out.println(indent+"Sound {");
        dumpField(i2,out,"spatialize",spatialize);
        dumpField(i2,out,"direction",direction);
        dumpField(i2,out,"intensity",intensity);
        dumpField(i2,out,"location",location);
        dumpField(i2,out,"maxBack",maxBack);
        dumpField(i2,out,"maxFront",maxFront);
        dumpField(i2,out,"minBack",minBack);
        dumpField(i2,out,"minFront",minFront);
        dumpField(i2,out,"priority",priority);
        dumpField(i2,out,"source",source);
        out.println(indent+"}");
    }
    public String getType() { return "Sound"; }
}
