/*
 * Copyright (C) 1998 Stephen Chenney and Jeffrey Ichnowski
 *
 * This program is free software.  You can redistribute it and/or
 * modify it under the terms of the GNU General Public License version
 * 2 as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
package berkeley.cs.dmc.vrml.parser.node;

import java.util.Hashtable;
import berkeley.cs.dmc.vrml.parser.*;
import berkeley.cs.dmc.vrml.field.*;
import berkeley.cs.dmc.vrml.Field;

public class SpotLight extends ParserNode {
    SFFloat ambientIntensity = new SFFloat(0f);
    SFVec3f attenuation = new SFVec3f(1f, 0f, 0f);
    SFFloat beamWidth = new SFFloat(1.570796f);
    SFColor color = new SFColor(1f, 1f, 1f);
    SFFloat cutOffAngle = new SFFloat(0.785398f);
    SFVec3f direction = new SFVec3f(0f, 0f, -1f);
    SFFloat intensity = new SFFloat(1f);
    SFVec3f location = new SFVec3f(0f, 0f, 0f);
    SFBool on = new SFBool(true);
    SFFloat radius = new SFFloat(100f);
    public int getFieldType(String n) {
        if("ambientIntensity".equals(n)) return 25;
        if("attenuation".equals(n)) return 33;
        if("beamWidth".equals(n)) return 25;
        if("color".equals(n)) return 24;
        if("cutOffAngle".equals(n)) return 25;
        if("direction".equals(n)) return 33;
        if("intensity".equals(n)) return 25;
        if("location".equals(n)) return 33;
        if("on".equals(n)) return 23;
        if("radius".equals(n)) return 25;
        return -1;
    }
    public void setField(String n, Field f) {
        if("ambientIntensity".equals(n)) ambientIntensity=(SFFloat)f;
        else if("attenuation".equals(n)) attenuation=(SFVec3f)f;
        else if("beamWidth".equals(n)) beamWidth=(SFFloat)f;
        else if("color".equals(n)) color=(SFColor)f;
        else if("cutOffAngle".equals(n)) cutOffAngle=(SFFloat)f;
        else if("direction".equals(n)) direction=(SFVec3f)f;
        else if("intensity".equals(n)) intensity=(SFFloat)f;
        else if("location".equals(n)) location=(SFVec3f)f;
        else if("on".equals(n)) on=(SFBool)f;
        else if("radius".equals(n)) radius=(SFFloat)f;
    }
    public ParserNode cloneThru(Hashtable i, Hashtable f) {
        SpotLight n = (SpotLight)super.cloneThru(i,f);
        n.ambientIntensity = (SFFloat)cloneField(ambientIntensity);
        n.attenuation = (SFVec3f)cloneField(attenuation);
        n.beamWidth = (SFFloat)cloneField(beamWidth);
        n.color = (SFColor)cloneField(color);
        n.cutOffAngle = (SFFloat)cloneField(cutOffAngle);
        n.direction = (SFVec3f)cloneField(direction);
        n.intensity = (SFFloat)cloneField(intensity);
        n.location = (SFVec3f)cloneField(location);
        n.on = (SFBool)cloneField(on);
        n.radius = (SFFloat)cloneField(radius);
        return n;
    }
    public void dump(String indent, java.io.PrintStream out) {
        String i2 = indent + "  ";
        out.println(indent+"SpotLight {");
        dumpField(i2,out,"ambientIntensity",ambientIntensity);
        dumpField(i2,out,"attenuation",attenuation);
        dumpField(i2,out,"beamWidth",beamWidth);
        dumpField(i2,out,"color",color);
        dumpField(i2,out,"cutOffAngle",cutOffAngle);
        dumpField(i2,out,"direction",direction);
        dumpField(i2,out,"intensity",intensity);
        dumpField(i2,out,"location",location);
        dumpField(i2,out,"on",on);
        dumpField(i2,out,"radius",radius);
        out.println(indent+"}");
    }
    public String getType() { return "SpotLight"; }
}
