/*
 * Copyright (C) 1998 Stephen Chenney and Jeffrey Ichnowski
 *
 * This program is free software.  You can redistribute it and/or
 * modify it under the terms of the GNU General Public License version
 * 2 as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
package berkeley.cs.dmc.vrml.parser.node;

import java.util.Hashtable;
import berkeley.cs.dmc.vrml.parser.*;
import berkeley.cs.dmc.vrml.field.*;
import berkeley.cs.dmc.vrml.Field;
import berkeley.cs.dmc.ji3d.Mat4;
import berkeley.cs.dmc.ji3d.Vec3;
import berkeley.cs.dmc.vrml.RenderState;

public class Transform extends ParserNode {
    SFVec3f bboxCenter = new SFVec3f(0f, 0f, 0f);
    SFVec3f bboxSize = new SFVec3f(-1f, -1f, -1f);
    SFVec3f center = new SFVec3f(0f, 0f, 0f);
    MFNode children = new MFNode();
    SFRotation rotation = new SFRotation(0f, 0f, 1f, 0f);
    SFVec3f scale = new SFVec3f(1f, 1f, 1f);
    SFRotation scaleOrientation = new SFRotation(0f, 0f, 1f, 0f);
    SFVec3f translation = new SFVec3f(0f, 0f, 0f);
    public int getFieldType(String n) {
        if("bboxCenter".equals(n)) return 33;
        if("bboxSize".equals(n)) return 33;
        if("center".equals(n)) return 33;
        if("children".equals(n)) return 18;
        if("rotation".equals(n)) return 29;
        if("scale".equals(n)) return 33;
        if("scaleOrientation".equals(n)) return 29;
        if("translation".equals(n)) return 33;
        return -1;
    }
    public void setField(String n, Field f) {
        if("bboxCenter".equals(n)) bboxCenter=(SFVec3f)f;
        else if("bboxSize".equals(n)) bboxSize=(SFVec3f)f;
        else if("center".equals(n)) center=(SFVec3f)f;
        else if("children".equals(n)) children=(MFNode)f;
        else if("rotation".equals(n)) rotation=(SFRotation)f;
        else if("scale".equals(n)) scale=(SFVec3f)f;
        else if("scaleOrientation".equals(n)) scaleOrientation=(SFRotation)f;
        else if("translation".equals(n)) translation=(SFVec3f)f;
    }
    public ParserNode cloneThru(Hashtable i, Hashtable f) {
        Transform n = (Transform)super.cloneThru(i,f);
        n.bboxCenter = (SFVec3f)cloneField(bboxCenter);
        n.bboxSize = (SFVec3f)cloneField(bboxSize);
        n.center = (SFVec3f)cloneField(center);
        n.children = cloneMFNode(children,i,f);
        n.rotation = (SFRotation)cloneField(rotation);
        n.scale = (SFVec3f)cloneField(scale);
        n.scaleOrientation = (SFRotation)cloneField(scaleOrientation);
        n.translation = (SFVec3f)cloneField(translation);
        return n;
    }
    public void dump(String indent, java.io.PrintStream out) {
        String i2 = indent + "  ";
        out.println(indent+"Transform {");
        dumpField(i2,out,"bboxCenter",bboxCenter);
        dumpField(i2,out,"bboxSize",bboxSize);
        dumpField(i2,out,"center",center);
        dumpField(i2,out,"children",children);
        dumpField(i2,out,"rotation",rotation);
        dumpField(i2,out,"scale",scale);
        dumpField(i2,out,"scaleOrientation",scaleOrientation);
        dumpField(i2,out,"translation",translation);
        out.println(indent+"}");
    }
    public String getType() { return "Transform"; }

    // Additions follow
    Mat4 xform;
    Mat4 renderxform;
    
    public void build() {
        float r[] = new float[4];
	rotation.getValue(r);
	Vec3 raxis = new Vec3( r[0], r[1], r[2] );
	float rangle = r[3];
	renderxform = new Mat4();

	scaleOrientation.getValue(r);
	Vec3 soaxis = new Vec3( r[0], r[1], r[2] );
	float soangle = r[3];

	xform = new Mat4();
	xform.translation( translation.getX(),
			   translation.getY(),
			   translation.getZ() );
	xform.appendTranslate( center.getX(),
			       center.getY(),
			       center.getZ() );
	xform.appendRotate( raxis, rangle );
	xform.appendRotate( soaxis, soangle );
	xform.appendScale( scale.getX(),
			   scale.getY(),
			   scale.getZ() );
	xform.appendRotate( soaxis, -soangle );
	xform.appendTranslate( -center.getX(),
			       -center.getY(),
			       -center.getZ() );
	
        build( children );
    }

    public void render( RenderState state ) {
        Mat4 oldxform = state.model;
        renderxform.mul( xform, state.model );
	state.model = renderxform;
	render( children, state );
	state.model = oldxform;
    }

}
