/*
 * NodeParser.java
 * Copyright (C) 1998 Stephen Chenney and Jeffrey Ichnowski
 *
 * This program is free software.  You can redistribute it and/or
 * modify it under the terms of the GNU General Public License version
 * 2 as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
package berkeley.cs.dmc.vrml.parser.spec;

import java.io.*;
import java.util.*;

public class NodeParser implements berkeley.cs.dmc.vrml.parser.VRMLParserConstants {
    protected LineNumberReader in;

    static Hashtable typeMap = new Hashtable();
    static Hashtable nameMap = new Hashtable();
    static void addType( String name, int val ) {
	Integer v = new Integer(val);
	typeMap.put( name, v );
	nameMap.put( v, name );
    }
    static {
	addType( "MFColor", MFCOLOR );
	addType( "MFFloat", MFFLOAT );
	addType( "MFInt32", MFINT32 );
	addType( "MFNode", MFNODE );
	addType( "MFRotation", MFROTATION );
	addType( "MFString", MFSTRING );
	addType( "MFVec2f", MFVEC2F );
	addType( "MFVec3f", MFVEC3F );
	addType( "SFBool", SFBOOL );
	addType( "SFColor", SFCOLOR );
	addType( "SFFloat", SFFLOAT );
	addType( "SFImage", SFIMAGE );
	addType( "SFInt32", SFINT32 );
	addType( "SFNode", SFNODE );
	addType( "SFRotation", SFROTATION );
	addType( "SFString", SFSTRING );
	addType( "SFTime", SFTIME );
	addType( "SFVec2f", SFVEC2F );
	addType( "SFVec3f", SFVEC3F );
    }
    static String getTypeName( int i ) { return (String)nameMap.get(new Integer(i)); }

    public static class Event {
	public int type;
	public String name;
    }

    public static class Field {
	public int type;
	public String name;
	public String init[];
    }

    public static class Node {
	public String name;
	public Vector eventIns = new Vector();
	public Vector eventOuts = new Vector();
	public Vector fields = new Vector();
	public Vector exposedFields = new Vector();

	public void writeNode( PrintWriter out ) {
	    int nf = fields.size();
	    int nt = nf + exposedFields.size();

	    out.println("package berkeley.cs.dmc.vrml.parser.node;");
	    out.println();
	    out.println("import java.util.Hashtable;");
	    out.println("import berkeley.cs.dmc.vrml.parser.*;");
	    out.println("import berkeley.cs.dmc.vrml.field.*;");
	    out.println("import berkeley.cs.dmc.vrml.Field;");
	    out.println();
	    out.println("public class "+name+" extends ParserNode {");
	    for( int i=0 ; i<nt ; i++ ) {
		Field f = (Field)(i<nf?fields.elementAt(i):exposedFields.elementAt(i-nf));
		String t = getTypeName(f.type);
		String n = f.name;
		String init[] = f.init;
		out.print("    "+t+" "+n+" = new "+t+"(");
		if( init.length == 0 )
		    out.println(");");
		else {
		    if( t.charAt(0) != 'M' )
			out.println( init[0]+");" );
		    else {
			out.println("); {");
			for( int j=0 ; j<init.length ; j++ )
			    out.println("        "+n+".addValue("+init[j]+");");
			out.println("    }");
		    }
		}
	    }
		
	    out.println("    public int getFieldType(String n) {");
	    for( int i=0 ; i<nt ; i++ ) {
		Field f = (Field)(i<nf?fields.elementAt(i):exposedFields.elementAt(i-nf));
		out.println("        if(\""+f.name+"\".equals(n)) return "+f.type+";");
	    }
	    out.println("        return -1;");
	    out.println("    }");
	    out.println("    public void setField(String n, Field f) {");
	    for( int i=0 ; i<nt ; i++ ) {
		Field f = (Field)(i<nf?fields.elementAt(i):exposedFields.elementAt(i-nf));
		String t = getTypeName(f.type);
		out.print("        ");
		if(i!=0) out.print("else ");
		out.println("if(\""+f.name+"\".equals(n)) "+f.name+"=("+t+")f;");
	    }
	    out.println("    }");
	    out.println("    public ParserNode cloneThru(Hashtable i, Hashtable f) {");
	    out.println("        "+name+" n = ("+name+")super.cloneThru(i,f);");
	    for( int i=0 ; i<nt ; i++ ) {
		Field f = (Field)(i<nf?fields.elementAt(i):exposedFields.elementAt(i-nf));
		switch( f.type ) {
		case SFNODE:
		    out.println("        n."+f.name+" = cloneSFNode("+f.name+",i,f);");
		    break;
		case MFNODE:
		    out.println("        n."+f.name+" = cloneMFNode("+f.name+",i,f);");
		    break;
		default:
		    String t = getTypeName(f.type);
		    out.println("        n."+f.name+" = ("+t+")cloneField("+f.name+");");
		    break;
		}
	    }
	    out.println("        return n;");
	    out.println("    }");
	    out.println("    public void dump(String indent, java.io.PrintStream out) {");
	    out.println("        String i2 = indent + \"  \";");
	    out.println("        out.println(indent+\""+name+" {\");");
	    for( int i=0 ; i<nt ; i++ ) {
		Field f = (Field)(i<nf?fields.elementAt(i):exposedFields.elementAt(i-nf));
		out.println("        dumpField(i2,out,\""+f.name+"\","+f.name+");");
	    }
	    out.println("        out.println(indent+\"}\");");
	    out.println("    }");
	    out.println("    public String getType() { return \""+name+"\"; }");
	    out.println("}");
	}

	public void dump( PrintStream out ) {
	    int i;

	    out.println( name + " {" );
	    for( i=0 ; i<eventIns.size() ; i++ ) {
		Event e = (Event)eventIns.elementAt(i);
		out.println( "  eventIn "+tokenImage[e.type]+" "+e.name );
	    }
	    for( i=0 ; i<eventOuts.size() ; i++ ) {
		Event e = (Event)eventOuts.elementAt(i);
		out.println( "  eventOut "+tokenImage[e.type]+" "+e.name );
	    }
	    for( i=0 ; i<fields.size() ; i++ ) {
	        Field f = (Field)fields.elementAt(i);
		out.print( "  field "+tokenImage[f.type]+" "+f.name );
		for( int j=0 ; j<f.init.length ; j++ )
		    out.print( " "+f.init[j] );
		out.println();
	    }
	    for( i=0 ; i<exposedFields.size() ; i++ ) {
		Field f = (Field)exposedFields.elementAt(i);
		out.print( "  exposedField "+tokenImage[f.type]+" "+f.name );
		for( int j=0 ; j<f.init.length ; j++ )
		    out.print( " "+f.init[j] );
		out.println();
	    }
	    out.println( "}" );
	}
    }

    public static class ParseException extends Exception {
	public int lineno;
	public ParseException( int lineno, String msg ) {
	    super(msg);
	    this.lineno = lineno;
	}
    }

    public NodeParser( LineNumberReader in ) {
	this.in = in;
    }
    public NodeParser( Reader in ) {
	this( new LineNumberReader(in) );
    }

    void expect( String exp, String fnd ) throws ParseException {
	if( !exp.equals( fnd ) )
	    throw new ParseException( in.getLineNumber(), "expected '"+exp+"', found '"+fnd+"'" );
    }
    void extraChars( StringTokenizer st, String after ) throws ParseException {
	if( st.hasMoreTokens() )
	    throw new ParseException( in.getLineNumber(), "extra characters after '"+after+"'" );
    }

    private int fieldCnt( int type ) {
	switch( type ) {
	case SFVEC2F:
	case MFVEC2F: return 2;
	case SFVEC3F:
	case MFVEC3F:
	case SFCOLOR:
	case MFCOLOR: return 3;
	case SFROTATION:
	case MFROTATION: return 4;
	default: return 1;
	}
    }

    private String fieldAppend( int type ) {
	switch(type) {
	case SFVEC2F:
	case SFVEC3F:
	case MFVEC2F:
	case MFVEC3F:
	case SFROTATION:
	case MFROTATION:
	case SFCOLOR:
	case MFCOLOR:
	case SFFLOAT:
	case MFFLOAT: return "f";
	default:      return "";
	}
    }
	    
    
    private String initString( StringTokenizer st, int cnt, String a ) {
	StringBuffer str = new StringBuffer();
	switch( cnt ) {
	case 4: str.append( st.nextToken() ).append(a).append( ", " );
	case 3: str.append( st.nextToken() ).append(a).append( ", " );
	case 2: str.append( st.nextToken() ).append(a).append( ", " );
	case 1: str.append( st.nextToken() ).append(a);
	}
	return new String(str);
    }

    public Node parseNode() throws ParseException, IOException {
	String line = in.readLine();
	if( line == null )
	    return null;

	StringTokenizer st = new StringTokenizer( line );
	
	String nodename = st.nextToken();
	expect( "{", st.nextToken() );
	extraChars( st, "{" );

	Node node = new Node();
	node.name = nodename;

	while( (line = in.readLine()) != null ) {
	    line = line.trim();
	    if( line.equals( "}" ) )
		break;

	    st = new StringTokenizer( line, " \t\n," );
	    String cat = st.nextToken();
	    String typestring = st.nextToken();
	    Integer typeint = (Integer)typeMap.get( typestring );
	    int type;

	    if( typeint != null ) {
		type = typeint.intValue();
	    } else {
		System.out.println( "Unknown type: "+typestring );
		type = -1;
	    }

	    String name = st.nextToken();


	    if( cat.endsWith("ield") ) {
		Field field = new Field();
		field.type = type;
		field.name = name;

		String init = st.nextToken("").trim();

		if( init.equals("[]") || init.equals("NULL") )
		    field.init = new String[0];
		else if( init.equals("\"\"") )
		    field.init = new String[] { "" };
		else if( init.equals( "FALSE" ) )
		    field.init = new String[] { "false" };
		else if( init.equals( "TRUE" ) )
		    field.init = new String[] { "true" };
		else {
		    int cnt = fieldCnt( type );
		    String a = fieldAppend(type);

		    if( init.length() > 0 && init.charAt(0) == '[' )
			init = init.substring( 1, init.length()-1 );

		    st = new StringTokenizer( init, ", \t\r\n" );
		    int num = st.countTokens() / cnt;

		    String inits[] = new String[num];

		    for( int i=0 ; i<num ; i++ )
			inits[i] = initString( st, cnt, a );

		    field.init = inits;
		}

		(cat.charAt(0) == 'e' ? node.exposedFields : node.fields).addElement( field );
	    } else {
		Event event = new Event();
		event.type = type;
		event.name = name;
		(cat.charAt(5) == 'I' ? node.eventIns : node.eventOuts).addElement( event );
	    }
	}

	return node;
    }

    public static void main( String args[] ) throws IOException, ParseException {
	NodeParser np = new NodeParser( new InputStreamReader( System.in ) );
	Node node;
	while( (node = np.parseNode()) != null ) {
	    // System.out.println( node.name );
	    // node.dump( System.out );
	    String filename = "..\\node\\"+node.name+".java";
	    System.out.println( "Writing \""+filename+"\"" );
	    PrintWriter pw = null;
	    try {
		pw = new PrintWriter(new FileWriter(filename));
		node.writeNode(pw);
	    } catch( IOException e ) {
		System.err.println( "IO Error: "+e.getMessage() );
	    } finally {
		if( pw != null )
		    pw.close();
	    }
	}
    }
}


